@@ -36,6 +36,7 @@ import os
 import yaml
 import gitlab
 from urllib.request import urlopen
+from urllib.parse import urlparse
 import hashlib
 import sys
 import json
@@ -64,8 +65,9 @@ eprint(f"Supported platforms: {job_mapping.keys()}")
 class Artifact(NamedTuple):
     job_name: str
     download_name: str
-    output_name: str
+    output_name: Optional[str]
     subdir: str
+    anchor_name: str
 
 # Platform spec provides a specification which is agnostic to Job
 # PlatformSpecs are converted into Artifacts by looking in the jobs-metadata.json file.
@@ -76,13 +78,15 @@ class PlatformSpec(NamedTuple):
 source_artifact = Artifact('source-tarball'
                           , 'ghc-{version}-src.tar.xz'
                           , 'ghc-{version}-src.tar.xz'
-                          , 'ghc-{version}' )
+                          , 'ghc-{version}'
+                          , 'ghc{version}-src')
 test_artifact = Artifact('source-tarball'
                         , 'ghc-{version}-testsuite.tar.xz'
                         , 'ghc-{version}-testsuite.tar.xz'
-                        , 'ghc-{version}' )
+                        , 'ghc-{version}/testsuite'
+                        , 'ghc{version}-testsuite')
 
-def debian(arch, n):
+def debian(n, arch='x86_64'):
     return linux_platform(arch, "{arch}-linux-deb{n}".format(arch=arch, n=n))
 
 def darwin(arch):
@@ -92,20 +96,17 @@ def darwin(arch):
 windowsArtifact = PlatformSpec ( 'x86_64-windows'
                                , 'ghc-{version}-x86_64-unknown-mingw32' )
 
-def centos(n):
-    return linux_platform("x86_64", "x86_64-linux-centos{n}".format(n=n))
+def fedora(n, arch='x86_64'):
+    return linux_platform(arch, "{arch}-linux-fedora{n}".format(n=n,arch=arch))
 
-def fedora(n):
-    return linux_platform("x86_64", "x86_64-linux-fedora{n}".format(n=n))
+def alpine(n, arch='x86_64'):
+    return linux_platform(arch, "{arch}-linux-alpine{n}".format(n=n,arch=arch))
 
-def alpine(n):
-    return linux_platform("x86_64", "x86_64-linux-alpine{n}".format(n=n))
+def rocky(n, arch='x86_64'):
+    return linux_platform(arch, "{arch}-linux-rocky{n}".format(n=n,arch=arch))
 
-def rocky(n):
-    return linux_platform("x86_64", "x86_64-linux-rocky{n}".format(n=n))
-
-def ubuntu(n):
-    return linux_platform("x86_64", "x86_64-linux-ubuntu{n}".format(n=n))
+def ubuntu(n, arch='x86_64'):
+    return linux_platform(arch, "{arch}-linux-ubuntu{n}".format(n=n,arch=arch))
 
 def linux_platform(arch, opsys):
     return PlatformSpec( opsys, 'ghc-{version}-{arch}-unknown-linux'.format(version="{version}", arch=arch) )
@@ -113,8 +114,7 @@ def linux_platform(arch, opsys):
 
 base_url = 'https://gitlab.haskell.org/api/v4/projects/1/jobs/{job_id}/artifacts/{artifact_name}'
 
-
-hash_cache = {}
+hash_cache = {} # type: Dict[str, str]
 
 # Download a URL and return its hash
 def download_and_hash(url):
@@ -132,6 +132,8 @@ def download_and_hash(url):
     hash_cache[url] = digest
     return digest
 
+uri_to_anchor_cache=dict()
+
 # Make the metadata for one platform.
 def mk_one_metadata(release_mode, version, job_map, artifact):
     job_id = job_map[artifact.job_name].id
@@ -156,48 +158,54 @@ def mk_one_metadata(release_mode, version, job_map, artifact):
     eprint(f"Bindist URL: {url}")
     eprint(f"Download URL: {final_url}")
 
-    #Download and hash from the release pipeline, this must not change anyway during upload.
+    # Download and hash from the release pipeline, this must not change anyway during upload.
     h = download_and_hash(url)
 
     res = { "dlUri": final_url
           , "dlSubdir": artifact.subdir.format(version=version)
-          , "dlOutput": artifact.output_name.format(version=version)
           , "dlHash" : h }
+
     eprint(res)
+
+    # add the uri to the anchor name cache so we can lookup an anchor for this uri
+    uri_to_anchor_cache[final_url] = artifact.anchor_name
     return res
 
 # Turns a platform into an Artifact respecting pipeline_type
 # Looks up the right job to use from the .gitlab/jobs-metadata.json file
-def mk_from_platform(pipeline_type, platform):
+def mk_from_platform(release_mode, pipeline_type, platform):
     info = job_mapping[platform.name][pipeline_type]
     eprint(f"From {platform.name} / {pipeline_type} selecting {info['name']}")
     return Artifact(info['name']
                    , f"{info['jobInfo']['bindistName']}.tar.xz"
                    , "ghc-{version}-{pn}.tar.xz".format(version="{version}", pn=platform.name)
-                   , platform.subdir)
-
+                   , platform.subdir
+                   , f"ghc{{version}}-{platform.name}")
 
 # Generate the new metadata for a specific GHC mode etc
-def mk_new_yaml(release_mode, version, date, pipeline_type, job_map):
+def mk_new_yaml(release_mode, version, date, pipeline_type, job_map) -> object:
     def mk(platform):
         eprint("\n=== " + platform.name + " " + ('=' * (75 - len(platform.name))))
-        return mk_one_metadata(release_mode, version, job_map, mk_from_platform(pipeline_type, platform))
+        return mk_one_metadata(release_mode, version, job_map, mk_from_platform(release_mode, pipeline_type, platform))
 
+    # Here are all the bindists we can distribute
     ubuntu1804 = mk(ubuntu("18_04"))
     ubuntu2004 = mk(ubuntu("20_04"))
     rocky8 = mk(rocky("8"))
-    # Here are all the bindists we can distribute
-    centos7 = mk(centos(7))
     fedora33 = mk(fedora(33))
     darwin_x86 = mk(darwin("x86_64"))
     darwin_arm64 = mk(darwin("aarch64"))
     windows = mk(windowsArtifact)
     alpine3_12 = mk(alpine("3_12"))
-    deb9 = mk(debian("x86_64", 9))
-    deb10 = mk(debian("x86_64", 10))
-    deb11 = mk(debian("x86_64", 11))
-    deb10_arm64 = mk(debian("aarch64", 10))
-    deb10_i386 = mk(debian("i386", 10))
+    alpine3_18 = mk(alpine("3_18"))
+    alpine3_18_arm64 = mk(alpine("3_18", arch='aarch64'))
+    deb9 = mk(debian(9, "x86_64"))
+    deb10 = mk(debian(10, "x86_64"))
+    deb11 = mk(debian(11, "x86_64"))
+    deb12 = mk(debian(12, "x86_64"))
+    deb10_arm64 = mk(debian(10, "aarch64"))
+    deb11_arm64 = mk(debian(11, "aarch64"))
+    deb10_i386 = mk(debian(10, "i386"))
 
     source = mk_one_metadata(release_mode, version, job_map, source_artifact)
     test = mk_one_metadata(release_mode, version, job_map, test_artifact)
@@ -206,24 +214,26 @@ def mk_new_yaml(release_mode, version, date, pipeline_type, job_map):
     # developers want.
 
     a64 = { "Linux_Debian": { "< 10": deb9
-                           , "(>= 10 && < 11)": deb10
-                           , ">= 11": deb11
+                           , "( >= 10 && < 11 )": deb10
+                           , "( >= 11 && < 12 )": deb11
+                           , ">= 12": deb12
                            , "unknown_versioning": deb11 }
           , "Linux_Ubuntu" : { "unknown_versioning": ubuntu2004
-                             , "( >= 16 && < 19 )": ubuntu1804
-                             }
+                             , "( >= 16 && < 18 )": deb9
+                             , "( >= 18 && < 19 )": ubuntu1804 }
           , "Linux_Mint"   : { "< 20": ubuntu1804
-                             , ">= 20": ubuntu2004 }
-          , "Linux_CentOS"  : { "( >= 7 && < 8 )" : centos7
-                              , "unknown_versioning" : centos7  }
-          , "Linux_Fedora"  : { ">= 33": fedora33
-                              , "unknown_versioning": centos7 }
-          , "Linux_RedHat"  : { "unknown_versioning": centos7 }
-          #MP: Replace here with Rocky8 when that job is in the pipeline
+                             , ">= 20": ubuntu2004
+                             , "unknown_versioning": ubuntu2004 }
+          , "Linux_Fedora"  : { ">= 33": fedora33 }
+          , "Linux_RedHat"  : { "< 9": rocky8
+                              , ">= 9": fedora33
+                              , "unknown_versioning": fedora33 }
           , "Linux_UnknownLinux" : { "unknown_versioning": rocky8 }
           , "Darwin" : { "unknown_versioning" : darwin_x86 }
           , "Windows" : { "unknown_versioning" :  windows }
-          , "Linux_Alpine" : { "unknown_versioning": alpine3_12 }
+          , "Linux_Alpine" : { "( >= 3.12 && < 3.18 )": alpine3_12
+                             , ">= 3.18": alpine3_18
+                             , "unknown_versioning": alpine3_12 }
 
           }
 
@@ -234,6 +244,11 @@ def mk_new_yaml(release_mode, version, date, pipeline_type, job_map):
           }
 
     arm64 = { "Linux_UnknownLinux": { "unknown_versioning": deb10_arm64 }
+            , "Linux_Alpine" : { "unknown_versioning": alpine3_18_arm64 }
+            , "Linux_Debian": { "( >= 10 && < 11 )": deb10_arm64
+                              , "( >= 11 && < 12 )": deb11_arm64
+                              , "unknown_versioning": deb10_arm64
+                              }
             , "Darwin": { "unknown_versioning": darwin_arm64 }
             }
 
@@ -273,6 +288,19 @@ def setNightlyTags(ghcup_metadata):
             ghcup_metadata['ghcupDownloads']['GHC'][version]["viTags"].append("Nightly")
 
 
+def mk_dumper(version):
+  class CustomAliasDumper(yaml.Dumper):
+      def __init__(self, *args, **kwargs):
+          super().__init__(*args, **kwargs)
+
+      def generate_anchor(self, node):
+          if isinstance(node, yaml.MappingNode):
+            node_dict = { k.value : v.value for (k,v) in node.value }
+            if 'dlUri' in node_dict:
+              return uri_to_anchor_cache[node_dict['dlUri']].format(version=version.replace('.',''))
+          return super().generate_anchor(node)
+
+  return CustomAliasDumper
 
 
 def main() -> None:
@@ -308,7 +336,7 @@ def main() -> None:
 
     new_yaml = mk_new_yaml(args.release_mode, args.version, args.date, pipeline_type, job_map)
     if args.fragment:
-        print(yaml.dump({ args.version : new_yaml }))
+        print(yaml.dump({ args.version : new_yaml }, Dumper=mk_dumper(args.version)))
 
     else:
         with open(args.metadata, 'r') as file:
