@@ -23,7 +23,6 @@ where
 import GHC.Prelude
 
 import Data.Array.IArray
-import Data.Foldable()
 import qualified Data.Tree as Tree
 
 import Data.Word
@@ -32,7 +31,6 @@ import qualified GHC.CmmToAsm.CFG.Dominators as LT
 
 import GHC.Cmm.Dataflow
 import GHC.Cmm.Dataflow.Block
-import GHC.Cmm.Dataflow.Collections
 import GHC.Cmm.Dataflow.Graph
 import GHC.Cmm.Dataflow.Label
 import GHC.Cmm
