@@ -104,12 +104,12 @@ worst regsOfClass regAlias neighbors classN classC
         regsC           = regsOfClass classC
 
         -- all the possible subsets of c which have size < m
-        regsS           = filter (\s -> sizeUniqSet s >= 1
+        regsS           = filter (\s -> not (isEmptyUniqSet s)
                                      && sizeUniqSet s <= neighbors)
                         $ powersetLS regsC
 
         -- for each of the subsets of C, the regs which conflict
-        -- with posiblities for N
+        -- with possibilities for N
         regsS_conflict
                 = map (\s -> intersectUniqSets regsN (regAliasS s)) regsS
 
