@@ -3,9 +3,7 @@
 (c) The GRASP/AQUA Project, Glasgow University, 1992-1998
 -}
 
-{-# LANGUAGE DeriveDataTypeable, FlexibleContexts #-}
-{-# LANGUAGE NamedFieldPuns #-}
-{-# LANGUAGE BangPatterns #-}
+{-# LANGUAGE NoPolyKinds #-}
 
 -- | GHC.Core holds all the main data types for use by for the Glasgow Haskell Compiler midsection
 module GHC.Core (
@@ -27,7 +25,7 @@ module GHC.Core (
         mkIntLit, mkIntLitWrap,
         mkWordLit, mkWordLitWrap,
         mkWord8Lit,
-        mkWord64LitWord64, mkInt64LitInt64,
+        mkWord32LitWord32, mkWord64LitWord64, mkInt64LitInt64,
         mkCharLit, mkStringLit,
         mkFloatLit, mkFloatLitFloat,
         mkDoubleLit, mkDoubleLitDouble,
@@ -35,6 +33,8 @@ module GHC.Core (
         mkConApp, mkConApp2, mkTyBind, mkCoBind,
         varToCoreExpr, varsToCoreExprs,
 
+        mkBinds,
+
         isId, cmpAltCon, cmpAlt, ltAlt,
 
         -- ** Simple 'Expr' access functions and predicates
@@ -112,7 +112,6 @@ import GHC.Utils.Binary
 import GHC.Utils.Misc
 import GHC.Utils.Outputable
 import GHC.Utils.Panic
-import GHC.Utils.Panic.Plain
 
 import Data.Data hiding (TyCon)
 import Data.Int
@@ -312,6 +311,17 @@ data Bind b = NonRec b (Expr b)
             | Rec [(b, (Expr b))]
   deriving Data
 
+-- | Helper function. You can use the result of 'mkBinds' with 'mkLets' for
+-- instance.
+--
+--   * @'mkBinds' 'Recursive' binds@ makes a single mutually-recursive
+--     bindings with all the rhs/lhs pairs in @binds@
+--   * @'mkBinds' 'NonRecursive' binds@ makes one non-recursive binding
+--     for each rhs/lhs pairs in @binds@
+mkBinds :: RecFlag -> [(b, (Expr b))] -> [Bind b]
+mkBinds Recursive binds = [Rec binds]
+mkBinds NonRecursive binds = map (uncurry NonRec) binds
+
 {-
 Note [Literal alternatives]
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -404,7 +414,7 @@ constructor worker or wrapper
       (This binding looks recursive, but isn't; it defines a top-level, curried
       function whose body just allocates and returns the data constructor.)
 
-      But if (a) the data contructor is nullary and (b) the data type is unlifted,
+      But if (a) the data constructor is nullary and (b) the data type is unlifted,
       this binding is unlifted.
       e.g.   data S :: UnliftedType where { S1 :: S, S2 :: S -> S }
       we generate
@@ -440,9 +450,6 @@ which will generate a @case@ if necessary
 
 The let-can-float invariant is initially enforced by mkCoreLet in GHC.Core.Make.
 
-For discussion of some implications of the let-can-float invariant primops see
-Note [Checking versus non-checking primops] in GHC.Builtin.PrimOps.
-
 Historical Note [The let/app invariant]
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 Before 2022 GHC used the "let/app invariant", which applied the let-can-float rules
@@ -464,7 +471,7 @@ TL;DR: we relaxed the let/app invariant to become the let-can-float invariant.
 Note [Core top-level string literals]
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 As an exception to the usual rule that top-level binders must be lifted,
-we allow binding primitive string literals (of type Addr#) of type Addr# at the
+we allow binding primitive string literals (of type Addr#) at the
 top level. This allows us to share string literals earlier in the pipeline and
 crucially allows other optimizations in the Core2Core pipeline to fire.
 Consider,
@@ -642,7 +649,7 @@ Note [Representation polymorphism invariants]
 GHC allows us to abstract over calling conventions using **representation polymorphism**.
 For example, we have:
 
-  ($) :: forall (r :: RuntimeRep) (a :: Type) (b :: TYPE r). a -> b -> b
+  ($) :: forall (r :: RuntimeRep) (a :: Type) (b :: TYPE r). (a -> b) -> a -> b
 
 In this example, the type `b` is representation-polymorphic: it has kind `TYPE r`,
 where the type variable `r :: RuntimeRep` abstracts over the runtime representation
@@ -657,13 +664,6 @@ The paper "Levity Polymorphism" [PLDI'17] states the first two invariants:
       (except for join points: See Note [Invariants on join points])
   I2. The type of a function argument must have a fixed runtime representation.
 
-On top of these two invariants, GHC's internal eta-expansion mechanism also requires:
-
-  I3. In any partial application `f e_1 .. e_n`, where `f` is `hasNoBinding`,
-      it must be the case that the application can be eta-expanded to match
-      the arity of `f`.
-      See Note [checkCanEtaExpand] in GHC.Core.Lint for more details.
-
 Example of I1:
 
   \(r::RuntimeRep). \(a::TYPE r). \(x::a). e
@@ -678,25 +678,25 @@ Example of I2:
     This contravenes I2: we are applying the function `f` to a value
     with an unknown runtime representation.
 
-Examples of I3:
+Note that these two invariants require us to check other types than just the
+types of bound variables and types of function arguments, due to transformations
+that GHC performs. For example, the definition
 
-  myUnsafeCoerce# :: forall {r1} (a :: TYPE r1) {r2} (b :: TYPE r2). a -> b
-  myUnsafeCoerce# = unsafeCoerce#
+  myCoerce :: forall {r} (a :: TYPE r) (b :: TYPE r). Coercible a b => a -> b
+  myCoerce = coerce
 
-    This contravenes I3: we are instantiating `unsafeCoerce#` without any
-    value arguments, and with a remaining argument type, `a`, which does not
-    have a fixed runtime representation.
-    But `unsafeCorce#` has no binding (see Note [Wiring in unsafeCoerce#]
-    in GHC.HsToCore).  So before code-generation we must saturate it
-    by eta-expansion (see GHC.CoreToStg.Prep.maybeSaturate), thus
-       myUnsafeCoerce# = \x. unsafeCoerce# x
-    But we can't do that because now the \x binding would violate I1.
+is invalid, because `coerce` has no binding (see GHC.Types.Id.Make.coerceId).
+So, before code-generation, GHC saturates the RHS of 'myCoerce' by performing
+an eta-expansion (see GHC.CoreToStg.Prep.maybeSaturate):
 
-  bar :: forall (a :: TYPE) r (b :: TYPE r). a -> b
-  bar = unsafeCoerce#
+  myCoerce = \ (x :: TYPE r) -> coerce x
 
-    OK: eta expand to `\ (x :: Type) -> unsafeCoerce# x`,
-    and `x` has a fixed RuntimeRep.
+However, this transformation would be invalid, because now the binding of x
+in the lambda abstraction would violate I1.
+
+See Note [Representation-polymorphism checking built-ins] in GHC.Tc.Gen.Head
+and Note [Linting representation-polymorphic builtins] in GHC.Core.Lint for
+more details.
 
 Note that we currently require something slightly stronger than a fixed runtime
 representation: we check whether bound variables and function arguments have a
@@ -748,9 +748,13 @@ this exhaustive list can be empty!
   its scrutinee is (see GHC.Core.Utils.exprIsTrivial).  This is actually
   important; see Note [Empty case is trivial] in GHC.Core.Utils
 
-* An empty case is replaced by its scrutinee during the CoreToStg
-  conversion; remember STG is un-typed, so there is no need for
-  the empty case to do the type conversion.
+* We lower empty cases in GHC.CoreToStg.coreToStgExpr to an eval on the
+  scrutinee.
+
+Historical Note: We used to lower EmptyCase in CorePrep by way of an
+unsafeCoercion on the scrutinee, but that yielded panics in CodeGen when
+we were beginning to eta expand in arguments, plus required to mess with
+heterogenously-kinded coercions. It's simpler to stick to it just a bit longer.
 
 Note [Join points]
 ~~~~~~~~~~~~~~~~~~
@@ -1897,6 +1901,9 @@ mkWordLitWrap platform w = Lit (mkLitWordWrap platform w)
 mkWord8Lit :: Integer -> Expr b
 mkWord8Lit    w = Lit (mkLitWord8 w)
 
+mkWord32LitWord32 :: Word32 -> Expr b
+mkWord32LitWord32 w = Lit (mkLitWord32 (toInteger w))
+
 mkWord64LitWord64 :: Word64 -> Expr b
 mkWord64LitWord64 w = Lit (mkLitWord64 (toInteger w))
 
