@@ -63,7 +63,6 @@ import Data.Array( Array, assocs )
 import GHC.Utils.Misc
 import GHC.Utils.Outputable
 import GHC.Utils.Panic
-import GHC.Utils.Panic.Plain
 
 import GHC.Types.Name.Set
 import GHC.Data.Bag
@@ -370,7 +369,7 @@ Nevertheless it is still useful to have data families in the FamInstEnv:
  - For finding overlaps and conflicts
 
  - For finding the representation type...see FamInstEnv.topNormaliseType
-   and its call site in GHC.Core.Opt.Simplify
+   and its call site in GHC.Core.Opt.Simplify.Iteration
 
  - In standalone deriving instance Eq (T [Int]) we need to find the
    representation type for T [Int]
