@@ -9,7 +9,7 @@ import GHC.Core.Opt.Pipeline.Types ( CoreToDo(..) )
 import GHC.Driver.Env
 import GHC.Driver.Session
 import GHC.Driver.Config.Core.Lint
-import GHC.Tc.Utils.Env
+import GHC.Driver.Config.Core.Opt.Arity
 import GHC.Types.Var
 import GHC.Utils.Outputable ( alwaysQualify )
 
@@ -17,14 +17,15 @@ import GHC.CoreToStg.Prep
 
 initCorePrepConfig :: HscEnv -> IO CorePrepConfig
 initCorePrepConfig hsc_env = do
-   convertNumLit <- do
-     let platform = targetPlatform $ hsc_dflags hsc_env
-         home_unit = hsc_home_unit hsc_env
-         lookup_global = lookupGlobal hsc_env
-     mkConvertNumLiteral platform home_unit lookup_global
+   let dflags = hsc_dflags hsc_env
    return $ CorePrepConfig
-      { cp_catchNonexhaustiveCases = gopt Opt_CatchNonexhaustiveCases $ hsc_dflags hsc_env
-      , cp_convertNumLit = convertNumLit
+      { cp_catchNonexhaustiveCases = gopt Opt_CatchNonexhaustiveCases dflags
+      , cp_platform  = targetPlatform dflags
+      , cp_arityOpts = if gopt Opt_DoCleverArgEtaExpansion dflags
+                       then Just (initArityOpts dflags)
+                       else Nothing
+      , cp_specEval  = gopt Opt_SpecEval dflags
+      , cp_specEvalDFun = gopt Opt_SpecEvalDictFun dflags
       }
 
 initCorePrepPgmConfig :: DynFlags -> [Var] -> CorePrepPgmConfig
