@@ -4,10 +4,8 @@
 {-# OPTIONS_GHC -fspec-constr-threshold=10000 #-}
 
 {-# LANGUAGE CPP #-}
-{-# LANGUAGE TupleSections, RecordWildCards #-}
-{-# LANGUAGE BangPatterns #-}
+{-# LANGUAGE RecordWildCards #-}
 {-# LANGUAGE LambdaCase #-}
-{-# LANGUAGE RankNTypes #-}
 
 module GHC.Linker.Deps
   ( LinkDepsOpts (..)
@@ -31,7 +29,6 @@ import GHC.Types.Unique.DFM
 
 import GHC.Utils.Outputable
 import GHC.Utils.Panic
-import GHC.Utils.Panic.Plain
 import GHC.Utils.Error
 
 import GHC.Unit.Env
@@ -56,7 +53,6 @@ import Control.Applicative
 import qualified Data.Set as Set
 import qualified Data.Map as M
 import Data.List (isSuffixOf)
-import Data.Either
 
 import System.FilePath
 import System.Directory
@@ -132,7 +128,7 @@ get_link_deps opts pls maybe_normal_osuf span mods = do
       let
         -- 2.  Exclude ones already linked
         --      Main reason: avoid findModule calls in get_linkable
-            (mods_needed, links_got) = partitionEithers (map split_mods mods_s)
+            (mods_needed, links_got) = partitionWith split_mods mods_s
             pkgs_needed = eltsUDFM $ getUniqDSet pkgs_s `minusUDFM` pkgs_loaded pls
 
             split_mods mod =
@@ -323,9 +319,8 @@ get_link_deps opts pls maybe_normal_osuf span mods = do
 {-
 Note [Using Byte Code rather than Object Code for Template Haskell]
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
 The `-fprefer-byte-code` flag allows a user to specify that they want to use
-byte code (if availble) rather than object code for home module dependenices
+byte code (if available) rather than object code for home module dependencies
 when executing Template Haskell splices.
 
 Why might you want to use byte code rather than object code?
@@ -336,7 +331,7 @@ Why might you want to use byte code rather than object code?
 So we allow the user to choose to use byte code rather than object files if they want to avoid these
 two pitfalls.
 
-When using `-fprefer-byte-code` you have to arrange to have the byte code availble.
+When using `-fprefer-byte-code` you have to arrange to have the byte code available.
 In normal --make mode it will not be produced unless you enable `-fbyte-code-and-object-code`.
 See Note [Home module build products] for some more information about that.
 
