@@ -249,8 +249,8 @@ data PsMessage
    -- | If-then-else syntax in pattern
    | PsErrIfThenElseInPat
 
-   -- | Lambda-case in pattern
-   | PsErrLambdaCaseInPat LamCaseVariant
+   -- | Lambda or Lambda-case in pattern
+   | PsErrLambdaInPat HsLamVariant
 
    -- | case..of in pattern
    | PsErrCaseInPat
@@ -258,9 +258,6 @@ data PsMessage
    -- | let-syntax in pattern
    | PsErrLetInPat
 
-   -- | Lambda-syntax in pattern
-   | PsErrLambdaInPat
-
    -- | Arrow expression-syntax in pattern
    | PsErrArrowExprInPat !(HsExpr GhcPs)
 
@@ -310,14 +307,11 @@ data PsMessage
    -- | @-operator in a pattern position
    | PsErrAtInPatPos
 
-   -- | Unexpected lambda command in function application
-   | PsErrLambdaCmdInFunAppCmd !(LHsCmd GhcPs)
-
    -- | Unexpected case command in function application
    | PsErrCaseCmdInFunAppCmd !(LHsCmd GhcPs)
 
-   -- | Unexpected \case(s) command in function application
-   | PsErrLambdaCaseCmdInFunAppCmd !LamCaseVariant !(LHsCmd GhcPs)
+   -- | Unexpected lambda or \case(s) command in function application
+   | PsErrLambdaCmdInFunAppCmd !HsLamVariant !(LHsCmd GhcPs)
 
    -- | Unexpected if command in function application
    | PsErrIfCmdInFunAppCmd !(LHsCmd GhcPs)
@@ -334,14 +328,11 @@ data PsMessage
    -- | Unexpected mdo block in function application
    | PsErrMDoInFunAppExpr !(Maybe ModuleName) !(LHsExpr GhcPs)
 
-   -- | Unexpected lambda expression in function application
-   | PsErrLambdaInFunAppExpr !(LHsExpr GhcPs)
-
    -- | Unexpected case expression in function application
    | PsErrCaseInFunAppExpr !(LHsExpr GhcPs)
 
-   -- | Unexpected \case(s) expression in function application
-   | PsErrLambdaCaseInFunAppExpr !LamCaseVariant !(LHsExpr GhcPs)
+   -- | Unexpected lambda or \case(s) expression in function application
+   | PsErrLambdaInFunAppExpr !HsLamVariant !(LHsExpr GhcPs)
 
    -- | Unexpected let expression in function application
    | PsErrLetInFunAppExpr !(LHsExpr GhcPs)
@@ -398,7 +389,7 @@ data PsMessage
    | PsErrIllegalRoleName !FastString [Role]
 
    -- | Invalid type signature
-   | PsErrInvalidTypeSignature !(LHsExpr GhcPs)
+   | PsErrInvalidTypeSignature !PsInvalidTypeSignature !(LHsExpr GhcPs)
 
    -- | Unexpected type in declaration
    | PsErrUnexpectedTypeInDecl !(LHsType GhcPs)
@@ -468,6 +459,8 @@ data PsMessage
       Char -- ^ the character it looks like
       String -- ^ the name of the character that it looks like
 
+   | PsErrInvalidPun !PsErrPunDetails
+
    deriving Generic
 
 -- | Extra details about a parse error, which helps
@@ -487,6 +480,11 @@ data PsErrParseDetails
     -- ^ Did we parse a \"pattern\" keyword?
   }
 
+data PsInvalidTypeSignature
+  = PsErrInvalidTypeSig_Qualified
+  | PsErrInvalidTypeSig_DataCon
+  | PsErrInvalidTypeSig_Other
+
 -- | Is the parsed pattern recursive?
 data PatIsRecursive
   = YesPatIsRecursive
@@ -518,6 +516,11 @@ data PsErrInPatDetails
                     !ParseContext
   | PEIP_OtherPatDetails !ParseContext
 
+data PsErrPunDetails
+  = PEP_QuoteDisambiguation
+  | PEP_TupleSyntaxType
+  | PEP_SumSyntaxType
+
 noParseContext :: ParseContext
 noParseContext = ParseContext Nothing NoIncompleteDoBlock
 
@@ -533,6 +536,7 @@ data NumUnderscoreReason
    | NumUnderscore_Float
    deriving (Show,Eq,Ord)
 
+
 data LexErrKind
    = LexErrKind_EOF        -- ^ End of input
    | LexErrKind_UTF8       -- ^ UTF-8 decoding error
