@@ -4,7 +4,6 @@ module GHC.Platform.Wasm32 where
 
 import GHC.Prelude
 
--- TODO
-#define MACHREGS_NO_REGS 1
--- #define MACHREGS_wasm32 1
+#define MACHREGS_NO_REGS 0
+#define MACHREGS_wasm32 1
 #include "CodeGen.Platform.h"
