@@ -8,6 +8,7 @@ module GHC.Runtime.Context
    , substInteractiveContext
    , replaceImportEnv
    , icReaderEnv
+   , icExtendGblRdrEnv
    , icInteractiveModule
    , icInScopeTTs
    , icNamePprCtx
@@ -18,7 +19,7 @@ import GHC.Prelude
 
 import GHC.Hs
 
-import GHC.Driver.Session
+import GHC.Driver.DynFlags
 import {-# SOURCE #-} GHC.Driver.Plugins
 
 import GHC.Runtime.Eval.Types ( IcGlobalRdrEnv(..), Resume )
@@ -30,7 +31,6 @@ import GHC.Core.FamInstEnv
 import GHC.Core.InstEnv
 import GHC.Core.Type
 
-import GHC.Types.Avail
 import GHC.Types.Fixity.Env
 import GHC.Types.Id.Info ( IdDetails(..) )
 import GHC.Types.Name
@@ -94,7 +94,7 @@ The details are a bit tricky though:
    call to initTc in initTcInteractive, which in turn get the module
    from it 'icInteractiveModule' field of the interactive context.
 
-   The 'homeUnitId' field stays as 'main' (or whatever -this-unit-id says.
+   The 'homeUnitId' field stays as 'main' (or whatever -this-unit-id says).
 
  * The main trickiness is that the type environment (tcg_type_env) and
    fixity envt (tcg_fix_env), now contain entities from all the
@@ -185,9 +185,12 @@ It's exactly the same for type-family instances.  See #7102
 Note [icReaderEnv recalculation]
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 The GlobalRdrEnv describing what’s in scope at the prompts consists
-of all the imported things, followed by all the things defined on the prompt, with
-shadowing. Defining new things on the prompt is easy: we shadow as needed and then extend the environment.  But changing the set of imports, which can happen later as well,
-is tricky: we need to re-apply the shadowing from all the things defined at the prompt!
+of all the imported things, followed by all the things defined on the prompt,
+with shadowing. Defining new things on the prompt is easy: we shadow as needed,
+and then extend the environment.
+
+But changing the set of imports, which can happen later as well, is tricky
+we need to re-apply the shadowing from all the things defined at the prompt!
 
 For example:
 
@@ -196,22 +199,21 @@ For example:
     ghci> empty   -- Still gets the 'empty' defined at the prompt
     True
 
-
-It would be correct ot re-construct the env from scratch based on
+It would be correct to re-construct the env from scratch based on
 `ic_tythings`, but that'd be quite expensive if there are many entries in
 `ic_tythings` that shadow each other.
 
-Therefore we keep around a that `GlobalRdrEnv` in `igre_prompt_env` that
-contians _just_ the things defined at the prompt, and use that in
-`replaceImportEnv` to rebuild the full env.  Conveniently, `shadowNames` takes
-such an `OccEnv` to denote the set of names to shadow.
+Therefore we keep around a `GlobalRdrEnv` in `igre_prompt_env` that contains
+_just_ the things defined at the prompt, and use that in `replaceImportEnv` to
+rebuild the full env.  Conveniently, `shadowNames` takes such an `OccEnv`
+to denote the set of names to shadow.
 
 INVARIANT: Every `OccName` in `igre_prompt_env` is present unqualified as well
-(else it would not be right to use pass `igre_prompt_env` to `shadowNames`.)
+(else it would not be right to pass `igre_prompt_env` to `shadowNames`.)
+
+The definition of the IcGlobalRdrEnv type should conceptually be in this module,
+and made abstract, but it’s used in `Resume`, so it lives in GHC.Runtime.Eval.Type.
 
-The definition of the IcGlobalRdrEnv type should conceptually be in this module, and
-made abstract, but it’s used in `Resume`, so it lives in GHC.Runtime.Eval.Type.
--
 -}
 
 -- | Interactive context, recording information about the state of the
@@ -328,7 +330,7 @@ icReaderEnv = igre_env . ic_gre_cache
 
 icInteractiveModule :: InteractiveContext -> Module
 icInteractiveModule (InteractiveContext { ic_mod_index = index })
-  = mkInteractiveModule index
+  = mkInteractiveModule (show index)
 
 -- | This function returns the list of visible TyThings (useful for
 -- e.g. showBindings).
@@ -343,12 +345,11 @@ icInScopeTTs ictxt = filter in_scope_unqualified (ic_tythings ictxt)
   where
     in_scope_unqualified thing = or
         [ unQualOK gre
-        | avail <- tyThingAvailInfo thing
-        , name <- availNames avail
+        | gre <- tyThingLocalGREs thing
+        , let name = greName gre
         , Just gre <- [lookupGRE_Name (icReaderEnv ictxt) name]
         ]
 
-
 -- | Get the NamePprCtx function based on the flags and this InteractiveContext
 icNamePprCtx :: UnitEnv -> InteractiveContext -> NamePprCtx
 icNamePprCtx unit_env ictxt = mkNamePprCtx ptc unit_env (icReaderEnv ictxt)
@@ -400,8 +401,11 @@ setInteractivePrintName ic n = ic{ic_int_print = n}
 
 icExtendIcGblRdrEnv :: IcGlobalRdrEnv -> [TyThing] -> IcGlobalRdrEnv
 icExtendIcGblRdrEnv igre tythings = IcGlobalRdrEnv
-    { igre_env = igre_env igre `icExtendGblRdrEnv` tythings
-    , igre_prompt_env = igre_prompt_env igre `icExtendGblRdrEnv` tythings
+    { igre_env        = icExtendGblRdrEnv False (igre_env igre)        tythings
+    , igre_prompt_env = icExtendGblRdrEnv True  (igre_prompt_env igre) tythings
+        -- Pass 'True' <=> drop names that are only available qualified.
+        -- This is done to maintain the invariant of Note [icReaderEnv recalculation]
+        -- that igre_prompt_env should only contain Names that are available unqualified.
     }
 
 -- This is used by setContext in GHC.Runtime.Eval when the set of imports
@@ -409,13 +413,14 @@ icExtendIcGblRdrEnv igre tythings = IcGlobalRdrEnv
 replaceImportEnv :: IcGlobalRdrEnv -> GlobalRdrEnv -> IcGlobalRdrEnv
 replaceImportEnv igre import_env = igre { igre_env = new_env }
   where
-    import_env_shadowed = import_env `shadowNames` igre_prompt_env igre
+    import_env_shadowed = shadowNames False import_env (igre_prompt_env igre)
     new_env = import_env_shadowed `plusGlobalRdrEnv` igre_prompt_env igre
 
--- | Add TyThings to the GlobalRdrEnv, earlier ones in the list shadowing
--- later ones, and shadowing existing entries in the GlobalRdrEnv.
-icExtendGblRdrEnv :: GlobalRdrEnv -> [TyThing] -> GlobalRdrEnv
-icExtendGblRdrEnv env tythings
+-- | Add 'TyThings' to the 'GlobalRdrEnv', earlier ones in the list shadowing
+-- later ones, and shadowing existing entries in the 'GlobalRdrEnv'.
+icExtendGblRdrEnv :: Bool -- ^ discard names that are only available qualified?
+                  -> GlobalRdrEnv -> [TyThing] -> GlobalRdrEnv
+icExtendGblRdrEnv drop_only_qualified env tythings
   = foldr add env tythings  -- Foldr makes things in the front of
                             -- the list shadow things at the back
   where
@@ -424,12 +429,10 @@ icExtendGblRdrEnv env tythings
        | is_sub_bndr thing
        = env
        | otherwise
-       = foldl' extendGlobalRdrEnv env1 (concatMap localGREsFromAvail avail)
+       = foldl' extendGlobalRdrEnv env1 new_gres
        where
-          new_gres = concatMap availGreNames avail
-          new_occs = occSetToEnv (mkOccSet (map occName new_gres))
-          env1  = shadowNames env new_occs
-          avail = tyThingAvailInfo thing
+          new_gres = tyThingLocalGREs thing
+          env1     = shadowNames drop_only_qualified env $ mkGlobalRdrEnv new_gres
 
     -- Ugh! The new_tythings may include record selectors, since they
     -- are not implicit-ids, and must appear in the TypeEnv.  But they
