@@ -73,10 +73,10 @@ import GHC.Cmm
 import GHC.StgToCmm.Config
 import GHC.StgToCmm.Closure
 import GHC.StgToCmm.Sequel
-import GHC.Cmm.Dataflow.Collections
 import GHC.Cmm.Graph as CmmGraph
 import GHC.Cmm.BlockId
 import GHC.Cmm.CLabel
+import GHC.Cmm.Dataflow.Label
 import GHC.Runtime.Heap.Layout
 import GHC.Unit
 import GHC.Types.Id
