@@ -26,7 +26,7 @@ import GHC.StgToJS.Object
 import GHC.StgToJS.Types
 import GHC.StgToJS.Ids
 
-import GHC.JS.Unsat.Syntax
+import GHC.JS.Ident
 
 import GHC.Types.Id
 import GHC.Types.Unique
@@ -48,7 +48,6 @@ import qualified Data.IntSet as IS
 import qualified GHC.Data.Word64Map as WM
 import GHC.Data.Word64Map (Word64Map)
 import Data.Array
-import Data.Either
 import Data.Word
 import Control.Monad
 
@@ -101,9 +100,9 @@ genDependencyData mod units = do
              -> Int
              -> StateT DependencyDataCache G (Int, BlockDeps, Bool, [ExportedFun])
       oneDep (LinkableUnit _ idExports otherExports idDeps pseudoIdDeps otherDeps req _frefs) n = do
-        (edi, bdi) <- partitionEithers <$> mapM (lookupIdFun n) idDeps
-        (edo, bdo) <- partitionEithers <$> mapM lookupOtherFun otherDeps
-        (edp, bdp) <- partitionEithers <$> mapM (lookupPseudoIdFun n) pseudoIdDeps
+        (edi, bdi) <- partitionWithM (lookupIdFun n) idDeps
+        (edo, bdo) <- partitionWithM lookupOtherFun otherDeps
+        (edp, bdp) <- partitionWithM (lookupPseudoIdFun n) pseudoIdDeps
         expi <- mapM lookupExportedId (filter isExportedId idExports)
         expo <- mapM lookupExportedOther otherExports
         -- fixme thin deps, remove all transitive dependencies!
@@ -140,7 +139,7 @@ genDependencyData mod units = do
             let k = getKey . getUnique $ i
                 addEntry :: StateT DependencyDataCache G ExportedFun
                 addEntry = do
-                  (TxtI idTxt) <- lift (identForId i)
+                  idTxt <- identFS <$> lift (identForId i)
                   lookupExternalFun (Just k) (OtherSymb m idTxt)
             in  if m == mod
                    then pprPanic "local id not found" (ppr m)
@@ -160,7 +159,7 @@ genDependencyData mod units = do
 
       lookupExportedId :: Id -> StateT DependencyDataCache G ExportedFun
       lookupExportedId i = do
-        (TxtI idTxt) <- lift (identForId i)
+        idTxt <- identFS <$> lift (identForId i)
         lookupExternalFun (Just . getKey . getUnique $ i) (OtherSymb mod idTxt)
 
       lookupExportedOther :: FastString -> StateT DependencyDataCache G ExportedFun
