@@ -1,5 +1,6 @@
 {-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE ViewPatterns      #-}
 
 module GHC.StgToJS.Literal
   ( genLit
@@ -9,8 +10,9 @@ where
 
 import GHC.Prelude
 
-import GHC.JS.Unsat.Syntax
+import GHC.JS.JStg.Syntax
 import GHC.JS.Make
+import GHC.JS.Ident
 
 import GHC.StgToJS.Types
 import GHC.StgToJS.Monad
@@ -35,15 +37,15 @@ import Data.Char (ord)
 --  * Addr# (Null and Strings): array and offset
 --  * 64-bit values: high 32-bit, low 32-bit
 --  * labels: call to h$mkFunctionPtr and 0, or function name and 0
-genLit :: HasDebugCallStack => Literal -> G [JExpr]
+genLit :: HasDebugCallStack => Literal -> G [JStgExpr]
 genLit = \case
   LitChar c     -> return [ toJExpr (ord c) ]
   LitString str ->
-    freshIdent >>= \strLit@(TxtI strLitT) ->
-      freshIdent >>= \strOff@(TxtI strOffT) -> do
+    freshIdent >>= \strLit@(identFS -> strLitT) ->
+      freshIdent >>= \strOff@(identFS -> strOffT) -> do
         emitStatic strLitT (StaticUnboxed (StaticUnboxedString str)) Nothing
         emitStatic strOffT (StaticUnboxed (StaticUnboxedStringOffset str)) Nothing
-        return [ ValExpr (JVar strLit), ValExpr (JVar strOff) ]
+        return [ Var strLit, Var strOff ]
   LitNullAddr              -> return [ null_, ValExpr (JInt 0) ]
   LitNumber nt v           -> case nt of
     LitNumInt     -> return [ toJExpr v ]
@@ -64,7 +66,7 @@ genLit = \case
                                                   [var (mkRawSymbol True name)]
                                        , ValExpr (JInt 0)
                                        ]
-    | otherwise              -> return [ toJExpr (TxtI (mkRawSymbol True name))
+    | otherwise              -> return [ toJExpr (global (mkRawSymbol True name))
                                        , ValExpr (JInt 0)
                                        ]
   LitRubbish _ rr_ty ->
@@ -86,7 +88,6 @@ genLit = \case
                   Int64Rep   -> [ ValExpr (JInt 0), ValExpr (JInt 0) ]
                   DoubleRep  -> [ ValExpr (JInt 0) ]
                   FloatRep   -> [ ValExpr (JInt 0) ]
-                  VoidRep    -> panic "GHC.StgToJS.Literal.genLit: LitRubbish(VoidRep)"
                   VecRep _ _ -> panic "GHC.StgToJS.Literal.genLit: VecRep unsupported"
     in return (concatMap rub reps)
 
@@ -131,11 +132,10 @@ genStaticLit = \case
         Word64Rep   -> pure [ IntLit 0, IntLit 0 ]
         FloatRep    -> pure [ DoubleLit (SaneDouble 0) ]
         DoubleRep   -> pure [ DoubleLit (SaneDouble 0) ]
-        VoidRep     -> panic "GHC.StgToJS.Literal.getStaticLit: LitRubbish(VoidRep)"
         VecRep {}   -> pprPanic "GHC.StgToJS.Literal.genStaticLit: LitRubbish(VecRep) isn't supported" (ppr rep)
 
 -- make an unsigned 32 bit number from this unsigned one, lower 32 bits
-toU32Expr :: Integer -> JExpr
+toU32Expr :: Integer -> JStgExpr
 toU32Expr i = Int (i Bits..&. 0xFFFFFFFF) .>>>. 0
 
 -- make an unsigned 32 bit number from this unsigned one, lower 32 bits
