@@ -10,19 +10,18 @@ import GHC.Linker.Types (SptEntry(..))
 import GHC.Fingerprint.Type
 import GHC.Types.Literal
 
-import GHC.JS.Unsat.Syntax
+import GHC.JS.JStg.Syntax
 import GHC.JS.Make
 
 import GHC.StgToJS.Types
 import GHC.StgToJS.Literal
 import GHC.StgToJS.Ids
 
-initStaticPtrs :: [SptEntry] -> G JStat
+initStaticPtrs :: [SptEntry] -> G JStgStat
 initStaticPtrs ptrs = mconcat <$> mapM initStatic ptrs
   where
     initStatic (SptEntry sp_id (Fingerprint w1 w2)) = do
       i <- varForId sp_id
       fpa <- concat <$> mapM (genLit . mkLitWord64 . fromIntegral) [w1,w2]
-      let sptInsert = ApplExpr (var "h$hs_spt_insert") (fpa ++ [i])
-      return $ (var "h$initStatic" .^ "push") `ApplStat` [jLam sptInsert]
-
+      let sptInsert = ApplStat (var "h$hs_spt_insert") (fpa ++ [i])
+      return $ (var "h$initStatic" .^ "push") `ApplStat` [Func [] sptInsert]
