@@ -54,7 +54,7 @@ import GHC.Tc.Types.TcRef
 import GHC.Tc.TyCl.Build ( TcMethInfo, MethInfo )
 import GHC.Tc.Utils.Env ( tcLookupGlobalOnly )
 import GHC.Tc.Utils.TcType
-import GHC.Tc.Utils.Monad ( setSrcSpanA, liftZonkM, traceTc, addErr )
+import GHC.Tc.Utils.Monad ( newZonkAnyType, setSrcSpanA, liftZonkM, traceTc, addErr )
 import GHC.Tc.Types.Constraint
 import GHC.Tc.Types.Evidence
 import GHC.Tc.Errors.Types
@@ -75,7 +75,6 @@ import GHC.Utils.Outputable
 import GHC.Utils.Misc
 import GHC.Utils.Monad
 import GHC.Utils.Panic
-import GHC.Utils.Panic.Plain
 
 import GHC.Core.Multiplicity
 import GHC.Core
@@ -330,15 +329,15 @@ type ZonkTcM = ZonkT TcM
 -- See Note [Zonking to Type].
 type ZonkBndrTcM = ZonkBndrT TcM
 
-wrapLocZonkMA :: (a -> ZonkTcM b) -> GenLocated (SrcSpanAnn' ann) a
-              -> ZonkTcM (GenLocated (SrcSpanAnn' ann) b)
+wrapLocZonkMA :: (a -> ZonkTcM b) -> GenLocated (EpAnn ann) a
+              -> ZonkTcM (GenLocated (EpAnn ann) b)
 wrapLocZonkMA fn (L loc a) = ZonkT $ \ ze ->
   setSrcSpanA loc $
   do { b <- runZonkT (fn a) ze
      ; return (L loc b) }
 
-wrapLocZonkBndrMA :: (a -> ZonkBndrTcM b) -> GenLocated (SrcSpanAnn' ann) a
-                  -> ZonkBndrTcM (GenLocated (SrcSpanAnn' ann) b)
+wrapLocZonkBndrMA :: (a -> ZonkBndrTcM b) -> GenLocated (EpAnn ann) a
+                  -> ZonkBndrTcM (GenLocated (EpAnn ann) b)
 wrapLocZonkBndrMA fn (L loc a) = ZonkBndrT $ \ k -> ZonkT $ \ ze ->
   setSrcSpanA loc $
   runZonkT ( runZonkBndrT (fn a) $ \ b -> k (L loc b) ) ze
@@ -469,8 +468,9 @@ commitFlexi tv zonked_kind
            -> do { addErr $ TcRnZonkerMessage (ZonkerCannotDefaultConcrete origin)
                  ; return (anyTypeOfKind zonked_kind) }
            | otherwise
-           -> do { traceTc "Defaulting flexi tyvar to Any:" (pprTyVar tv)
-                 ; return (anyTypeOfKind zonked_kind) }
+           -> do { traceTc "Defaulting flexi tyvar to ZonkAny:" (pprTyVar tv)
+                   -- See Note [Any types] in GHC.Builtin.Types, esp wrinkle (Any4)
+                 ; newZonkAnyType zonked_kind }
 
          RuntimeUnkFlexi
            -> do { traceTc "Defaulting flexi tyvar to RuntimeUnk:" (pprTyVar tv)
@@ -719,11 +719,14 @@ zonk_lbind = wrapLocZonkMA zonk_bind
 
 zonk_bind :: HsBind GhcTc -> ZonkTcM (HsBind GhcTc)
 zonk_bind bind@(PatBind { pat_lhs = pat, pat_rhs = grhss
+                        , pat_mult = mult_ann
                         , pat_ext = (ty, ticks)})
   = do  { new_pat   <- don'tBind $ zonkPat pat            -- Env already extended
         ; new_grhss <- zonkGRHSs zonkLExpr grhss
         ; new_ty    <- zonkTcTypeToTypeX ty
+        ; new_mult  <- zonkMultAnn mult_ann
         ; return (bind { pat_lhs = new_pat, pat_rhs = new_grhss
+                       , pat_mult = new_mult
                        , pat_ext = (new_ty, ticks) }) }
 
 zonk_bind (VarBind { var_ext = x
@@ -810,6 +813,17 @@ zonk_bind (PatSynBind x bind@(PSB { psb_id   = L loc id
                        , psb_def  = lpat'
                        , psb_dir  = dir' } } }
 
+zonkMultAnn :: HsMultAnn GhcTc -> ZonkTcM (HsMultAnn GhcTc)
+zonkMultAnn (HsNoMultAnn mult)
+  = do { mult' <- zonkTcTypeToTypeX mult
+       ; return (HsNoMultAnn mult') }
+zonkMultAnn (HsPct1Ann mult)
+  = do { mult' <- zonkTcTypeToTypeX mult
+       ; return (HsPct1Ann mult') }
+zonkMultAnn (HsMultAnn mult hs_ty)
+  = do { mult' <- zonkTcTypeToTypeX mult
+       ; return (HsMultAnn mult' hs_ty) }
+
 zonkPatSynDetails :: HsPatSynDetails GhcTc
                   -> ZonkTcM (HsPatSynDetails GhcTc)
 zonkPatSynDetails (PrefixCon _ as)
@@ -851,7 +865,7 @@ zonkLTcSpecPrags ps
 ************************************************************************
 -}
 
-zonkMatchGroup :: Anno (GRHS GhcTc (LocatedA (body GhcTc))) ~ SrcAnn NoEpAnns
+zonkMatchGroup :: Anno (GRHS GhcTc (LocatedA (body GhcTc))) ~ EpAnnCO
                => (LocatedA (body GhcTc) -> ZonkTcM (LocatedA (body GhcTc)))
                -> MatchGroup GhcTc (LocatedA (body GhcTc))
                -> ZonkTcM (MatchGroup GhcTc (LocatedA (body GhcTc)))
@@ -865,7 +879,7 @@ zonkMatchGroup zBody (MG { mg_alts = L l ms
                      , mg_ext = MatchGroupTc arg_tys' res_ty' origin
                      }) }
 
-zonkMatch :: Anno (GRHS GhcTc (LocatedA (body GhcTc))) ~ SrcAnn NoEpAnns
+zonkMatch :: Anno (GRHS GhcTc (LocatedA (body GhcTc))) ~ EpAnnCO
           => (LocatedA (body GhcTc) -> ZonkTcM (LocatedA (body GhcTc)))
           -> LMatch GhcTc (LocatedA (body GhcTc))
           -> ZonkTcM (LMatch GhcTc (LocatedA (body GhcTc)))
@@ -876,7 +890,7 @@ zonkMatch zBody (L loc match@(Match { m_pats = pats
       ; return (L loc (match { m_pats = new_pats, m_grhss = new_grhss })) }
 
 -------------------------------------------------------------------------
-zonkGRHSs :: Anno (GRHS GhcTc (LocatedA (body GhcTc))) ~ SrcAnn NoEpAnns
+zonkGRHSs :: Anno (GRHS GhcTc (LocatedA (body GhcTc))) ~ EpAnnCO
           => (LocatedA (body GhcTc) -> ZonkTcM (LocatedA (body GhcTc)))
           -> GRHSs GhcTc (LocatedA (body GhcTc))
           -> ZonkTcM (GRHSs GhcTc (LocatedA (body GhcTc)))
@@ -939,23 +953,19 @@ zonkExpr (HsOverLit x lit)
   = do  { lit' <- zonkOverLit lit
         ; return (HsOverLit x lit') }
 
-zonkExpr (HsLam x matches)
+zonkExpr (HsLam x lam_variant matches)
   = do new_matches <- zonkMatchGroup zonkLExpr matches
-       return (HsLam x new_matches)
-
-zonkExpr (HsLamCase x lc_variant matches)
-  = do new_matches <- zonkMatchGroup zonkLExpr matches
-       return (HsLamCase x lc_variant new_matches)
+       return (HsLam x lam_variant new_matches)
 
 zonkExpr (HsApp x e1 e2)
   = do new_e1 <- zonkLExpr e1
        new_e2 <- zonkLExpr e2
        return (HsApp x new_e1 new_e2)
 
-zonkExpr (HsAppType ty e at t)
+zonkExpr (HsAppType ty e t)
   = do new_e <- zonkLExpr e
        new_ty <- zonkTcTypeToTypeX ty
-       return (HsAppType new_ty new_e at t)
+       return (HsAppType new_ty new_e t)
        -- NB: the type is an HsType; can't zonk that!
 
 zonkExpr (HsTypedBracket hsb_tc body)
@@ -975,9 +985,9 @@ zonkExpr (NegApp x expr op)
     do { new_expr <- zonkLExpr expr
        ; return (NegApp x new_expr new_op) }
 
-zonkExpr (HsPar x lpar e rpar)
+zonkExpr (HsPar x e)
   = do { new_e <- zonkLExpr e
-       ; return (HsPar x lpar new_e rpar) }
+       ; return (HsPar x new_e) }
 
 zonkExpr (SectionL x _ _) = dataConCantHappen x
 zonkExpr (SectionR x _ _) = dataConCantHappen x
@@ -1016,10 +1026,10 @@ zonkExpr (HsMultiIf ty alts)
             do { expr' <- zonkLExpr expr
                ; return $ GRHS x guard' expr' }
 
-zonkExpr (HsLet x tkLet binds tkIn expr)
+zonkExpr (HsLet x binds expr)
   = runZonkBndrT (zonkLocalBinds binds) $ \ new_binds ->
     do { new_expr <- zonkLExpr expr
-       ; return (HsLet x tkLet new_binds tkIn new_expr) }
+       ; return (HsLet x new_binds new_expr) }
 
 zonkExpr (HsDo ty do_or_lc (L l stmts))
   = do new_stmts <- don'tBind $ zonkStmts zonkLExpr stmts
@@ -1064,13 +1074,17 @@ zonkExpr (HsStatic (fvs, ty) expr)
   = do new_ty <- zonkTcTypeToTypeX ty
        HsStatic (fvs, new_ty) <$> zonkLExpr expr
 
+zonkExpr (HsEmbTy x _) = dataConCantHappen x
+
 zonkExpr (XExpr (WrapExpr (HsWrap co_fn expr)))
   = runZonkBndrT (zonkCoFn co_fn) $ \ new_co_fn ->
     do new_expr <- zonkExpr expr
        return (XExpr (WrapExpr (HsWrap new_co_fn new_expr)))
 
-zonkExpr (XExpr (ExpansionExpr (HsExpanded a b)))
-  = XExpr . ExpansionExpr . HsExpanded a <$> zonkExpr b
+zonkExpr (XExpr (ExpandedThingTc thing e))
+  = do e' <- zonkExpr e
+       return $ XExpr (ExpandedThingTc thing e')
+
 
 zonkExpr (XExpr (ConLikeTc con tvs tys))
   = XExpr . ConLikeTc con tvs <$> mapM zonk_scale tys
@@ -1148,22 +1162,18 @@ zonkCmd (HsCmdApp x c e)
        new_e <- zonkLExpr e
        return (HsCmdApp x new_c new_e)
 
-zonkCmd (HsCmdLam x matches)
-  = do new_matches <- zonkMatchGroup zonkLCmd matches
-       return (HsCmdLam x new_matches)
-
-zonkCmd (HsCmdPar x lpar c rpar)
+zonkCmd (HsCmdPar x c)
   = do new_c <- zonkLCmd c
-       return (HsCmdPar x lpar new_c rpar)
+       return (HsCmdPar x new_c)
 
 zonkCmd (HsCmdCase x expr ms)
   = do new_expr <- zonkLExpr expr
        new_ms <- zonkMatchGroup zonkLCmd ms
        return (HsCmdCase x new_expr new_ms)
 
-zonkCmd (HsCmdLamCase x lc_variant ms)
+zonkCmd (HsCmdLam x lam_variant ms)
   = do new_ms <- zonkMatchGroup zonkLCmd ms
-       return (HsCmdLamCase x lc_variant new_ms)
+       return (HsCmdLam x lam_variant new_ms)
 
 zonkCmd (HsCmdIf x eCond ePred cThen cElse)
   = runZonkBndrT (zonkSyntaxExpr eCond) $ \ new_eCond ->
@@ -1172,10 +1182,10 @@ zonkCmd (HsCmdIf x eCond ePred cThen cElse)
        ; new_cElse <- zonkLCmd cElse
        ; return (HsCmdIf x new_eCond new_ePred new_cThen new_cElse) }
 
-zonkCmd (HsCmdLet x tkLet binds tkIn cmd)
+zonkCmd (HsCmdLet x binds cmd)
   = runZonkBndrT (zonkLocalBinds binds) $ \ new_binds ->
     do new_cmd <- zonkLCmd cmd
-       return (HsCmdLet x tkLet new_binds tkIn new_cmd)
+       return (HsCmdLet x new_binds new_cmd)
 
 zonkCmd (HsCmdDo ty (L l stmts))
   = do new_stmts <- don'tBind $ zonkStmts zonkLCmd stmts
@@ -1194,7 +1204,7 @@ zonk_cmd_top (HsCmdTop (CmdTopTc stack_tys ty ids) cmd)
        new_ty <- zonkTcTypeToTypeX ty
        new_ids <- mapSndM zonkExpr ids
 
-       massert (isLiftedTypeKind (typeKind new_stack_tys))
+       massert (definitelyLiftedType new_stack_tys)
          -- desugarer assumes that this is not representation-polymorphic...
          -- but indeed it should always be lifted due to the typing
          -- rules for arrows
@@ -1467,6 +1477,7 @@ zonkRecFields (HsRecFields flds dd)
 ************************************************************************
 -}
 
+
 zonkPat :: LPat GhcTc -> ZonkBndrTcM (LPat GhcTc)
 -- Extend the environment as we go, because it's possible for one
 -- pattern to bind something that is used in another (inside or
@@ -1474,9 +1485,9 @@ zonkPat :: LPat GhcTc -> ZonkBndrTcM (LPat GhcTc)
 zonkPat pat = wrapLocZonkBndrMA zonk_pat pat
 
 zonk_pat :: Pat GhcTc -> ZonkBndrTcM (Pat GhcTc)
-zonk_pat (ParPat x lpar p rpar)
+zonk_pat (ParPat x p)
   = do  { p' <- zonkPat p
-        ; return (ParPat x lpar p' rpar) }
+        ; return (ParPat x p') }
 
 zonk_pat (WildPat ty)
   = do  { ty' <- noBinders $ zonkTcTypeToTypeX ty
@@ -1494,10 +1505,10 @@ zonk_pat (BangPat x pat)
   = do  { pat' <- zonkPat pat
         ; return (BangPat x pat') }
 
-zonk_pat (AsPat x (L loc v) at pat)
+zonk_pat (AsPat x (L loc v) pat)
   = do  { v'   <- zonkIdBndrX v
         ; pat' <- zonkPat pat
-        ; return (AsPat x (L loc v') at pat') }
+        ; return (AsPat x (L loc v') pat') }
 
 zonk_pat (ViewPat ty expr pat)
   = do  { expr' <- noBinders $ zonkLExpr expr
@@ -1577,6 +1588,14 @@ zonk_pat (NPlusKPat ty (L loc n) (L l lit1) lit2 e1 e2)
         ; n'    <- zonkIdBndrX n
         ; return (NPlusKPat ty' (L loc n') (L l lit1') lit2' e1' e2') }
 
+zonk_pat (EmbTyPat ty tp)
+  = do { ty' <- noBinders $ zonkTcTypeToTypeX ty
+       ; return (EmbTyPat ty' tp) }
+
+zonk_pat (InvisPat ty tp)
+  = do { ty' <- noBinders $ zonkTcTypeToTypeX ty
+       ; return (InvisPat ty' tp) }
+
 zonk_pat (XPat ext) = case ext of
   { ExpansionPat orig pat->
     do { pat' <- zonk_pat pat
