@@ -3,8 +3,8 @@
 (c) The GRASP/AQUA Project, Glasgow University, 1992-1998
 -}
 
-{-# LANGUAGE DeriveDataTypeable #-}
-{-# LANGUAGE BangPatterns #-}
+{-# LANGUAGE LambdaCase #-}
+{-# LANGUAGE MagicHash #-}
 {-# LANGUAGE OverloadedStrings #-}
 
 -- |
@@ -29,7 +29,7 @@ module GHC.Types.Name.Occurrence (
 
         -- ** Construction
         -- $real_vs_source_data_constructors
-        tcName, clsName, tcClsName, dataName, varName,
+        tcName, clsName, tcClsName, dataName, varName, fieldName,
         tvName, srcDataName,
 
         -- ** Pretty Printing
@@ -37,11 +37,12 @@ module GHC.Types.Name.Occurrence (
 
         -- * The 'OccName' type
         OccName,        -- Abstract, instance of Outputable
-        pprOccName,
+        pprOccName, occNameMangledFS,
 
         -- ** Construction
         mkOccName, mkOccNameFS,
         mkVarOcc, mkVarOccFS,
+        mkRecFieldOcc, mkRecFieldOccFS,
         mkDataOcc, mkDataOccFS,
         mkTyVarOcc, mkTyVarOccFS,
         mkTcOcc, mkTcOccFS,
@@ -49,7 +50,10 @@ module GHC.Types.Name.Occurrence (
         mkDFunOcc,
         setOccNameSpace,
         demoteOccName,
+        demoteOccTvName,
         promoteOccName,
+        varToRecFieldOcc,
+        recFieldToVarOcc,
         HasOccName(..),
 
         -- ** Derived 'OccName's
@@ -66,30 +70,40 @@ module GHC.Types.Name.Occurrence (
         mkSuperDictSelOcc, mkSuperDictAuxOcc,
         mkLocalOcc, mkMethodOcc, mkInstTyTcOcc,
         mkInstTyCoOcc, mkEqPredCoOcc,
-        mkRecFldSelOcc,
         mkTyConRepOcc,
 
         -- ** Deconstruction
         occNameFS, occNameString, occNameSpace,
 
         isVarOcc, isTvOcc, isTcOcc, isDataOcc, isDataSymOcc, isSymOcc, isValOcc,
-        parenSymOcc, startsWithUnderscore,
+        isFieldOcc, fieldOcc_maybe,
+        parenSymOcc, startsWithUnderscore, isUnderscore,
 
         isTcClsNameSpace, isTvNameSpace, isDataConNameSpace, isVarNameSpace, isValNameSpace,
+        isFieldNameSpace, isTermVarOrFieldNameSpace,
 
         -- * The 'OccEnv' type
-        OccEnv, emptyOccEnv, unitOccEnv, extendOccEnv, mapOccEnv,
-        lookupOccEnv, mkOccEnv, mkOccEnv_C, extendOccEnvList, elemOccEnv,
-        nonDetOccEnvElts, foldOccEnv, plusOccEnv, plusOccEnv_C, extendOccEnv_C,
+        OccEnv, emptyOccEnv, unitOccEnv, extendOccEnv,
+        mapOccEnv, strictMapOccEnv,
+        mapMaybeOccEnv,
+        lookupOccEnv, lookupOccEnv_AllNameSpaces,
+        lookupOccEnv_WithFields, lookupFieldsOccEnv,
+        mkOccEnv, mkOccEnv_C, extendOccEnvList, elemOccEnv,
+        nonDetOccEnvElts, nonDetFoldOccEnv,
+        plusOccEnv, plusOccEnv_C,
         extendOccEnv_Acc, filterOccEnv, delListFromOccEnv, delFromOccEnv,
-        alterOccEnv, minusOccEnv, minusOccEnv_C, pprOccEnv,
+        alterOccEnv, minusOccEnv, minusOccEnv_C, minusOccEnv_C_Ns,
+        pprOccEnv, forceOccEnv,
+        intersectOccEnv_C,
 
         -- * The 'OccSet' type
         OccSet, emptyOccSet, unitOccSet, mkOccSet, extendOccSet,
         extendOccSetList,
-        unionOccSets, unionManyOccSets, minusOccSet, elemOccSet,
-        isEmptyOccSet, intersectOccSet,
-        filterOccSet, occSetToEnv,
+        unionOccSets, unionManyOccSets, elemOccSet,
+        isEmptyOccSet,
+
+        -- * Dealing with main
+        mainOcc, ppMainFn,
 
         -- * Tidying up
         TidyOccEnv, emptyTidyOccEnv, initTidyOccEnv,
@@ -101,9 +115,9 @@ module GHC.Types.Name.Occurrence (
 
 import GHC.Prelude
 
+import GHC.Builtin.Uniques
 import GHC.Utils.Misc
 import GHC.Types.Unique
-import GHC.Builtin.Uniques
 import GHC.Types.Unique.FM
 import GHC.Types.Unique.Set
 import GHC.Data.FastString
@@ -111,10 +125,13 @@ import GHC.Data.FastString.Env
 import GHC.Utils.Outputable
 import GHC.Utils.Lexeme
 import GHC.Utils.Binary
+import GHC.Utils.Panic.Plain
+
 import Control.DeepSeq
 import Data.Char
 import Data.Data
 import qualified Data.Semigroup as S
+import GHC.Exts( Int(I#), dataToTag# )
 
 {-
 ************************************************************************
@@ -124,32 +141,108 @@ import qualified Data.Semigroup as S
 ************************************************************************
 -}
 
-data NameSpace = VarName        -- Variables, including "real" data constructors
-               | DataName       -- "Source" data constructors
-               | TvName         -- Type variables
-               | TcClsName      -- Type constructors and classes; Haskell has them
-                                -- in the same name space for now.
-               deriving( Eq, Ord )
+data NameSpace
+  -- | Variable name space (including "real" data constructors).
+  = VarName
+  -- | Record field namespace for the given record.
+  | FldName
+    { fldParent :: !FastString
+      -- ^ The textual name of the parent of the field.
+      --
+      --   - For a field of a datatype, this is the name of the first constructor
+      --     of the datatype (regardless of whether this constructor has this field).
+      --   - For a field of a pattern synonym, this is the name of the pattern synonym.
+    }
+  -- | "Source" data constructor namespace.
+  | DataName
+  -- | Type variable namespace.
+  | TvName
+  -- | Type constructor and class namespace.
+  | TcClsName
+    -- Haskell has type constructors and classes in the same namespace, for now.
+   deriving Eq
 
--- Note [Data Constructors]
--- ~~~~~~~~~~~~~~~~~~~~~~~~
--- see also: Note [Data Constructor Naming] in GHC.Core.DataCon
---
--- $real_vs_source_data_constructors
--- There are two forms of data constructor:
---
---      [Source data constructors] The data constructors mentioned in Haskell source code
---
---      [Real data constructors] The data constructors of the representation type, which may not be the same as the source type
---
--- For example:
---
--- > data T = T !(Int, Int)
---
--- The source datacon has type @(Int, Int) -> T@
--- The real   datacon has type @Int -> Int -> T@
---
--- GHC chooses a representation based on the strictness etc.
+instance Ord NameSpace where
+  compare ns1 ns2 =
+    case compare (I# (dataToTag# ns1)) (I# (dataToTag# ns2)) of
+      LT -> LT
+      GT -> GT
+      EQ
+        | FldName { fldParent = p1 } <- ns1
+        , FldName { fldParent = p2 } <- ns2
+        -> lexicalCompareFS p1 p2
+        | otherwise
+        -> EQ
+
+instance Uniquable NameSpace where
+  getUnique (FldName fs) = mkFldNSUnique  fs
+  getUnique VarName      = varNSUnique
+  getUnique DataName     = dataNSUnique
+  getUnique TvName       = tvNSUnique
+  getUnique TcClsName    = tcNSUnique
+
+instance NFData NameSpace where
+  rnf VarName = ()
+  rnf (FldName par) = rnf par
+  rnf DataName = ()
+  rnf TvName = ()
+  rnf TcClsName = ()
+
+{-
+Note [Data Constructors]
+~~~~~~~~~~~~~~~~~~~~~~~~
+see also: Note [Data Constructor Naming] in GHC.Core.DataCon
+
+$real_vs_source_data_constructors
+There are two forms of data constructor:
+
+     [Source data constructors] The data constructors mentioned in Haskell source code
+
+     [Real data constructors] The data constructors of the representation type, which may not be the same as the source type
+
+For example:
+
+> data T = T !(Int, Int)
+
+The source datacon has type @(Int, Int) -> T@
+The real   datacon has type @Int -> Int -> T@
+
+GHC chooses a representation based on the strictness etc.
+
+Note [Record field namespacing]
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+Record fields have a separate namespace from variables, to support
+DuplicateRecordFields, e.g. in
+
+  data X = MkX { fld :: Int }
+  data Y = MkY { fld :: Bool }
+
+  f x = x { fld = 3 }
+  g y = y { fld = False }
+
+we want the two occurrences of "fld" to refer to the field names associated with
+the corresponding data type.
+
+The namespace for a record field is as follows:
+
+  - for a data type, it is the textual name of the first constructor of the
+    datatype, whether this constructor has this field or not;
+  - for a pattern synonym, it is the textual name of the pattern synonym itself.
+
+Record fields are initially parsed as variables, but the renamer resolves their
+namespace in GHC.Rename.Names.newRecordFieldLabel, which is called when renaming
+record data declarations and record pattern synonym declarations.
+
+To illustrate the namespacing, consider the record field "fld" in the following datatype
+
+  data instance A Int Bool Char
+    = MkA1 | MkA2 { fld :: Int } | MkA3 { bar :: Bool, fld :: Int }
+
+Its namespace is `FldName "MkA1"`. This is a convention used throughout GHC
+to circumvent the fact that we don't have a way to refer to the type constructor
+"A Int Bool Char" in the renamer, as data family instances only get given
+'Name's in the typechecker.
+-}
 
 tcName, clsName, tcClsName :: NameSpace
 dataName, srcDataName      :: NameSpace
@@ -168,6 +261,9 @@ srcDataName = DataName  -- Haskell-source data constructors should be
 tvName      = TvName
 varName     = VarName
 
+fieldName :: FastString -> NameSpace
+fieldName = FldName
+
 isDataConNameSpace :: NameSpace -> Bool
 isDataConNameSpace DataName = True
 isDataConNameSpace _        = False
@@ -181,30 +277,44 @@ isTvNameSpace TvName = True
 isTvNameSpace _      = False
 
 isVarNameSpace :: NameSpace -> Bool     -- Variables or type variables, but not constructors
-isVarNameSpace TvName  = True
-isVarNameSpace VarName = True
-isVarNameSpace _       = False
+isVarNameSpace TvName       = True
+isVarNameSpace VarName      = True
+isVarNameSpace (FldName {}) = True
+isVarNameSpace _            = False
+
+-- | Is this a term variable or field name namespace?
+isTermVarOrFieldNameSpace :: NameSpace -> Bool
+isTermVarOrFieldNameSpace VarName      = True
+isTermVarOrFieldNameSpace (FldName {}) = True
+isTermVarOrFieldNameSpace _            = False
 
 isValNameSpace :: NameSpace -> Bool
-isValNameSpace DataName = True
-isValNameSpace VarName  = True
-isValNameSpace _        = False
+isValNameSpace DataName     = True
+isValNameSpace VarName      = True
+isValNameSpace (FldName {}) = True
+isValNameSpace _            = False
+
+isFieldNameSpace :: NameSpace -> Bool
+isFieldNameSpace (FldName {}) = True
+isFieldNameSpace _            = False
 
 pprNameSpace :: NameSpace -> SDoc
-pprNameSpace DataName  = text "data constructor"
-pprNameSpace VarName   = text "variable"
-pprNameSpace TvName    = text "type variable"
-pprNameSpace TcClsName = text "type constructor or class"
+pprNameSpace DataName    = text "data constructor"
+pprNameSpace VarName     = text "variable"
+pprNameSpace TvName      = text "type variable"
+pprNameSpace TcClsName   = text "type constructor or class"
+pprNameSpace (FldName p) = text "record field of" <+> ftext p
 
 pprNonVarNameSpace :: NameSpace -> SDoc
 pprNonVarNameSpace VarName = empty
 pprNonVarNameSpace ns = pprNameSpace ns
 
-pprNameSpaceBrief :: IsLine doc => NameSpace -> doc
-pprNameSpaceBrief DataName  = char 'd'
-pprNameSpaceBrief VarName   = char 'v'
-pprNameSpaceBrief TvName    = text "tv"
-pprNameSpaceBrief TcClsName = text "tc"
+pprNameSpaceBrief :: NameSpace -> SDoc
+pprNameSpaceBrief DataName     = char 'd'
+pprNameSpaceBrief VarName      = char 'v'
+pprNameSpaceBrief TvName       = text "tv"
+pprNameSpaceBrief TcClsName    = text "tc"
+pprNameSpaceBrief (FldName {}) = text "fld"
 
 -- demoteNameSpace lowers the NameSpace if possible.  We can not know
 -- in advance, since a TvName can appear in an HsTyVar.
@@ -214,6 +324,16 @@ demoteNameSpace VarName = Nothing
 demoteNameSpace DataName = Nothing
 demoteNameSpace TvName = Nothing
 demoteNameSpace TcClsName = Just DataName
+demoteNameSpace (FldName {}) = Nothing
+
+-- demoteTvNameSpace lowers the NameSpace of a type variable.
+-- See Note [Demotion] in GHC.Rename.Env.
+demoteTvNameSpace :: NameSpace -> Maybe NameSpace
+demoteTvNameSpace TvName = Just VarName
+demoteTvNameSpace VarName = Nothing
+demoteTvNameSpace DataName = Nothing
+demoteTvNameSpace TcClsName = Nothing
+demoteTvNameSpace (FldName {}) = Nothing
 
 -- promoteNameSpace promotes the NameSpace as follows.
 -- See Note [Promotion] in GHC.Rename.Env.
@@ -222,6 +342,7 @@ promoteNameSpace DataName = Just TcClsName
 promoteNameSpace VarName = Just TvName
 promoteNameSpace TcClsName = Nothing
 promoteNameSpace TvName = Nothing
+promoteNameSpace (FldName {}) = Nothing
 
 {-
 ************************************************************************
@@ -246,7 +367,8 @@ instance Eq OccName where
 
 instance Ord OccName where
         -- Compares lexicographically, *not* by Unique of the string
-    compare (OccName sp1 s1) (OccName sp2 s2) = lexicalCompareFS s1 s2 S.<> compare sp1 sp2
+    compare (OccName sp1 s1) (OccName sp2 s2) =
+      lexicalCompareFS s1 s2 S.<> compare sp1 sp2
 
 instance Data OccName where
   -- don't traverse?
@@ -278,10 +400,40 @@ instance OutputableBndr OccName where
 
 pprOccName :: IsLine doc => OccName -> doc
 pprOccName (OccName sp occ)
-  = docWithContext $ \ sty ->
-    if codeStyle (sdocStyle sty)
-    then ztext (zEncodeFS occ)
-    else ftext occ <> whenPprDebug (braces (pprNameSpaceBrief sp))
+  = docWithStyle (ztext (zEncodeFS occ))
+    (\_ -> ftext occ <> whenPprDebug (braces (pprNameSpaceBrief sp)))
+{-# SPECIALIZE pprOccName :: OccName -> SDoc #-}
+{-# SPECIALIZE pprOccName :: OccName -> HLine #-} -- see Note [SPECIALIZE to HDoc] in GHC.Utils.Outputable
+
+-- | Mangle field names to avoid duplicate symbols.
+--
+-- See Note [Mangling OccNames].
+occNameMangledFS :: OccName -> FastString
+occNameMangledFS (OccName ns fs) =
+  case ns of
+    -- Fields need to include the constructor, to ensure that we don't define
+    -- duplicate symbols when using DuplicateRecordFields.
+    FldName con -> concatFS [fsLit "$fld:", con, ":", fs]
+    -- Otherwise, we can ignore the namespace, as there is no risk of name
+    -- clashes.
+    _           -> fs
+
+{- Note [Mangling OccNames]
+~~~~~~~~~~~~~~~~~~~~~~~~~~~
+When generating a symbol for a Name, we usually discard the NameSpace entirely
+(see GHC.Types.Name.pprName). This is because clashes are usually not possible,
+e.g. a variable and a data constructor can't clash because data constructors
+start with a capital letter or a colon, while variables never do.
+
+However, record field names, in the presence of DuplicateRecordFields, need this
+disambiguation. So, for a record field like
+
+  data A = MkA { foo :: Int }
+
+we generate the symbol $fld:MkA:foo. We use the constructor 'MkA' to disambiguate,
+and not the TyCon A as one might naively expect: this is explained in
+Note [Record field namespacing].
+-}
 
 {-
 ************************************************************************
@@ -303,6 +455,27 @@ mkVarOcc s = mkOccName varName s
 mkVarOccFS :: FastString -> OccName
 mkVarOccFS fs = mkOccNameFS varName fs
 
+mkRecFieldOcc :: FastString -> String -> OccName
+mkRecFieldOcc dc = mkOccName (fieldName dc)
+
+mkRecFieldOccFS :: FastString -> FastString -> OccName
+mkRecFieldOccFS dc = mkOccNameFS (fieldName dc)
+
+varToRecFieldOcc :: HasDebugCallStack => FastString -> OccName -> OccName
+varToRecFieldOcc dc (OccName ns s) =
+  assert makes_sense $ mkRecFieldOccFS dc s
+    where
+      makes_sense = case ns of
+        VarName    -> True
+        FldName {} -> True
+          -- NB: it's OK to change the parent data constructor,
+          -- see e.g. test T23220 in which we construct with TH
+          -- a datatype using the fields of a different datatype.
+        _          -> False
+
+recFieldToVarOcc :: HasDebugCallStack => OccName -> OccName
+recFieldToVarOcc (OccName _ns s) = mkVarOccFS s
+
 mkDataOcc :: String -> OccName
 mkDataOcc = mkOccName dataName
 
@@ -334,11 +507,18 @@ demoteOccName (OccName space name) = do
   space' <- demoteNameSpace space
   return $ OccName space' name
 
+demoteOccTvName :: OccName -> Maybe OccName
+demoteOccTvName (OccName space name) = do
+  space' <- demoteTvNameSpace space
+  return $ OccName space' name
+
 -- promoteOccName promotes the NameSpace of OccName.
 -- See Note [Promotion] in GHC.Rename.Env.
 promoteOccName :: OccName -> Maybe OccName
 promoteOccName (OccName space name) = do
-  space' <- promoteNameSpace space
+  promoted_space <- promoteNameSpace space
+  let tyop   = isTvNameSpace promoted_space && isLexVarSym name
+      space' = if tyop then tcClsName else promoted_space   -- special case for type operators (#24570)
   return $ OccName space' name
 
 {- | Other names in the compiler add additional information to an OccName.
@@ -353,90 +533,290 @@ class HasOccName name where
 *                                                                      *
 ************************************************************************
 
-OccEnvs are used mainly for the envts in ModIfaces.
+OccEnvs are used for the GlobalRdrEnv and for the envts in ModIface.
 
-Note [The Unique of an OccName]
-~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-They are efficient, because FastStrings have unique Int# keys.  We assume
-this key is less than 2^24, and indeed FastStrings are allocated keys
-sequentially starting at 0.
+Note [OccEnv]
+~~~~~~~~~~~~~
+An OccEnv is a map keyed on OccName. Recall that an OccEnv consists of two
+components:
 
-So we can make a Unique using
-        mkUnique ns key  :: Unique
-where 'ns' is a Char representing the name space.  This in turn makes it
-easy to build an OccEnv.
+  - a namespace,
+  - a textual name (in the form of a FastString).
+
+In general, for a given textual name, there is only one appropriate namespace.
+However, sometimes we do get an occurrence that belongs to several namespaces:
+
+  - Symbolic identifiers such as (:+) can belong to both the data constructor and
+    type constructor/class namespaces.
+  - With duplicate record fields, a field name can belong to several different
+    namespaces, one for each parent datatype (or pattern synonym).
+
+So we represent an OccEnv as a nested data structure
+
+  FastStringEnv (UniqFM NameSpace a)
+
+in which we can first look up the textual name, and then choose which of the
+namespaces are relevant. This supports the two main uses of OccEnvs:
+
+  1. One wants to look up a specific OccName in the environment, at a specific
+     namespace. One looks up the textual name, and then the namespace.
+  2. One wants to look up something, but isn't sure in advance of the namespace.
+     So one looks up the textual name, and then can decide what to do based on
+     the returned map of namespaces.
+
+This data structure isn't performance critical in most situations, but some
+improvements to its performance that might be worth it are as follows:
+
+  A. Use a tailor-made data structure for a map keyed on NameSpaces.
+
+     Recall that we have:
+
+        data IntMap a = Bin !Int !Int !(IntMap a) !(IntMap a)
+                      | Tip !Key a
+                      | Nil
+
+     This is already pretty efficient for singletons, but we don't need the
+     empty case (as we would simply omit the parent key in the OccEnv instead
+     of storing an empty inner map).
+
+  B. Always ensure the inner map (keyed on namespaces) is evaluated, i.e.
+     is never a thunk. For this, we would need to use strict operations on
+     the outer FastStringEnv (but we'd keep using lazy operations on the inner
+     UniqFM).
 -}
 
-instance Uniquable OccName where
-      -- See Note [The Unique of an OccName]
-  getUnique (OccName VarName   fs) = mkVarOccUnique  fs
-  getUnique (OccName DataName  fs) = mkDataOccUnique fs
-  getUnique (OccName TvName    fs) = mkTvOccUnique   fs
-  getUnique (OccName TcClsName fs) = mkTcOccUnique   fs
-
-newtype OccEnv a = A (UniqFM OccName a)
-  deriving Data
+-- | A map keyed on 'OccName'. See Note [OccEnv].
+newtype OccEnv a = MkOccEnv (FastStringEnv (UniqFM NameSpace a))
+  deriving Functor
 
+-- | The empty 'OccEnv'.
 emptyOccEnv :: OccEnv a
-unitOccEnv  :: OccName -> a -> OccEnv a
+emptyOccEnv = MkOccEnv emptyFsEnv
+
+-- | A singleton 'OccEnv'.
+unitOccEnv :: OccName -> a -> OccEnv a
+unitOccEnv (OccName ns s) a = MkOccEnv $ unitFsEnv s (unitUFM ns a)
+
+-- | Add a single element to an 'OccEnv'.
 extendOccEnv :: OccEnv a -> OccName -> a -> OccEnv a
+extendOccEnv (MkOccEnv as) (OccName ns s) a =
+  MkOccEnv $ extendFsEnv_C plusUFM as s (unitUFM ns a)
+
+-- | Extend an 'OccEnv' by a list.
+--
+-- 'OccName's later on in the list override earlier 'OccName's.
 extendOccEnvList :: OccEnv a -> [(OccName, a)] -> OccEnv a
+extendOccEnvList = foldl' $ \ env (occ, a) -> extendOccEnv env occ a
+
+-- | Look an element up in an 'OccEnv'.
 lookupOccEnv :: OccEnv a -> OccName -> Maybe a
-mkOccEnv     :: [(OccName,a)] -> OccEnv a
-mkOccEnv_C   :: (a -> a -> a) -> [(OccName,a)] -> OccEnv a
-elemOccEnv   :: OccName -> OccEnv a -> Bool
-foldOccEnv   :: (a -> b -> b) -> b -> OccEnv a -> b
-nonDetOccEnvElts   :: OccEnv a -> [a]
-extendOccEnv_C :: (a->a->a) -> OccEnv a -> OccName -> a -> OccEnv a
-extendOccEnv_Acc :: (a->b->b) -> (a->b) -> OccEnv b -> OccName -> a -> OccEnv b
-plusOccEnv     :: OccEnv a -> OccEnv a -> OccEnv a
-plusOccEnv_C   :: (a->a->a) -> OccEnv a -> OccEnv a -> OccEnv a
-mapOccEnv      :: (a->b) -> OccEnv a -> OccEnv b
-delFromOccEnv      :: OccEnv a -> OccName -> OccEnv a
+lookupOccEnv (MkOccEnv as) (OccName ns s)
+  = do { m <- lookupFsEnv as s
+       ; lookupUFM m ns }
+
+-- | Lookup an element in an 'OccEnv', ignoring 'NameSpace's entirely.
+lookupOccEnv_AllNameSpaces :: OccEnv a -> OccName -> [a]
+lookupOccEnv_AllNameSpaces (MkOccEnv as) (OccName _ s)
+  = case lookupFsEnv as s of
+      Nothing -> []
+      Just r  -> nonDetEltsUFM r
+
+-- | Lookup an element in an 'OccEnv', looking in the record field
+-- namespace for a variable.
+lookupOccEnv_WithFields :: OccEnv a -> OccName -> [a]
+lookupOccEnv_WithFields env occ =
+  case lookupOccEnv env occ of
+      Nothing  -> fieldGREs
+      Just gre -> gre : fieldGREs
+  where
+    fieldGREs
+      -- If the 'OccName' is a variable, also look up
+      -- in the record field namespaces.
+      | isVarOcc occ
+      = lookupFieldsOccEnv env (occNameFS occ)
+      | otherwise
+      = []
+
+-- | Look up all the record fields that match with the given 'FastString'
+-- in an 'OccEnv'.
+lookupFieldsOccEnv :: OccEnv a -> FastString -> [a]
+lookupFieldsOccEnv (MkOccEnv as) fld =
+  case lookupFsEnv as fld of
+    Nothing   -> []
+    Just flds -> nonDetEltsUFM $ filter_flds flds
+  -- NB: non-determinism is OK: in practice we will either end up resolving
+  -- to a single field or throwing an error.
+  where
+    filter_flds = filterUFM_Directly (\ uniq _ -> isFldNSUnique uniq)
+
+-- | Create an 'OccEnv' from a list.
+--
+-- 'OccName's later on in the list override earlier 'OccName's.
+mkOccEnv :: [(OccName,a)] -> OccEnv a
+mkOccEnv = extendOccEnvList emptyOccEnv
+
+-- | Create an 'OccEnv' from a list, combining different values
+-- with the same 'OccName' using the combining function.
+mkOccEnv_C :: (a -> a -> a) -- ^ old -> new -> result
+           -> [(OccName,a)]
+           -> OccEnv a
+mkOccEnv_C f elts
+  = MkOccEnv $ foldl' g emptyFsEnv elts
+    where
+      g env (OccName ns s, a) =
+        extendFsEnv_C (plusUFM_C $ flip f) env s (unitUFM ns a)
+
+-- | Compute whether there is a value keyed by the given 'OccName'.
+elemOccEnv :: OccName -> OccEnv a -> Bool
+elemOccEnv (OccName ns s) (MkOccEnv as)
+  = case lookupFsEnv as s of
+      Nothing -> False
+      Just m  -> ns `elemUFM` m
+
+-- | Fold over an 'OccEnv'. Non-deterministic, unless the folding function
+-- is commutative (i.e. @a1 `f` ( a2 `f` b ) == a2 `f` ( a1 `f` b )@ for all @a1@, @a2@, @b@).
+nonDetFoldOccEnv :: (a -> b -> b) -> b -> OccEnv a -> b
+nonDetFoldOccEnv f b0 (MkOccEnv as) =
+  nonDetFoldFsEnv (flip $ nonDetFoldUFM f) b0 as
+
+-- | Obtain the elements of an 'OccEnv'.
+--
+-- The resulting order is non-deterministic.
+nonDetOccEnvElts :: OccEnv a -> [a]
+nonDetOccEnvElts = nonDetFoldOccEnv (:) []
+
+-- | Union of two 'OccEnv's, right-biased.
+plusOccEnv :: OccEnv a -> OccEnv a -> OccEnv a
+plusOccEnv (MkOccEnv env1) (MkOccEnv env2)
+  = MkOccEnv $ plusFsEnv_C plusUFM env1 env2
+
+-- | Union of two 'OccEnv's with a combining function.
+plusOccEnv_C :: (a->a->a) -> OccEnv a -> OccEnv a -> OccEnv a
+plusOccEnv_C f (MkOccEnv env1) (MkOccEnv env2)
+  = MkOccEnv $ plusFsEnv_C (plusUFM_C f) env1 env2
+
+-- | Map over an 'OccEnv' ('Functor' instance).
+mapOccEnv :: (a->b) -> OccEnv a -> OccEnv b
+mapOccEnv = fmap
+
+-- | 'mapMaybe' for b 'OccEnv'.
+mapMaybeOccEnv :: (a -> Maybe b) -> OccEnv a -> OccEnv b
+mapMaybeOccEnv f (MkOccEnv env)
+  = MkOccEnv $ mapMaybeUFM g env
+    where
+      g as =
+        case mapMaybeUFM f as of
+          m' | isNullUFM m' -> Nothing
+             | otherwise    -> Just m'
+
+-- | Add a single element to an 'OccEnv', using a different function whether
+-- the 'OccName' already exists or not.
+extendOccEnv_Acc :: forall a b
+                 .  (a->b->b)    -- ^ add to existing
+                 -> (a->b)       -- ^ new element
+                 -> OccEnv b     -- ^ old
+                 -> OccName -> a -- ^ new
+                 -> OccEnv b
+extendOccEnv_Acc f g (MkOccEnv env) (OccName ns s) =
+  MkOccEnv . extendFsEnv_Acc f' g' env s
+    where
+     f' :: a -> UniqFM NameSpace b -> UniqFM NameSpace b
+     f' a bs = alterUFM (Just . \ case { Nothing -> g a ; Just b -> f a b }) bs ns
+     g' a = unitUFM ns (g a)
+
+-- | Delete one element from an 'OccEnv'.
+delFromOccEnv :: forall a. OccEnv a -> OccName -> OccEnv a
+delFromOccEnv (MkOccEnv env1) (OccName ns s) =
+  MkOccEnv $ alterFsEnv f env1 s
+    where
+      f :: Maybe (UniqFM NameSpace a) -> Maybe (UniqFM NameSpace a)
+      f Nothing = Nothing
+      f (Just m) =
+        case delFromUFM m ns of
+          m' | isNullUFM m' -> Nothing
+             | otherwise    -> Just m'
+
+-- | Delete multiple elements from an 'OccEnv'.
 delListFromOccEnv :: OccEnv a -> [OccName] -> OccEnv a
-filterOccEnv       :: (elt -> Bool) -> OccEnv elt -> OccEnv elt
-alterOccEnv        :: (Maybe elt -> Maybe elt) -> OccEnv elt -> OccName -> OccEnv elt
+delListFromOccEnv = foldl' delFromOccEnv
+
+-- | Filter out all elements in an 'OccEnv' using a predicate.
+filterOccEnv :: forall a. (a -> Bool) -> OccEnv a -> OccEnv a
+filterOccEnv f (MkOccEnv env) =
+  MkOccEnv $ mapMaybeFsEnv g env
+    where
+      g :: UniqFM NameSpace a -> Maybe (UniqFM NameSpace a)
+      g ms =
+        case filterUFM f ms of
+          m' | isNullUFM m' -> Nothing
+             | otherwise    -> Just m'
+
+-- | Alter an 'OccEnv', adding or removing an element at the given key.
+alterOccEnv :: forall a. (Maybe a -> Maybe a) -> OccEnv a -> OccName -> OccEnv a
+alterOccEnv f (MkOccEnv env) (OccName ns s) =
+  MkOccEnv $ alterFsEnv g env s
+    where
+      g :: Maybe (UniqFM NameSpace a) -> Maybe (UniqFM NameSpace a)
+      g Nothing  = fmap (unitUFM ns) (f Nothing)
+      g (Just m) =
+        case alterUFM f m ns of
+          m' | isNullUFM m' -> Nothing
+             | otherwise    -> Just m'
+
+intersectOccEnv_C :: (a -> b -> c) -> OccEnv a -> OccEnv b -> OccEnv c
+intersectOccEnv_C f (MkOccEnv as) (MkOccEnv bs)
+  = MkOccEnv $ intersectUFM_C (intersectUFM_C f) as bs
+
+-- | Remove elements of the first 'OccEnv' that appear in the second 'OccEnv'.
 minusOccEnv :: OccEnv a -> OccEnv b -> OccEnv a
+minusOccEnv = minusOccEnv_C_Ns minusUFM
 
--- | Alters (replaces or removes) those elements of the map that are mentioned in the second map
-minusOccEnv_C :: (a -> b -> Maybe a) -> OccEnv a -> OccEnv b -> OccEnv a
+-- | Alters (replaces or removes) those elements of the first 'OccEnv' that are
+-- mentioned in the second 'OccEnv'.
+--
+-- Same idea as 'Data.Map.differenceWith'.
+minusOccEnv_C :: (a -> b -> Maybe a)
+              -> OccEnv a -> OccEnv b -> OccEnv a
+minusOccEnv_C f = minusOccEnv_C_Ns (minusUFM_C f)
 
-emptyOccEnv      = A emptyUFM
-unitOccEnv x y = A $ unitUFM x y
-extendOccEnv (A x) y z = A $ addToUFM x y z
-extendOccEnvList (A x) l = A $ addListToUFM x l
-lookupOccEnv (A x) y = lookupUFM x y
-mkOccEnv     l    = A $ listToUFM l
-elemOccEnv x (A y)       = elemUFM x y
-foldOccEnv a b (A c)     = foldUFM a b c
-nonDetOccEnvElts (A x)         = nonDetEltsUFM x
-plusOccEnv (A x) (A y)   = A $ plusUFM x y
-plusOccEnv_C f (A x) (A y)       = A $ plusUFM_C f x y
-extendOccEnv_C f (A x) y z   = A $ addToUFM_C f x y z
-extendOccEnv_Acc f g (A x) y z   = A $ addToUFM_Acc f g x y z
-mapOccEnv f (A x)        = A $ mapUFM f x
-mkOccEnv_C comb l = A $ addListToUFM_C comb emptyUFM l
-delFromOccEnv (A x) y    = A $ delFromUFM x y
-delListFromOccEnv (A x) y  = A $ delListFromUFM x y
-filterOccEnv x (A y)       = A $ filterUFM x y
-alterOccEnv fn (A y) k     = A $ alterUFM fn y k
-minusOccEnv (A x) (A y) = A $ minusUFM x y
-minusOccEnv_C fn (A x) (A y) = A $ minusUFM_C fn x y
+minusOccEnv_C_Ns :: forall a b
+                 .  (UniqFM NameSpace a -> UniqFM NameSpace b -> UniqFM NameSpace a)
+                 -> OccEnv a -> OccEnv b -> OccEnv a
+minusOccEnv_C_Ns f (MkOccEnv as) (MkOccEnv bs) =
+  MkOccEnv $ minusUFM_C g as bs
+    where
+      g :: UniqFM NameSpace a -> UniqFM NameSpace b -> Maybe (UniqFM NameSpace a)
+      g as bs =
+        let m = f as bs
+        in if isNullUFM m
+           then Nothing
+           else Just m
 
 instance Outputable a => Outputable (OccEnv a) where
     ppr x = pprOccEnv ppr x
 
 pprOccEnv :: (a -> SDoc) -> OccEnv a -> SDoc
-pprOccEnv ppr_elt (A env) = pprUniqFM ppr_elt env
+pprOccEnv ppr_elt (MkOccEnv env)
+    = brackets $ fsep $ punctuate comma $
+    [ ppr uq <+> text ":->" <+> ppr_elt elt
+    | (uq, elts) <- nonDetUFMToList env
+    , elt <- nonDetEltsUFM elts ]
 
 instance NFData a => NFData (OccEnv a) where
   rnf = forceOccEnv rnf
 
+-- | Map over an 'OccEnv' strictly.
+strictMapOccEnv :: (a -> b) -> OccEnv a -> OccEnv b
+strictMapOccEnv f (MkOccEnv as) =
+  MkOccEnv $ strictMapFsEnv (strictMapUFM f) as
+
 -- | Force an 'OccEnv' with the provided function.
 forceOccEnv :: (a -> ()) -> OccEnv a -> ()
-forceOccEnv nf (A fs) = seqEltsUFM nf fs
+forceOccEnv nf (MkOccEnv fs) = seqEltsUFM (seqEltsUFM nf) fs
 
-type OccSet = UniqSet OccName
+--------------------------------------------------------------------------------
+
+newtype OccSet = OccSet (FastStringEnv (UniqSet NameSpace))
 
 emptyOccSet       :: OccSet
 unitOccSet        :: OccName -> OccSet
@@ -445,27 +825,18 @@ extendOccSet      :: OccSet -> OccName -> OccSet
 extendOccSetList  :: OccSet -> [OccName] -> OccSet
 unionOccSets      :: OccSet -> OccSet -> OccSet
 unionManyOccSets  :: [OccSet] -> OccSet
-minusOccSet       :: OccSet -> OccSet -> OccSet
 elemOccSet        :: OccName -> OccSet -> Bool
 isEmptyOccSet     :: OccSet -> Bool
-intersectOccSet   :: OccSet -> OccSet -> OccSet
-filterOccSet      :: (OccName -> Bool) -> OccSet -> OccSet
--- | Converts an OccSet to an OccEnv (operationally the identity)
-occSetToEnv       :: OccSet -> OccEnv OccName
 
-emptyOccSet       = emptyUniqSet
-unitOccSet        = unitUniqSet
-mkOccSet          = mkUniqSet
-extendOccSet      = addOneToUniqSet
-extendOccSetList  = addListToUniqSet
-unionOccSets      = unionUniqSets
-unionManyOccSets  = unionManyUniqSets
-minusOccSet       = minusUniqSet
-elemOccSet        = elementOfUniqSet
-isEmptyOccSet     = isEmptyUniqSet
-intersectOccSet   = intersectUniqSets
-filterOccSet      = filterUniqSet
-occSetToEnv       = A . getUniqSet
+emptyOccSet       = OccSet emptyFsEnv
+unitOccSet (OccName ns s) = OccSet $ unitFsEnv s (unitUniqSet ns)
+mkOccSet          = extendOccSetList emptyOccSet
+extendOccSet      (OccSet occs) (OccName ns s) = OccSet $ extendFsEnv occs s (unitUniqSet ns)
+extendOccSetList  = foldl' extendOccSet
+unionOccSets      (OccSet xs) (OccSet ys) = OccSet $ plusFsEnv_C unionUniqSets xs ys
+unionManyOccSets  = foldl' unionOccSets emptyOccSet
+elemOccSet (OccName ns s) (OccSet occs) = maybe False (elementOfUniqSet ns) $ lookupFsEnv occs s
+isEmptyOccSet     (OccSet occs) = isNullUFM occs
 
 {-
 ************************************************************************
@@ -481,7 +852,7 @@ occNameString (OccName _ s) = unpackFS s
 setOccNameSpace :: NameSpace -> OccName -> OccName
 setOccNameSpace sp (OccName _ occ) = OccName sp occ
 
-isVarOcc, isTvOcc, isTcOcc, isDataOcc :: OccName -> Bool
+isVarOcc, isTvOcc, isTcOcc, isDataOcc, isFieldOcc :: OccName -> Bool
 
 isVarOcc (OccName VarName _) = True
 isVarOcc _                   = False
@@ -492,12 +863,20 @@ isTvOcc _                  = False
 isTcOcc (OccName TcClsName _) = True
 isTcOcc _                     = False
 
+isFieldOcc (OccName (FldName {}) _) = True
+isFieldOcc _                        = False
+
+fieldOcc_maybe :: OccName -> Maybe FastString
+fieldOcc_maybe (OccName (FldName con) _) = Just con
+fieldOcc_maybe _                         = Nothing
+
 -- | /Value/ 'OccNames's are those that are either in
--- the variable or data constructor namespaces
+-- the variable, field name or data constructor namespaces
 isValOcc :: OccName -> Bool
-isValOcc (OccName VarName  _) = True
-isValOcc (OccName DataName _) = True
-isValOcc _                    = False
+isValOcc (OccName VarName      _) = True
+isValOcc (OccName DataName     _) = True
+isValOcc (OccName (FldName {}) _) = True
+isValOcc _                        = False
 
 isDataOcc (OccName DataName _) = True
 isDataOcc _                    = False
@@ -512,10 +891,12 @@ isDataSymOcc _                    = False
 -- | Test if the 'OccName' is that for any operator (whether
 -- it is a data constructor or variable or whatever)
 isSymOcc :: OccName -> Bool
-isSymOcc (OccName DataName s)  = isLexConSym s
-isSymOcc (OccName TcClsName s) = isLexSym s
-isSymOcc (OccName VarName s)   = isLexSym s
-isSymOcc (OccName TvName s)    = isLexSym s
+isSymOcc (OccName ns s) = case ns of
+  DataName   -> isLexConSym s
+  TcClsName  -> isLexSym s
+  VarName    -> isLexSym s
+  TvName     -> isLexSym s
+  FldName {} -> isLexSym s
 -- Pretty inefficient!
 
 parenSymOcc :: OccName -> SDoc -> SDoc
@@ -530,6 +911,9 @@ startsWithUnderscore occ = case unpackFS (occNameFS occ) of
   '_':_ -> True
   _     -> False
 
+isUnderscore :: OccName -> Bool
+isUnderscore occ = occNameFS occ == fsLit "_"
+
 {-
 ************************************************************************
 *                                                                      *
@@ -652,10 +1036,6 @@ mkTyConRepOcc occ = mk_simple_deriv varName prefix occ
 mkGenR   = mk_simple_deriv tcName "Rep_"
 mkGen1R  = mk_simple_deriv tcName "Rep1_"
 
--- Overloaded record field selectors
-mkRecFldSelOcc :: FastString -> OccName
-mkRecFldSelOcc s = mk_deriv varName "$sel" [s]
-
 mk_simple_deriv :: NameSpace -> FastString -> OccName -> OccName
 mk_simple_deriv sp px occ = mk_deriv sp px [occNameFS occ]
 
@@ -883,6 +1263,24 @@ tidyOccName env occ@(OccName occ_sp fs)
                      -- See Note [TidyOccEnv]
 
 
+{-
+************************************************************************
+*                                                                      *
+                            Utilies for "main"
+*                                                                      *
+************************************************************************
+-}
+
+mainOcc :: OccName
+mainOcc = mkVarOccFS (fsLit "main")
+
+ppMainFn :: OccName -> SDoc
+ppMainFn main_occ
+  | main_occ == mainOcc
+  = text "IO action" <+> quotes (ppr main_occ)
+  | otherwise
+  = text "main IO action" <+> quotes (ppr main_occ)
+
 {-
 ************************************************************************
 *                                                                      *
@@ -901,13 +1299,19 @@ instance Binary NameSpace where
             putByte bh 2
     put_ bh TcClsName =
             putByte bh 3
+    put_ bh (FldName parent) = do
+            putByte bh 4
+            put_ bh parent
     get bh = do
             h <- getByte bh
             case h of
               0 -> return VarName
               1 -> return DataName
               2 -> return TvName
-              _ -> return TcClsName
+              3 -> return TcClsName
+              _ -> do
+                parent <- get bh
+                return $ FldName { fldParent = parent }
 
 instance Binary OccName where
     put_ bh (OccName aa ab) = do
