@@ -356,11 +356,11 @@ tyThingGREInfo = \case
             RecSelData   tc ->
               let dcs = map RealDataCon $ tyConDataCons tc in
               case conLikesWithFields dcs [flLabel fl] of
-                [] -> pprPanic "tyThingGREInfo: no DataCons with this FieldLabel" $
+                ([], _) -> pprPanic "tyThingGREInfo: no DataCons with this FieldLabel" $
                         vcat [ text "id:"  <+> ppr id
                              , text "fl:"  <+> ppr fl
                              , text "dcs:" <+> ppr dcs ]
-                cons -> mkUniqSet $ map conLikeConLikeName cons
+                (cons, _) -> mkUniqSet $ map conLikeConLikeName cons
        in IAmRecField $
             RecFieldInfo
               { recFieldLabel = fl
