@@ -1,18 +1,9 @@
 
 {-# LANGUAGE CPP #-}
-{-# LANGUAGE FlexibleInstances #-}
-{-# LANGUAGE PolyKinds #-}
-{-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE GADTs #-}
-{-# LANGUAGE BangPatterns #-}
-{-# LANGUAGE StandaloneDeriving #-}
-{-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE UnboxedTuples #-}
 
 {-# OPTIONS_GHC -O2 -funbox-strict-fields #-}
-#if MIN_VERSION_base(4,16,0)
-#define HAS_TYPELITCHAR
-#endif
 -- We always optimise this, otherwise performance of a non-optimised
 -- compiler is severely affected
 
@@ -56,6 +47,8 @@ module GHC.Utils.Binary
    -- * For writing instances
    putByte,
    getByte,
+   putByteString,
+   getByteString,
 
    -- * Variable length encodings
    putULEB128,
@@ -97,6 +90,7 @@ import GHC.Utils.Fingerprint
 import GHC.Types.SrcLoc
 import GHC.Types.Unique
 import qualified GHC.Data.Strict as Strict
+import GHC.Utils.Outputable( JoinPointHood(..) )
 
 import Control.DeepSeq
 import Foreign hiding (shiftL, shiftR, void)
@@ -606,9 +600,9 @@ getSLEB128 bh = do
 -- is to the interface file without the variable length encoding we usually
 -- apply.
 
--- | Encode the argument in it's full length. This is different from many default
+-- | Encode the argument in its full length. This is different from many default
 -- binary instances which make no guarantee about the actual encoding and
--- might do things use variable length encoding.
+-- might do things using variable length encoding.
 newtype FixedLengthEncoding a
   = FixedLengthEncoding { unFixedLength :: a }
   deriving (Eq,Ord,Show)
@@ -812,6 +806,17 @@ instance Binary DiffTime where
     get bh = do r <- get bh
                 return $ fromRational r
 
+instance Binary JoinPointHood where
+    put_ bh NotJoinPoint = putByte bh 0
+    put_ bh (JoinPoint ar) = do
+        putByte bh 1
+        put_ bh ar
+    get bh = do
+        h <- getByte bh
+        case h of
+            0 -> return NotJoinPoint
+            _ -> do { ar <- get bh; return (JoinPoint ar) }
+
 {-
 Finally - a reasonable portable Integer instance.
 
@@ -822,11 +827,11 @@ we stored a tag byte to discriminate between the two cases.
 This made some sense as it's highly portable but also not very
 efficient.
 
-However GHC stores a surprisingly large number off large Integer
+However GHC stores a surprisingly large number of large Integer
 values. In the examples looked at between 25% and 50% of Integers
 serialized were outside of the Int32 range.
 
-Consider a valie like `2724268014499746065`, some sort of hash
+Consider a value like `2724268014499746065`, some sort of hash
 actually generated by GHC.
 In the old scheme this was encoded as a list of 19 chars. This
 gave a size of 77 Bytes, one for the length of the list and 76
@@ -1227,6 +1232,19 @@ getFS bh = do
   l  <- get bh :: IO Int
   getPrim bh l (\src -> pure $! mkFastStringBytes src l )
 
+-- | Put a ByteString without its length (can't be read back without knowing the
+-- length!)
+putByteString :: BinHandle -> ByteString -> IO ()
+putByteString bh bs =
+  BS.unsafeUseAsCStringLen bs $ \(ptr, l) -> do
+    putPrim bh l (\op -> copyBytes op (castPtr ptr) l)
+
+-- | Get a ByteString whose length is known
+getByteString :: BinHandle -> Int -> IO ByteString
+getByteString bh l =
+  BS.create l $ \dest -> do
+    getPrim bh l (\src -> copyBytes dest src l)
+
 putBS :: BinHandle -> ByteString -> IO ()
 putBS bh bs =
   BS.unsafeUseAsCStringLen bs $ \(ptr, l) -> do
