@@ -447,6 +447,7 @@ type family XStatic         x
 type family XTick           x
 type family XBinTick        x
 type family XPragE          x
+type family XEmbTy          x
 type family XXExpr          x
 
 -- -------------------------------------
@@ -606,6 +607,8 @@ type family XLitPat      x
 type family XNPat        x
 type family XNPlusKPat   x
 type family XSigPat      x
+type family XEmbTyPat    x
+type family XInvisPat    x
 type family XCoPat       x
 type family XXPat        x
 type family XHsFieldBind x
@@ -640,6 +643,11 @@ type family XXHsWildCardBndrs  x b
 type family XHsPS x
 type family XXHsPatSigType x
 
+-- -------------------------------------
+-- HsTyPat type families
+type family XHsTP x
+type family XXHsTyPat x
+
 -- -------------------------------------
 -- HsType type families
 type family XForAllTy        x
