@@ -10,5 +10,5 @@ int main(int argc, char** argv) {
     binDir = getExecutablePath();
     exePath = mkString("%s/haddock.exe", binDir);
 
-    run(exePath, 0, NULL, argc - 1, argv + 1);
+    run(exePath, 0, NULL, argc - 1, argv + 1, NULL);
 }
