@@ -1,4 +1,6 @@
+{-# OPTIONS_GHC -Wno-orphans #-} -- Orphan instances for Toolchain.Target
 {-# LANGUAGE TypeFamilies #-}
+{-# LANGUAGE StandaloneDeriving #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module     : Hadrian.Oracles.TextFile
@@ -12,7 +14,9 @@
 -----------------------------------------------------------------------------
 module Hadrian.Oracles.TextFile (
     lookupValue, lookupValueOrEmpty, lookupValueOrError, lookupSystemConfig, lookupValues,
-    lookupValuesOrEmpty, lookupValuesOrError, lookupDependencies, textFileOracle
+    lookupValuesOrEmpty, lookupValuesOrError, lookupDependencies, textFileOracle,
+    getBuildTarget, getHostTarget, getTargetTarget,
+    queryBuildTarget, queryHostTarget, queryTargetTarget
     ) where
 
 import Control.Monad
@@ -23,6 +27,9 @@ import Development.Shake
 import Development.Shake.Classes
 import Development.Shake.Config
 import Base
+import Text.Read (readMaybe)
+
+import qualified GHC.Toolchain.Target as Toolchain
 
 -- | Lookup a value in a text file, tracking the result. Each line of the file
 -- is expected to have @key = value@ format.
@@ -75,10 +82,41 @@ lookupDependencies depFile file = do
           | otherwise            = 1
     deps <- fmap (sortOn weigh) <$> lookupValues depFile file
     case deps of
-        Nothing -> error $ "No dependencies found for file " ++ quote file
-        Just [] -> error $ "No source file found for file " ++ quote file
+        Nothing -> error $ "No dependencies found for file " ++ quote file ++ " in " ++ quote depFile
+        Just [] -> error $ "No source file found for file " ++ quote file ++ " in " ++ quote depFile
         Just (source : files) -> return (source, files)
 
+-- | Parse a target from a text file, tracking the result. The file is expected
+-- to contain a parseable Toolchain.Target value generated by ghc-toolchain.
+getTargetConfig :: FilePath -> Action Toolchain.Target
+getTargetConfig file0 = do
+  useGhcToolchain <- lookupSystemConfig "use-ghc-toolchain"
+  let file1 = if useGhcToolchain == "YES" then file0 <.> "ghc-toolchain" else file0
+  askOracle $ TargetFile file1
+
+-- | Get the build target configuration through 'getTargetConfig'
+getBuildTarget :: Action Toolchain.Target
+getBuildTarget = getTargetConfig buildTargetFile
+
+-- | Get the host target configuration through 'getTargetConfig'
+getHostTarget :: Action Toolchain.Target
+getHostTarget = getTargetConfig hostTargetFile
+  -- where
+  --   msg = "The host's target configuration file " ++ quote hostTargetFile ++ " does not exist! ghc-toolchain might have failed to generate it."
+
+-- | Get the target target configuration through 'getTargetConfig'
+getTargetTarget :: Action Toolchain.Target
+getTargetTarget = getTargetConfig targetTargetFile
+
+queryBuildTarget :: (Toolchain.Target -> a) -> Action a
+queryBuildTarget f = f <$> getBuildTarget
+
+queryHostTarget :: (Toolchain.Target -> a) -> Action a
+queryHostTarget f = f <$> getHostTarget
+
+queryTargetTarget :: (Toolchain.Target -> a) -> Action a
+queryTargetTarget f = f <$> getTargetTarget
+
 newtype KeyValue = KeyValue (FilePath, String)
     deriving (Binary, Eq, Hashable, NFData, Show, Typeable)
 type instance RuleResult KeyValue = Maybe String
@@ -87,6 +125,10 @@ newtype KeyValues = KeyValues (FilePath, String)
     deriving (Binary, Eq, Hashable, NFData, Show, Typeable)
 type instance RuleResult KeyValues = Maybe [String]
 
+newtype TargetFile = TargetFile FilePath
+    deriving (Binary, Eq, Hashable, NFData, Show, Typeable)
+type instance RuleResult TargetFile = Toolchain.Target
+
 -- | These oracle rules are used to cache and track answers to the following
 -- queries, which are implemented by parsing text files:
 --
@@ -97,6 +139,10 @@ type instance RuleResult KeyValues = Maybe [String]
 --
 -- 2) Parsing Makefile dependency files generated by commands like @gcc -MM@:
 --    see 'lookupDependencies'.
+--
+-- 3) Parsing target files as generated by ghc-toolchain. See functions
+--    'lookupTarget' and lookupTargetConfig'
+--
 textFileOracle :: Rules ()
 textFileOracle = do
     kv <- newCache $ \file -> do
@@ -111,3 +157,33 @@ textFileOracle = do
         contents <- map words <$> readFileLines file
         return $ Map.fromList [ (key, values) | (key:values) <- contents ]
     void $ addOracleCache $ \(KeyValues (file, key)) -> Map.lookup key <$> kvs file
+
+    tf <- newCache $ \file -> do
+        need [file]
+        putVerbose $ "| TargetFile oracle: reading " ++ quote file ++ "..."
+        mtarget <- readMaybe <$> readFile' file
+        case mtarget of
+          Nothing -> error $ unlines ["Error parsing a Toolchain.Target from " ++ quote file,
+                                      "Perhaps the `.target` file is out of date.",
+                                      "Try re-running `./configure`."
+                                     ]
+
+          Just target -> return (target :: Toolchain.Target)
+    void $ addOracleCache $ \(TargetFile file) -> tf file
+
+-- Orphan instances for (ShakeValue Toolchain.Target)
+instance Binary Toolchain.Target where
+  put = put . show
+  get = fromMaybe (error $ unlines ["Error parsing a toolchain `.target` file from its binary representation in hadrian.",
+                                    "This is likely caused by a stale hadrian/shake cache",
+                                    "which has saved an old `.target` file that can't be parsed",
+                                    "into a more recent `Toolchain.Target`. It is recommended to reset",
+                                    "by running `./hadrian/build clean`."
+                                   ]) . readMaybe <$> get
+
+instance Hashable Toolchain.Target where
+  hashWithSalt s = hashWithSalt s . show
+
+instance NFData Toolchain.Target where
+  rnf = flip seq () -- ROMES:TODO: Would be better to have a correct instance, perhaps we could instance NFData in ghc-toolchain?
+
