@@ -86,10 +86,9 @@ multiSetup pkg_s = do
       let rexp m = ["-reexported-module", m]
       let hidir = root </> "interfaces" </> pkgPath p
       writeFile' (resp_file root p) (intercalate "\n" (th_hack arg_list
-                                                      ++ modules cd
+                                                      ++  modules cd
                                                       ++ concatMap rexp (reexportModules cd)
-                                                      ++ ["-outputdir", hidir,
-                                                          "-this-package-name", pkgName p]))
+                                                      ++ ["-outputdir", hidir]))
       return (resp_file root p)
 
 
@@ -155,10 +154,14 @@ toolTargets = [ binary
               , process
               , exceptions
               , filepath
+              , osString
               -- , ghc     -- # depends on ghc library
               -- , runGhc  -- # depends on ghc library
               , ghcBoot
               , ghcBootTh
+              , ghcPlatform
+              , ghcToolchain
+              , ghcToolchainBin
               , ghcHeap
               , ghci
               , ghcPkg  -- # executable
