@@ -25,8 +25,7 @@ toolArgs = do
   builder (Ghc ToolArgs) ? mconcat
               [ packageGhcArgs
               , includeGhcArgs
-              , map ("-optc" ++) <$> getStagedSettingList ConfCcArgs
-              , map ("-optP" ++) <$> getStagedSettingList ConfCppArgs
+              , map ("-optc" ++) <$> getStagedCCFlags
               , map ("-optP" ++) <$> getContextData cppOpts
               , getContextData hcOpts
               ]
@@ -58,7 +57,7 @@ compileC :: Args
 compileC = builder (Ghc CompileCWithGhc) ? do
     way <- getWay
     let ccArgs = [ getContextData ccOpts
-                 , getStagedSettingList ConfCcArgs
+                 , getStagedCCFlags
                  , cIncludeArgs
                  , Dynamic `wayUnit` way ? pure [ "-fPIC", "-DDYNAMIC" ] ]
     mconcat [ arg "-Wall"
@@ -75,7 +74,7 @@ compileCxx :: Args
 compileCxx = builder (Ghc CompileCppWithGhc) ? do
     way <- getWay
     let ccArgs = [ getContextData cxxOpts
-                 , getStagedSettingList ConfCcArgs
+                 , getStagedCCFlags
                  , cIncludeArgs
                  , Dynamic `wayUnit` way ? pure [ "-fPIC", "-DDYNAMIC" ] ]
     mconcat [ arg "-Wall"
@@ -99,9 +98,7 @@ ghcLinkArgs = builder (Ghc LinkHs) ? do
     -- Relative path from the output (rpath $ORIGIN).
     originPath <- dropFileName <$> getOutput
     context <- getContext
-    libPath' <- expr (libPath context)
-    st <- getStage
-    distDir <- expr (Context.distDir st)
+    distPath <- expr (Context.distDynDir context)
 
     useSystemFfi <- expr (flag UseSystemFfi)
     buildPath <- getBuildPath
@@ -113,7 +110,6 @@ ghcLinkArgs = builder (Ghc LinkHs) ? do
 
     let
         dynamic = Dynamic `wayUnit` way
-        distPath = libPath' -/- distDir
         originToLibsDir = makeRelativeNoSysLink originPath distPath
         rpath
             -- Programs will end up in the bin dir ($ORIGIN) and will link to
@@ -205,8 +201,7 @@ commonGhcArgs = do
             -- to the @ghc-version@ file, to prevent GHC from trying to open the
             -- RTS package in the package database and failing.
             , package rts ? notStage0 ? arg "-ghcversion-file=rts/include/ghcversion.h"
-            , map ("-optc" ++) <$> getStagedSettingList ConfCcArgs
-            , map ("-optP" ++) <$> getStagedSettingList ConfCppArgs
+            , map ("-optc" ++) <$> getStagedCCFlags
             , map ("-optP" ++) <$> getContextData cppOpts
             , arg "-outputdir", arg path
               -- we need to enable color explicitly because the output is
@@ -229,11 +224,14 @@ wayGhcArgs = do
     mconcat [ if Dynamic `wayUnit` way
                 then pure ["-fPIC", "-dynamic"]
                 else arg "-static"
-            , (Threaded  `wayUnit` way) ? arg "-optc-DTHREADED_RTS"
-            , (Debug     `wayUnit` way) ? arg "-optc-DDEBUG"
             , (Profiling `wayUnit` way) ? arg "-prof"
-            , (way == debug || way == debugDynamic) ?
-              pure ["-ticky", "-DTICKY_TICKY"] ]
+            , (way == debug || way == debugDynamic) ? arg "-ticky"
+            , wayCcArgs
+              -- We must pass CPP flags via -optc as well to ensure that they
+              -- are passed to the preprocessor when, e.g., compiling Cmm
+              -- sources.
+            , map ("-optc"++) <$> wayCcArgs
+            ]
 
 -- | Args related to correct handling of packages, such as setting
 -- -this-unit-id and passing -package-id for dependencies
@@ -253,13 +251,17 @@ packageGhcArgs = do
     -- sets `-this-unit-id ghc` when hadrian is building stage0, which will
     -- overwrite this one.
     pkgId   <- expr $ pkgUnitId stage package
+    pkgName <- expr $ pkgPackageName package
     mconcat [ arg "-hide-all-packages"
             , arg "-no-user-package-db"
             , arg "-package-env -"
             , packageDatabaseArgs
             -- We want to pass -this-unit-id for executables as well for multi-repl to
             -- work with executable packages but this is buggy on GHC-9.0.2
-            , (isLibrary package || (ghc_ver >= makeVersion [9,2,1])) ?  arg ("-this-unit-id " ++ pkgId)
+            , (isLibrary package || (ghc_ver >= makeVersion [9,2,1])) ? mconcat
+                [ arg ("-this-unit-id " ++ pkgId)
+                , arg ("-this-package-name " ++ pkgName)
+                ]
             , map ("-package-id " ++) <$> getContextData depIds ]
 
 includeGhcArgs :: Args
