@@ -12,13 +12,16 @@ import Packages
 -- get a context/contexts for a given stage and package.
 programContext :: Stage -> Package -> Action Context
 programContext stage pkg = do
-    profiled <- askGhcProfiled
+    profiled <- askGhcProfiled stage
     dynGhcProgs <- askDynGhcPrograms --dynamicGhcPrograms =<< flavour
-    return $ Context stage pkg (wayFor profiled dynGhcProgs)
+    return $ Context stage pkg (wayFor profiled dynGhcProgs) Final
 
     where wayFor prof dyn
             | prof && dyn                          =
                 error "programContext: profiling+dynamic not supported"
-            | pkg == ghc && prof && stage > Stage0 = profiling
-            | dyn && stage > Stage0                = dynamic
+            | pkg == ghc && prof && notStage0 stage = profiling
+            | dyn && notStage0 stage                = dynamic
             | otherwise                            = vanilla
+
+          notStage0 (Stage0 {}) = False
+          notStage0 _ = True
