@@ -1,4 +1,5 @@
 -----------------------------------------------------------------------------
+
 -- |
 -- Module      :  Distribution.Compat.DList
 -- Copyright   :  (c) Ben Gamari 2015-2019
@@ -9,18 +10,18 @@
 -- Portability :  portable
 --
 -- A very simple difference list.
-module Distribution.Compat.DList (
-    DList,
-    runDList,
-    empty,
-    singleton,
-    fromList,
-    toList,
-    snoc,
-) where
+module Distribution.Compat.DList
+  ( DList
+  , runDList
+  , empty
+  , singleton
+  , fromList
+  , toList
+  , snoc
+  ) where
 
+import Distribution.Compat.Prelude hiding (empty, toList)
 import Prelude ()
-import Distribution.Compat.Prelude hiding (toList, empty)
 
 -- | Difference list.
 newtype DList a = DList ([a] -> [a])
@@ -30,7 +31,7 @@ runDList (DList run) = run []
 
 -- | Make 'DList' containing single element.
 singleton :: a -> DList a
-singleton a = DList (a:)
+singleton a = DList (a :)
 
 -- | @since 3.4.0.0
 empty :: DList a
@@ -46,7 +47,7 @@ snoc :: DList a -> a -> DList a
 snoc xs x = xs <> singleton x
 
 instance Monoid (DList a) where
-  mempty  = empty
+  mempty = empty
   mappend = (<>)
 
 instance Semigroup (DList a) where
