@@ -1,10 +1,11 @@
 {-# LANGUAGE DeriveDataTypeable #-}
-{-# LANGUAGE DeriveGeneric #-}
-{-# LANGUAGE DeriveFunctor #-}
 {-# LANGUAGE DeriveFoldable #-}
+{-# LANGUAGE DeriveFunctor #-}
+{-# LANGUAGE DeriveGeneric #-}
 {-# LANGUAGE DeriveTraversable #-}
 
 -----------------------------------------------------------------------------
+
 -- |
 -- Module      :  Distribution.Compiler
 -- Copyright   :  Isaac Jones 2003-2004
@@ -26,44 +27,55 @@
 -- 'UserHooks' api, which would break all custom @Setup.hs@ files, so for the
 -- moment we just have to live with this deficiency. If you're interested, see
 -- ticket #57.
+module Distribution.Compiler
+  ( -- * Compiler flavor
+    CompilerFlavor (..)
+  , buildCompilerId
+  , buildCompilerFlavor
+  , defaultCompilerFlavor
+  , classifyCompilerFlavor
+  , knownCompilerFlavors
 
-module Distribution.Compiler (
-  -- * Compiler flavor
-  CompilerFlavor(..),
-  buildCompilerId,
-  buildCompilerFlavor,
-  defaultCompilerFlavor,
-  classifyCompilerFlavor,
-  knownCompilerFlavors,
+    -- * Per compiler flavor
+  , PerCompilerFlavor (..)
+  , perCompilerFlavorToList
 
-  -- * Per compiler flavor
-  PerCompilerFlavor (..),
-  perCompilerFlavorToList,
+    -- * Compiler id
+  , CompilerId (..)
 
-  -- * Compiler id
-  CompilerId(..),
-
-  -- * Compiler info
-  CompilerInfo(..),
-  unknownCompilerInfo,
-  AbiTag(..), abiTagString
+    -- * Compiler info
+  , CompilerInfo (..)
+  , unknownCompilerInfo
+  , AbiTag (..)
+  , abiTagString
   ) where
 
-import Prelude ()
 import Distribution.Compat.Prelude
+import Prelude ()
 
 import Language.Haskell.Extension
 
 import Distribution.Version (Version, mkVersion', nullVersion)
 
-import qualified System.Info (compilerName, compilerVersion)
+import qualified Distribution.Compat.CharParsing as P
 import Distribution.Parsec (Parsec (..))
 import Distribution.Pretty (Pretty (..), prettyShow)
-import qualified Distribution.Compat.CharParsing as P
+import qualified System.Info (compilerName, compilerVersion)
 import qualified Text.PrettyPrint as Disp
 
-data CompilerFlavor =
-  GHC | GHCJS | NHC | YHC | Hugs | HBC | Helium | JHC | LHC | UHC | Eta
+data CompilerFlavor
+  = GHC
+  | GHCJS
+  | NHC
+  | YHC
+  | Hugs
+  | HBC
+  | Helium
+  | JHC
+  | LHC
+  | UHC
+  | Eta
+  | MHS -- MicroHS, see https://github.com/augustss/MicroHs
   | HaskellSuite String -- string is the id of the actual compiler
   | OtherCompiler String
   deriving (Generic, Show, Read, Eq, Ord, Typeable, Data)
@@ -74,27 +86,29 @@ instance NFData CompilerFlavor where rnf = genericRnf
 
 knownCompilerFlavors :: [CompilerFlavor]
 knownCompilerFlavors =
-  [GHC, GHCJS, NHC, YHC, Hugs, HBC, Helium, JHC, LHC, UHC, Eta]
+  [GHC, GHCJS, NHC, YHC, Hugs, HBC, Helium, JHC, LHC, UHC, Eta, MHS]
 
 instance Pretty CompilerFlavor where
   pretty (OtherCompiler name) = Disp.text name
-  pretty (HaskellSuite name)  = Disp.text name
-  pretty NHC                  = Disp.text "nhc98"
-  pretty other                = Disp.text (lowercase (show other))
+  pretty (HaskellSuite name) = Disp.text name
+  pretty NHC = Disp.text "nhc98"
+  pretty other = Disp.text (lowercase (show other))
 
 instance Parsec CompilerFlavor where
-    parsec = classifyCompilerFlavor <$> component
-      where
-        component = do
-          cs <- P.munch1 isAlphaNum
-          if all isDigit cs then fail "all digits compiler name" else return cs
+  parsec = classifyCompilerFlavor <$> component
+    where
+      component = do
+        cs <- P.munch1 isAlphaNum
+        if all isDigit cs then fail "all digits compiler name" else return cs
 
 classifyCompilerFlavor :: String -> CompilerFlavor
 classifyCompilerFlavor s =
   fromMaybe (OtherCompiler s) $ lookup (lowercase s) compilerMap
   where
-    compilerMap = [ (lowercase (prettyShow compiler), compiler)
-                  | compiler <- knownCompilerFlavors ]
+    compilerMap =
+      [ (lowercase (prettyShow compiler), compiler)
+      | compiler <- knownCompilerFlavors
+      ]
 
 buildCompilerFlavor :: CompilerFlavor
 buildCompilerFlavor = classifyCompilerFlavor System.Info.compilerName
@@ -110,11 +124,10 @@ buildCompilerId = CompilerId buildCompilerFlavor buildCompilerVersion
 --
 -- However if it's not a recognised compiler then it's 'Nothing' and the user
 -- will have to specify which compiler they want.
---
 defaultCompilerFlavor :: Maybe CompilerFlavor
 defaultCompilerFlavor = case buildCompilerFlavor of
   OtherCompiler _ -> Nothing
-  _               -> Just buildCompilerFlavor
+  _ -> Just buildCompilerFlavor
 
 -------------------------------------------------------------------------------
 -- Per compiler data
@@ -123,10 +136,19 @@ defaultCompilerFlavor = case buildCompilerFlavor of
 -- | 'PerCompilerFlavor' carries only info per GHC and GHCJS
 --
 -- Cabal parses only @ghc-options@ and @ghcjs-options@, others are omitted.
---
 data PerCompilerFlavor v = PerCompilerFlavor v v
-  deriving (Generic, Show, Read, Eq, Ord, Typeable, Data, Functor, Foldable
-           , Traversable)
+  deriving
+    ( Generic
+    , Show
+    , Read
+    , Eq
+    , Ord
+    , Typeable
+    , Data
+    , Functor
+    , Foldable
+    , Traversable
+    )
 
 instance Binary a => Binary (PerCompilerFlavor a)
 instance Structured a => Structured (PerCompilerFlavor a)
@@ -136,15 +158,19 @@ perCompilerFlavorToList :: PerCompilerFlavor v -> [(CompilerFlavor, v)]
 perCompilerFlavorToList (PerCompilerFlavor a b) = [(GHC, a), (GHCJS, b)]
 
 instance Semigroup a => Semigroup (PerCompilerFlavor a) where
-    PerCompilerFlavor a b <> PerCompilerFlavor a' b' = PerCompilerFlavor
-        (a <> a') (b <> b')
+  PerCompilerFlavor a b <> PerCompilerFlavor a' b' =
+    PerCompilerFlavor
+      (a <> a')
+      (b <> b')
 
 instance (Semigroup a, Monoid a) => Monoid (PerCompilerFlavor a) where
-    mempty = PerCompilerFlavor mempty mempty
-    mappend = (<>)
+  mempty = PerCompilerFlavor mempty mempty
+  mappend = (<>)
 
 -- ------------------------------------------------------------
+
 -- * Compiler Id
+
 -- ------------------------------------------------------------
 
 data CompilerId = CompilerId CompilerFlavor Version
@@ -157,7 +183,7 @@ instance NFData CompilerId where rnf = genericRnf
 instance Pretty CompilerId where
   pretty (CompilerId f v)
     | v == nullVersion = pretty f
-    | otherwise        = pretty f <<>> Disp.char '-' <<>> pretty v
+    | otherwise = pretty f <<>> Disp.char '-' <<>> pretty v
 
 instance Parsec CompilerId where
   parsec = do
@@ -169,28 +195,29 @@ lowercase :: String -> String
 lowercase = map toLower
 
 -- ------------------------------------------------------------
+
 -- * Compiler Info
+
 -- ------------------------------------------------------------
 
 -- | Compiler information used for resolving configurations. Some
 --   fields can be set to Nothing to indicate that the information is
 --   unknown.
-
-data CompilerInfo = CompilerInfo {
-         compilerInfoId         :: CompilerId,
-         -- ^ Compiler flavour and version.
-         compilerInfoAbiTag     :: AbiTag,
-         -- ^ Tag for distinguishing incompatible ABI's on the same
-         -- architecture/os.
-         compilerInfoCompat     :: Maybe [CompilerId],
-         -- ^ Other implementations that this compiler claims to be
-         -- compatible with, if known.
-         compilerInfoLanguages  :: Maybe [Language],
-         -- ^ Supported language standards, if known.
-         compilerInfoExtensions :: Maybe [Extension]
-         -- ^ Supported extensions, if known.
-     }
-     deriving (Generic, Show, Read)
+data CompilerInfo = CompilerInfo
+  { compilerInfoId :: CompilerId
+  -- ^ Compiler flavour and version.
+  , compilerInfoAbiTag :: AbiTag
+  -- ^ Tag for distinguishing incompatible ABI's on the same
+  -- architecture/os.
+  , compilerInfoCompat :: Maybe [CompilerId]
+  -- ^ Other implementations that this compiler claims to be
+  -- compatible with, if known.
+  , compilerInfoLanguages :: Maybe [Language]
+  -- ^ Supported language standards, if known.
+  , compilerInfoExtensions :: Maybe [Extension]
+  -- ^ Supported extensions, if known.
+  }
+  deriving (Generic, Show, Read)
 
 instance Binary CompilerInfo
 
@@ -203,7 +230,7 @@ instance Binary AbiTag
 instance Structured AbiTag
 
 instance Pretty AbiTag where
-  pretty NoAbiTag     = Disp.empty
+  pretty NoAbiTag = Disp.empty
   pretty (AbiTag tag) = Disp.text tag
 
 instance Parsec AbiTag where
@@ -212,7 +239,7 @@ instance Parsec AbiTag where
     if null tag then return NoAbiTag else return (AbiTag tag)
 
 abiTagString :: AbiTag -> String
-abiTagString NoAbiTag     = ""
+abiTagString NoAbiTag = ""
 abiTagString (AbiTag tag) = tag
 
 -- | Make a CompilerInfo of which only the known information is its CompilerId,
