@@ -1,7 +1,8 @@
 {-# LANGUAGE DeriveDataTypeable #-}
-{-# LANGUAGE DeriveGeneric      #-}
+{-# LANGUAGE DeriveGeneric #-}
+
 module Distribution.Types.Dependency
-  ( Dependency(..)
+  ( Dependency (..)
   , mkDependency
   , depPkgName
   , depVerRange
@@ -14,11 +15,11 @@ import Distribution.Compat.Prelude
 import Prelude ()
 
 import Distribution.Types.VersionRange (isAnyVersionLight)
-import Distribution.Version            (VersionRange, anyVersion, simplifyVersionRange)
+import Distribution.Version (VersionRange, anyVersion, simplifyVersionRange)
 
 import Distribution.CabalSpecVersion
-import Distribution.Compat.CharParsing        (char, spaces)
-import Distribution.Compat.Parsing            (between, option)
+import Distribution.Compat.CharParsing (char, spaces)
+import Distribution.Compat.Parsing (between, option)
 import Distribution.Parsec
 import Distribution.Pretty
 import Distribution.Types.LibraryName
@@ -26,7 +27,7 @@ import Distribution.Types.PackageName
 import Distribution.Types.UnqualComponentName
 
 import qualified Distribution.Compat.NonEmptySet as NES
-import qualified Text.PrettyPrint                as PP
+import qualified Text.PrettyPrint as PP
 
 -- | Describes a dependency on a source package (API)
 --
@@ -36,15 +37,15 @@ import qualified Text.PrettyPrint                as PP
 -- /Note:/ 'Dependency' is not an instance of 'Ord', and so it cannot be used
 -- in 'Set' or as the key to a 'Map'.  For these and similar use cases see
 -- 'DependencyMap'.
---
-data Dependency = Dependency
-                    PackageName
-                    VersionRange
-                    (NonEmptySet LibraryName)
-                    -- ^ The set of libraries required from the package.
-                    -- Only the selected libraries will be built.
-                    -- It does not affect the cabal-install solver yet.
-                  deriving (Generic, Read, Show, Eq, Ord, Typeable, Data)
+data Dependency
+  = -- | The set of libraries required from the package.
+    -- Only the selected libraries will be built.
+    -- It does not affect the cabal-install solver yet.
+    Dependency
+      PackageName
+      VersionRange
+      (NonEmptySet LibraryName)
+  deriving (Generic, Read, Show, Eq, Ord, Typeable, Data)
 
 depPkgName :: Dependency -> PackageName
 depPkgName (Dependency pn _ _) = pn
@@ -61,15 +62,15 @@ depLibraries (Dependency _ _ cs) = cs
 -- it is automatically converted to 'LMainLibName'.
 --
 -- @since 3.4.0.0
---
 mkDependency :: PackageName -> VersionRange -> NonEmptySet LibraryName -> Dependency
 mkDependency pn vr lb = Dependency pn vr (NES.map conv lb)
   where
     pn' = packageNameToUnqualComponentName pn
 
-    conv l@LMainLibName                 = l
-    conv l@(LSubLibName ln) | ln == pn' = LMainLibName
-                            | otherwise = l
+    conv l@LMainLibName = l
+    conv l@(LSubLibName ln)
+      | ln == pn' = LMainLibName
+      | otherwise = l
 
 instance Binary Dependency
 instance Structured Dependency
@@ -88,23 +89,23 @@ instance NFData Dependency where rnf = genericRnf
 --
 -- >>> prettyShow $ Dependency (mkPackageName "pkg") anyVersion $ NES.insert (LSubLibName $ mkUnqualComponentName "sublib-b") $ NES.singleton (LSubLibName $ mkUnqualComponentName "sublib-a")
 -- "pkg:{sublib-a, sublib-b}"
---
 instance Pretty Dependency where
-    pretty (Dependency name ver sublibs) = withSubLibs (pretty name) <+> pver
-      where
-        -- TODO: change to isAnyVersion after #6736
-        pver | isAnyVersionLight ver = PP.empty
-             | otherwise             = pretty ver
+  pretty (Dependency name ver sublibs) = withSubLibs (pretty name) <+> pver
+    where
+      -- TODO: change to isAnyVersion after #6736
+      pver
+        | isAnyVersionLight ver = PP.empty
+        | otherwise = pretty ver
 
-        withSubLibs doc = case NES.toList sublibs of
-            [LMainLibName]   -> doc
-            [LSubLibName uq] -> doc <<>> PP.colon <<>> pretty uq
-            _                -> doc <<>> PP.colon <<>> PP.braces prettySublibs
+      withSubLibs doc = case NES.toList sublibs of
+        [LMainLibName] -> doc
+        [LSubLibName uq] -> doc <<>> PP.colon <<>> pretty uq
+        _ -> doc <<>> PP.colon <<>> PP.braces prettySublibs
 
-        prettySublibs = PP.hsep $ PP.punctuate PP.comma $ prettySublib <$> NES.toList sublibs
+      prettySublibs = PP.hsep $ PP.punctuate PP.comma $ prettySublib <$> NES.toList sublibs
 
-        prettySublib LMainLibName     = PP.text $ unPackageName name
-        prettySublib (LSubLibName un) = PP.text $ unUnqualComponentName un
+      prettySublib LMainLibName = PP.text $ unPackageName name
+      prettySublib (LSubLibName un) = PP.text $ unUnqualComponentName un
 
 -- |
 --
@@ -137,36 +138,37 @@ instance Pretty Dependency where
 --
 -- >>> map (`simpleParsec'` "mylib:sub") [CabalSpecV2_4, CabalSpecV3_0] :: [Maybe Dependency]
 -- [Nothing,Just (Dependency (PackageName "mylib") (OrLaterVersion (mkVersion [0])) (fromNonEmpty (LSubLibName (UnqualComponentName "sub") :| [])))]
---
 instance Parsec Dependency where
-    parsec = do
-        name <- parsec
+  parsec = do
+    name <- parsec
 
-        libs <- option mainLibSet $ do
-          _ <- char ':'
-          versionGuardMultilibs
-          NES.singleton <$> parseLib <|> parseMultipleLibs
+    libs <- option mainLibSet $ do
+      _ <- char ':'
+      versionGuardMultilibs
+      NES.singleton <$> parseLib <|> parseMultipleLibs
 
-        spaces -- https://github.com/haskell/cabal/issues/5846
-
-        ver  <- parsec <|> pure anyVersion
-        return $ mkDependency name ver libs
-      where
-        parseLib          = LSubLibName <$> parsec
-        parseMultipleLibs = between
-            (char '{' *> spaces)
-            (spaces *> char '}')
-            (NES.fromNonEmpty <$> parsecCommaNonEmpty parseLib)
+    spaces -- https://github.com/haskell/cabal/issues/5846
+    ver <- parsec <|> pure anyVersion
+    return $ mkDependency name ver libs
+    where
+      parseLib = LSubLibName <$> parsec
+      parseMultipleLibs =
+        between
+          (char '{' *> spaces)
+          (spaces *> char '}')
+          (NES.fromNonEmpty <$> parsecCommaNonEmpty parseLib)
 
 versionGuardMultilibs :: CabalParsing m => m ()
 versionGuardMultilibs = do
   csv <- askCabalSpecVersion
-  when (csv < CabalSpecV3_0) $ fail $ unwords
-    [ "Sublibrary dependency syntax used."
-    , "To use this syntax the package needs to specify at least 'cabal-version: 3.0'."
-    , "Alternatively, if you are depending on an internal library, you can write"
-    , "directly the library name as it were a package."
-    ]
+  when (csv < CabalSpecV3_0) $
+    fail $
+      unwords
+        [ "Sublibrary dependency syntax used."
+        , "To use this syntax the package needs to specify at least 'cabal-version: 3.0'."
+        , "Alternatively, if you are depending on an internal library, you can write"
+        , "directly the library name as it were a package."
+        ]
 
 -- | Library set with main library.
 --
@@ -176,7 +178,6 @@ mainLibSet = NES.singleton LMainLibName
 
 -- | Simplify the 'VersionRange' expression in a 'Dependency'.
 -- See 'simplifyVersionRange'.
---
 simplifyDependency :: Dependency -> Dependency
 simplifyDependency (Dependency name range comps) =
   Dependency name (simplifyVersionRange range) comps
