@@ -3,7 +3,6 @@ module Main
     ) where
 
 import Test.Tasty
-import Test.Tasty.ExpectedFailure
 import Test.Tasty.Golden.Advanced (goldenTest)
 
 import Data.Algorithm.Diff                    (PolyDiff (..), getGroupedDiff)
@@ -57,6 +56,7 @@ checkTests = testGroup "regressions"
     , checkTest "issue-7776-b.cabal"
     , checkTest "issue-7776-c.cabal"
     , checkTest "issue-8646.cabal"
+    , checkTest "decreasing-indentation.cabal"
     ]
 
 checkTest :: FilePath -> TestTree
@@ -70,7 +70,7 @@ checkTest fp = cabalGoldenTest fp correct $ do
             -- Note: parser warnings are reported by `cabal check`, but not by
             -- D.PD.Check functionality.
             unlines (map (showPWarning fp) ws) ++
-            unlines (map show (checkPackage gpd Nothing))
+            unlines (map show (checkPackage gpd))
         Left (_, errs) -> unlines $ map (("ERROR: " ++) . showPError fp) $ NE.toList errs
   where
     input = "tests" </> "ParserTests" </> "regressions" </> fp
