@@ -10,10 +10,8 @@ import Test.Tasty.HUnit              (testCase, (@?=), Assertion)
 import Distribution.SPDX.License       (License)
 import Distribution.Types.VersionRange (VersionRange)
 
-#if MIN_VERSION_base(4,7,0)
 import Distribution.Types.GenericPackageDescription (GenericPackageDescription)
 import Distribution.Types.LocalBuildInfo            (LocalBuildInfo)
-#endif
 
 import UnitTests.Orphans ()
 
@@ -25,16 +23,25 @@ tests = testGroup "Distribution.Utils.Structured"
     , testCase "SPDX.License" $
       md5Check (Proxy :: Proxy License) 0xd3d4a09f517f9f75bc3d16370d5a853a
     -- The difference is in encoding of newtypes
-#if MIN_VERSION_base(4,7,0)
-    , testCase "GenericPackageDescription" $
-      md5Check (Proxy :: Proxy GenericPackageDescription) 0xa3e9433662ecf0c7a3c26f6d75a53ba1
-    , testCase "LocalBuildInfo" $
-      md5Check (Proxy :: Proxy LocalBuildInfo) 0x91ffcd61bbd83525e8edba877435a031
-#endif
+    , testCase "GenericPackageDescription" $ md5CheckGenericPackageDescription (Proxy :: Proxy GenericPackageDescription)
+    , testCase "LocalBuildInfo" $ md5CheckLocalBuildInfo (Proxy :: Proxy LocalBuildInfo)
     ]
 
--- -------------------------------------------------------------------- --
--- utils
-
 md5Check :: Structured a => Proxy a -> Integer -> Assertion
 md5Check proxy md5Int = structureHash proxy @?= md5FromInteger md5Int
+
+md5CheckGenericPackageDescription :: Proxy GenericPackageDescription -> Assertion
+md5CheckGenericPackageDescription proxy = md5Check proxy
+#if MIN_VERSION_base(4,19,0)
+    0x4d8ea0a071f0d5a3aca3fbdfef91bae2
+#else
+    0x0e3685f57e01297a9aa9637729681b94
+#endif
+
+md5CheckLocalBuildInfo :: Proxy LocalBuildInfo -> Assertion
+md5CheckLocalBuildInfo proxy = md5Check proxy
+#if MIN_VERSION_base(4,19,0)
+    0x1be858ee00c3e2d4be5331d5f07bfdf7
+#else
+    0x8a5431ab053f8f48c15b303444fa2c39
+#endif
