@@ -1,8 +1,9 @@
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE DeriveGeneric #-}
-module Distribution.Types.GivenComponent (
-  GivenComponent(..)
-) where
+
+module Distribution.Types.GivenComponent
+  ( GivenComponent (..)
+  ) where
 
 import Distribution.Compat.Prelude
 
@@ -16,12 +17,12 @@ import Distribution.Types.PackageName
 -- It enables Cabal to know which 'ComponentId' to associate with a library
 --
 -- @since 2.3.0.0
-data GivenComponent =
-  GivenComponent
-    { givenComponentPackage :: PackageName
-    , givenComponentName    :: LibraryName -- --dependency is for libraries
-                                           -- only, not for any component
-    , givenComponentId      :: ComponentId }
+data GivenComponent = GivenComponent
+  { givenComponentPackage :: PackageName
+  , givenComponentName :: LibraryName -- --dependency is for libraries
+  -- only, not for any component
+  , givenComponentId :: ComponentId
+  }
   deriving (Generic, Read, Show, Eq, Typeable)
 
 instance Binary GivenComponent
