@@ -6,10 +6,12 @@ import qualified UnitTests.Distribution.Solver.Modular.MemoryUsage
 
 tests :: TestTree
 tests =
-  testGroup "Memory Usage"
-  [ testGroup "UnitTests.Distribution.Solver.Modular.MemoryUsage"
+  testGroup
+    "Memory Usage"
+    [ testGroup
+        "UnitTests.Distribution.Solver.Modular.MemoryUsage"
         UnitTests.Distribution.Solver.Modular.MemoryUsage.tests
-  ]
+    ]
 
 main :: IO ()
 main = defaultMain tests
