@@ -1,5 +1,5 @@
-module UnitTests.Distribution.Solver.Modular.Builder (
-  tests
+module UnitTests.Distribution.Solver.Modular.Builder
+  ( tests
   ) where
 
 import Distribution.Solver.Modular.Builder
@@ -8,13 +8,14 @@ import Test.Tasty
 import Test.Tasty.QuickCheck
 
 tests :: [TestTree]
-tests = [ testProperty "splitsAltImplementation" splitsTest
-        ]
+tests =
+  [ testProperty "splitsAltImplementation" splitsTest
+  ]
 
 -- | Simpler splits implementation
 splits' :: [a] -> [(a, [a])]
 splits' [] = []
-splits' (x : xs) = (x, xs) : map (\ (y, ys) -> (y, x : ys)) (splits' xs)
+splits' (x : xs) = (x, xs) : map (\(y, ys) -> (y, x : ys)) (splits' xs)
 
 splitsTest :: [Int] -> Property
 splitsTest xs = splits' xs === splits xs
