@@ -1,8 +1,8 @@
 import Test.Cabal.Prelude
 
 main = cabalTest $ do
-    skipUnless =<< ghcVersionIs (>= mkVersion [8,1])
-    skipIf =<< isWindows -- TODO: https://github.com/haskell/cabal/issues/6271
+    skipUnlessGhcVersion ">= 8.1"
+    skipIfWindows -- TODO: https://github.com/haskell/cabal/issues/6271
     withProjectFile "cabal.internal.project" $ do
         cabal "v2-build" ["exe"]
         withPlan $ do
