@@ -1,5 +1,5 @@
 import Test.Cabal.Prelude
 main = setupAndCabalTest $ do
     skipUnlessGhcVersion ">= 8.1"
-    r <- fails $ setup' "configure" ["--cabal-file", "Includes2.cabal.fail"]
+    r <- fails $ withDirectory "Includes2-fail" $ setup' "configure" []
     assertOutputContains "mysql" r
