@@ -1,8 +1,7 @@
 import Test.Cabal.Prelude
-main = withShorterPathForNewBuildStore $ \storeDir ->
-  cabalTest $ do
+main = cabalTest $ withShorterPathForNewBuildStore $ do
     skipUnlessGhcVersion ">= 8.1"
     skipIfWindows -- TODO: https://github.com/haskell/cabal/issues/6271
     withProjectFile "cabal.repo.project" $ do
       withRepo "repo" $ do
-        cabalG ["--store-dir=" ++ storeDir] "v2-build" ["exe"]
+        cabal "v2-build" ["exe"]
