@@ -10,16 +10,16 @@ import Test.Cabal.Prelude
 -- qualifier as pkg, even though they are both build targets of the project.
 -- The solution must use --independent-goals to give pkg and setup-dep different
 -- qualifiers.
-main = withShorterPathForNewBuildStore $ \storeDir ->
-  cabalTest $ do
-    skipUnless =<< hasNewBuildCompatBootCabal
+main =
+  cabalTest $ withShorterPathForNewBuildStore $ do
+    skipUnless "no v2-build compatible boot-Cabal" =<< hasNewBuildCompatBootCabal
     withRepo "repo" $ do
-      fails $ cabalG ["--store-dir=" ++ storeDir] "v2-build" ["pkg:my-exe", "--dry-run"]
+      fails $ cabal "v2-build" ["pkg:my-exe", "--dry-run"]
       -- Disabled recording because whether or not we get
       -- detailed information for the build of my-exe depends
       -- on whether or not the Cabal library version is recent
       -- enough
-      r1 <- recordMode DoNotRecord $ cabalG' ["--store-dir=" ++ storeDir] "v2-build" ["pkg:my-exe", "--independent-goals"]
+      r1 <- recordMode DoNotRecord $ cabal' "v2-build" ["pkg:my-exe", "--independent-goals"]
       assertOutputContains "Setup.hs: setup-dep from project" r1
       withPlan $ do
         r2 <- runPlanExe' "pkg" "my-exe" []
