@@ -7,7 +7,7 @@ import System.Environment
 -- Test PATH-munging
 -- TODO: Enable this test on Windows
 main = setupAndCabalTest $ do
-    skipIf =<< isWindows
+    skipIfWindows
     path <- liftIO $ getEnv "PATH"
     cwd <- testCurrentDir <$> getTestEnv
     r <- withEnv [("PATH", Just $ cwd ++ ":" ++ path)] $ setup_build []
