@@ -2,4 +2,4 @@ import Test.Cabal.Prelude
 
 -- `cxx-options`, do not use `-O1`.
 main = cabalTest $
-  fails $ cabal "check" []
+  cabal "check" []
