@@ -2,4 +2,4 @@ import Test.Cabal.Prelude
 
 -- `cmm-sources` and friends need ≥3.0.
 main = cabalTest $
-  fails $ cabal "check" []
+  cabal "check" []
