@@ -2,4 +2,4 @@ import Test.Cabal.Prelude
 
 -- Tricky option in `ghc-shared-options`.
 main = cabalTest $
-  fails $ cabal "check" []
+  cabal "check" []
