@@ -2,4 +2,4 @@ import Test.Cabal.Prelude
 
 -- Suspicious license version.
 main = cabalTest $
-  fails $ cabal "check" []
+  cabal "check" []
