@@ -1,7 +1,7 @@
 import Test.Cabal.Prelude
 import System.Directory (doesFileExist, removeDirectory)
 
-main = cabalTest . withRepo "repo" . withSourceCopy $ do
+main = cabalTest . withRepo "repo" $ do
     skipUnlessGhcVersion ">= 9.4.0"
     env <- getTestEnv
     let testDir = testCurrentDir env
