@@ -1,4 +1,4 @@
 import Test.Cabal.Prelude
-main = cabalTest $
+main = cabalTest $ do
     cabal' "v2-run" ["pkg-abc:program"] >>= assertOutputContains "pkg-def:publib"
-
+    return ()
