@@ -1,6 +1,5 @@
 import Test.Cabal.Prelude
-main = cabalTest $ do
-    withSourceCopy . withDelay $ do
+main = cabalTest $ withDelay $ do
         copySourceFileTo "q/q-broken.cabal.in" "q/q.cabal"
         fails $ cabal "v2-build" ["q"]
         delay
