@@ -1,4 +1,4 @@
 import Test.Cabal.Prelude
 import Data.List
-main = cabalTest $ withSourceCopy $
+main = cabalTest $
   cabal "v2-sdist" ["a", "b", "--list-only"]
