@@ -1,5 +1,5 @@
 import Test.Cabal.Prelude
-main = cabalTest $ withSourceCopy $ do
+main = cabalTest $ do
   cwd <- fmap testCurrentDir getTestEnv
   fails $ cabal "v2-sdist" ["a", "b", "a-tests"]
   shouldNotExist $ cwd </> "dist-newstyle/sdist/a-0.1.tar.gz"
