@@ -1,6 +1,6 @@
 import Test.Cabal.Prelude
 main = setupAndCabalTest $ do
     skipUnlessGhcVersion ">= 7.9"
-    withDirectory "p" $ do
-        r <- fails $ setup' "configure" ["--cabal-file", "p.cabal.fail-other"]
+    withDirectory "p-fail-other" $ do
+        r <- fails $ setup' "configure" []
         assertOutputContains "Private" r
