@@ -1,4 +1,5 @@
 import Test.Cabal.Prelude
 import Test.Cabal.Script
 main = setupTest $
-  void $ setup'' "pkg" "configure" ["--cabal-file", "pkg/a.cabal"]
+  withDirectory "pkg" $
+  void $ setup' "configure" []
