@@ -3,7 +3,10 @@ import Test.Cabal.Prelude
 -- an executable RPATH.  Don't test on Windows, which doesn't
 -- support RPATH.
 main = setupAndCabalTest $ do
-    skipIf =<< isWindows
+  skipIfWindows
+  osx <- isOSX
+  ghc <- isGhcVersion ">= 8.10.7"
+  expectBrokenIf (osx && ghc) 7610 $ do -- see also issue #7988
     setup "configure" ["--enable-executable-dynamic"]
     setup "build" []
     -- This should fail as it we should NOT be able to find the
