@@ -9,8 +9,8 @@ import qualified Distribution.Verbosity as Verbosity
 import Test.Cabal.Prelude
 
 main = cabalTest $ do
-    skipIf =<< isOSX -- TODO: re-enable this once the macOS Travis
-                     -- issues are resolved, see discussion in #4902.
+    skipIf "osx" =<< isOSX -- TODO: re-enable this once the macOS CI
+                           -- issues are resolved, see discussion in #4902.
 
     hasShared   <- hasSharedLibraries
     hasProfiled <- hasProfiledLibraries
