@@ -1,5 +1,5 @@
 import Test.Cabal.Prelude
 -- Test if detailed-0.9 builds correctly
 main = setupAndCabalTest $ do
-    skipUnless =<< hasCabalForGhc
+    skipIfAllCabalVersion "< 1.20"
     setup_build ["--enable-tests"]
