@@ -29,16 +29,17 @@ import Distribution.Simple.Program.Types
 import Distribution.Simple.Program.Db
 import Distribution.Simple.Program
 import Distribution.System (OS(Windows,Linux,OSX), Arch(JavaScript), buildOS, buildArch)
-import Distribution.Simple.Utils
-    ( withFileContents, withTempDirectory, tryFindPackageDesc )
 import Distribution.Simple.Configure
     ( getPersistBuildConfig )
+import Distribution.Simple.Utils
+    ( withFileContents, tryFindPackageDesc )
 import Distribution.Version
 import Distribution.Package
-import Distribution.Parsec (eitherParsec)
+import Distribution.Parsec (eitherParsec, simpleParsec)
 import Distribution.Types.UnqualComponentName
 import Distribution.Types.LocalBuildInfo
 import Distribution.PackageDescription
+import Test.Utils.TempTestDir (withTestDir)
 import Distribution.Verbosity (normal)
 
 import Distribution.Compat.Stack
@@ -59,16 +60,16 @@ import Data.List.NonEmpty (NonEmpty (..))
 import qualified Data.List.NonEmpty as NE
 import Data.Maybe (mapMaybe, fromMaybe)
 import System.Exit (ExitCode (..))
-import System.FilePath ((</>), takeExtensions, takeDrive, takeDirectory, normalise, splitPath, joinPath, splitFileName, (<.>), dropTrailingPathSeparator)
+import System.FilePath
 import Control.Concurrent (threadDelay)
 import qualified Data.Char as Char
-import System.Directory (getTemporaryDirectory, getCurrentDirectory, canonicalizePath, copyFile, copyFile, doesDirectoryExist, doesFileExist, createDirectoryIfMissing, getDirectoryContents, listDirectory)
+import System.Directory
 import Control.Retry (exponentialBackoff, limitRetriesByCumulativeDelay)
 import Network.Wait (waitTcpVerbose)
+import System.Environment
 
 #ifndef mingw32_HOST_OS
 import Control.Monad.Catch ( bracket_ )
-import System.Directory    ( removeFile )
 import System.Posix.Files  ( createSymbolicLink )
 import System.Posix.Resource
 #endif
@@ -107,11 +108,25 @@ withDirectory :: FilePath -> TestM a -> TestM a
 withDirectory f = withReaderT
     (\env -> env { testRelativeCurrentDir = testRelativeCurrentDir env </> f })
 
+withStoreDir :: FilePath -> TestM a -> TestM a
+withStoreDir fp =
+  withReaderT (\env -> env { testMaybeStoreDir = Just fp })
+
 -- We append to the environment list, as per 'getEffectiveEnvironment'
 -- which prefers the latest override.
 withEnv :: [(String, Maybe String)] -> TestM a -> TestM a
 withEnv e = withReaderT (\env -> env { testEnvironment = testEnvironment env ++ e })
 
+-- | Prepend a directory to the PATH
+addToPath :: FilePath -> TestM a -> TestM a
+addToPath exe_dir action = do
+  env <- getTestEnv
+  path <- liftIO $ getEnv "PATH"
+  let newpath = exe_dir ++ [searchPathSeparator] ++ path
+  let new_env = (("PATH", Just newpath) : (testEnvironment env))
+  withEnv new_env action
+
+
 -- HACK please don't use me
 withEnvFilter :: (String -> Bool) -> TestM a -> TestM a
 withEnvFilter p = withReaderT (\env -> env { testEnvironment = filter (p . fst) (testEnvironment env) })
@@ -278,9 +293,6 @@ cabalG' global_args cmd args = cabalGArgs global_args cmd args Nothing
 cabalGArgs :: [String] -> String -> [String] -> Maybe String -> TestM Result
 cabalGArgs global_args cmd args input = do
     env <- getTestEnv
-    -- Freeze writes out cabal.config to source directory, this is not
-    -- overwritable
-    when (cmd == "v1-freeze") requireHasSourceCopy
     let extra_args
           | cmd `elem`
               [ "v1-update"
@@ -298,27 +310,36 @@ cabalGArgs global_args cmd args input = do
           = [ ]
 
           -- new-build commands are affected by testCabalProjectFile
-          | cmd == "v2-sdist"
-          = [ "--project-file", testCabalProjectFile env ]
+          | cmd `elem` ["v2-sdist", "path"]
+          = [ "--project-file=" ++ fp | Just fp <- [testCabalProjectFile env] ]
 
-          | cmd == "v2-clean"
-          = [ "--builddir", testDistDir env
-            , "--project-file", testCabalProjectFile env ]
+          | cmd == "v2-clean" || cmd == "clean"
+          = [ "--builddir", testDistDir env ]
+            ++ [ "--project-file=" ++ fp | Just fp <- [testCabalProjectFile env] ]
 
           | "v2-" `isPrefixOf` cmd
           = [ "--builddir", testDistDir env
-            , "--project-file", testCabalProjectFile env
             , "-j1" ]
+            ++ [ "--project-file=" ++ fp | Just fp <- [testCabalProjectFile env] ]
+            ++ ["--package-db=" ++ db | Just db <- [testPackageDbPath env]]
+          | "v1-" `isPrefixOf` cmd
+          = [ "--builddir", testDistDir env ]
+            ++ install_args
 
           | otherwise
-          = [ "--builddir", testDistDir env ] ++
-            install_args
+          = [ "--builddir", testDistDir env ]
+            ++ ["--package-db=" ++ db | Just db <- [testPackageDbPath env]]
+            ++ install_args
 
         install_args
           | cmd == "v1-install" || cmd == "v1-build" = [ "-j1" ]
           | otherwise                                = []
 
-        cabal_args = global_args
+        global_args' =
+            [ "--store-dir=" ++ storeDir | Just storeDir <- [testMaybeStoreDir env] ]
+            ++ global_args
+
+        cabal_args = global_args'
                   ++ [ cmd, marked_verbose ]
                   ++ extra_args
                   ++ args
@@ -331,7 +352,7 @@ cabal_raw' cabal_args input = runProgramM cabalProgram cabal_args input
 
 withProjectFile :: FilePath -> TestM a -> TestM a
 withProjectFile fp m =
-    withReaderT (\env -> env { testCabalProjectFile = fp }) m
+    withReaderT (\env -> env { testCabalProjectFile = Just fp }) m
 
 -- | Assuming we've successfully configured a new-build project,
 -- read out the plan metadata so that we can use it to do other
@@ -356,15 +377,21 @@ runPlanExe pkg_name cname args = void $ runPlanExe' pkg_name cname args
 runPlanExe' :: String {- package name -} -> String {- component name -}
             -> [String] -> TestM Result
 runPlanExe' pkg_name cname args = do
+    exePath <- planExePath pkg_name cname
+    defaultRecordMode RecordAll $ do
+    recordHeader [pkg_name, cname]
+    runM exePath args Nothing
+
+planExePath :: String {- package name -} -> String {- component name -}
+            -> TestM FilePath
+planExePath pkg_name cname = do
     Just plan <- testPlan `fmap` getTestEnv
     let distDirOrBinFile = planDistDir plan (mkPackageName pkg_name)
                                (CExeName (mkUnqualComponentName cname))
         exePath = case distDirOrBinFile of
           DistDir dist_dir -> dist_dir </> "build" </> cname </> cname
           BinFile bin_file -> bin_file
-    defaultRecordMode RecordAll $ do
-    recordHeader [pkg_name, cname]
-    runM exePath args Nothing
+    return exePath
 
 ------------------------------------------------------------------------
 -- * Running ghc-pkg
@@ -384,6 +411,12 @@ withPackageDb m = do
                $ do ghcPkg "init" [db_path]
                     m
 
+-- | Don't pass `--package-db` to cabal-install, so it won't find the specific version of
+-- `Cabal` which you have configured the testsuite to run with. You probably don't want to use
+-- this unless you are testing the `--package-db` flag itself.
+noCabalPackageDb :: TestM a -> TestM a
+noCabalPackageDb m = withReaderT (\nenv -> nenv { testPackageDbPath = Nothing }) m
+
 ghcPkg :: String -> [String] -> TestM ()
 ghcPkg cmd args = void (ghcPkg' cmd args)
 
@@ -649,26 +682,6 @@ withRemoteRepo repoDir m = do
             runReaderT m (env { testHaveRepo = True }))
 
 
-------------------------------------------------------------------------
--- * Subprocess run results
-
-requireSuccess :: Result -> TestM Result
-requireSuccess r@Result { resultCommand = cmd
-                        , resultExitCode = exitCode
-                        , resultOutput = output } = withFrozenCallStack $ do
-    env <- getTestEnv
-    when (exitCode /= ExitSuccess && not (testShouldFail env)) $
-        assertFailure $ "Command " ++ cmd ++ " failed.\n" ++
-        "Output:\n" ++ output ++ "\n"
-    when (exitCode == ExitSuccess && testShouldFail env) $
-        assertFailure $ "Command " ++ cmd ++ " succeeded.\n" ++
-        "Output:\n" ++ output ++ "\n"
-    return r
-
-initWorkDir :: TestM ()
-initWorkDir = do
-    env <- getTestEnv
-    liftIO $ createDirectoryIfMissing True (testWorkDir env)
 
 -- | Record a header to help identify the output to the expect
 -- log.  Unlike the 'recordLog', we don't record all arguments;
@@ -680,46 +693,21 @@ recordHeader args = do
     env <- getTestEnv
     let mode = testRecordMode env
         str_header = "# " ++ intercalate " " args ++ "\n"
-        header = C.pack (testRecordNormalizer env str_header)
+        rec_header = C.pack str_header
     case mode of
         DoNotRecord -> return ()
         _ -> do
             initWorkDir
             liftIO $ putStr str_header
-            liftIO $ C.appendFile (testWorkDir env </> "test.log") header
-            liftIO $ C.appendFile (testActualFile env) header
+            liftIO $ C.appendFile (testWorkDir env </> "test.log") rec_header
+            liftIO $ C.appendFile (testActualFile env) rec_header
 
-recordLog :: Result -> TestM ()
-recordLog res = do
-    env <- getTestEnv
-    let mode = testRecordMode env
-    initWorkDir
-    liftIO $ C.appendFile (testWorkDir env </> "test.log")
-                         (C.pack $ "+ " ++ resultCommand res ++ "\n"
-                            ++ resultOutput res ++ "\n\n")
-    liftIO . C.appendFile (testActualFile env) . C.pack . testRecordNormalizer env $
-        case mode of
-            RecordAll    -> unlines (lines (resultOutput res))
-            RecordMarked -> getMarkedOutput (resultOutput res)
-            DoNotRecord  -> ""
-
-getMarkedOutput :: String -> String -- trailing newline
-getMarkedOutput out = unlines (go (lines out) False)
-  where
-    go [] _ = []
-    go (x:xs) True
-        | "-----END CABAL OUTPUT-----"   `isPrefixOf` x
-                    =     go xs False
-        | otherwise = x : go xs True
-    go (x:xs) False
-        -- NB: Windows has extra goo at the end
-        | "-----BEGIN CABAL OUTPUT-----" `isPrefixOf` x
-                    = go xs True
-        | otherwise = go xs False
 
 ------------------------------------------------------------------------
 -- * Test helpers
 
+------------------------------------------------------------------------
+-- * Subprocess run results
 assertFailure :: WithCallStack (String -> m ())
 assertFailure msg = withFrozenCallStack $ error msg
 
@@ -788,10 +776,6 @@ recordMode mode = withReaderT (\env -> env {
     testRecordUserMode = Just mode
     })
 
-recordNormalizer :: (String -> String) -> TestM a -> TestM a
-recordNormalizer f =
-    withReaderT (\env -> env { testRecordNormalizer = testRecordNormalizer env . f })
-
 assertOutputContains :: MonadIO m => WithCallStack (String -> Result -> m ())
 assertOutputContains needle result =
     withFrozenCallStack $
@@ -874,6 +858,44 @@ hasCabalShared = do
   env <- getTestEnv
   return (testHaveCabalShared env)
 
+
+anyCabalVersion :: WithCallStack ( String -> TestM Bool )
+anyCabalVersion = isCabalVersion any
+
+allCabalVersion :: WithCallStack ( String -> TestM Bool )
+allCabalVersion = isCabalVersion all
+
+-- Used by cabal-install tests to determine which Cabal library versions are
+-- available. Given a version range, and a predicate on version ranges,
+-- are there any installed packages Cabal library
+-- versions which satisfy these.
+isCabalVersion :: WithCallStack (((Version -> Bool) -> [Version] -> Bool) -> String -> TestM Bool)
+isCabalVersion decide range = do
+  env <- getTestEnv
+  cabal_pkgs <- ghcPkg_raw' $ ["--global", "list", "Cabal", "--simple"] ++ ["--package-db=" ++ db | Just db <- [testPackageDbPath env]]
+  let pkg_versions :: [PackageIdentifier] = mapMaybe simpleParsec (words (resultOutput cabal_pkgs))
+  vr <- case eitherParsec range of
+          Left err -> fail err
+          Right vr -> return vr
+  return $ decide (`withinRange` vr)  (map pkgVersion pkg_versions)
+
+-- | Skip a test unless any available Cabal library version matches the predicate.
+skipUnlessAnyCabalVersion :: String -> TestM ()
+skipUnlessAnyCabalVersion range = skipUnless ("needs any Cabal " ++ range) =<< anyCabalVersion range
+
+
+-- | Skip a test if any available Cabal library version matches the predicate.
+skipIfAnyCabalVersion :: String -> TestM ()
+skipIfAnyCabalVersion range = skipIf ("incompatible with Cabal " ++ range) =<< anyCabalVersion range
+
+-- | Skip a test unless all Cabal library versions match the predicate.
+skipUnlessAllCabalVersion :: String -> TestM ()
+skipUnlessAllCabalVersion range = skipUnless ("needs all Cabal " ++ range) =<< allCabalVersion range
+
+-- | Skip a test if all the Cabal library version matches a predicate.
+skipIfAllCabalVersion :: String -> TestM ()
+skipIfAllCabalVersion range = skipIf ("incompatible with Cabal " ++ range) =<< allCabalVersion range
+
 isGhcVersion :: WithCallStack (String -> TestM Bool)
 isGhcVersion range = do
     ghc_program <- requireProgramM ghcProgram
@@ -928,24 +950,6 @@ getOpenFilesLimit = liftIO $ do
         _                                     -> return Nothing
 #endif
 
-hasCabalForGhc :: TestM Bool
-hasCabalForGhc = do
-    env <- getTestEnv
-    ghc_program <- requireProgramM ghcProgram
-    (runner_ghc_program, _) <- liftIO $ requireProgram
-        (testVerbosity env)
-        ghcProgram
-        (runnerProgramDb (testScriptEnv env))
-
-    -- TODO: I guess, to be more robust what we should check for
-    -- specifically is that the Cabal library we want to use
-    -- will be picked up by the package db stack of ghc-program
-
-    -- liftIO $ putStrLn $ "ghc_program:        " ++ show ghc_program
-    -- liftIO $ putStrLn $ "runner_ghc_program: " ++ show runner_ghc_program
-
-    return (programPath ghc_program == programPath runner_ghc_program)
-
 -- | If you want to use a Custom setup with new-build, it needs to
 -- be 1.20 or later.  Ordinarily, Cabal can go off and build a
 -- sufficiently recent Cabal if necessary, but in our test suite,
@@ -980,8 +984,7 @@ expectBrokenIf True ticket m = expectBroken ticket m
 expectBrokenUnless :: Bool -> Int -> TestM a -> TestM ()
 expectBrokenUnless b = expectBrokenIf (not b)
 
-------------------------------------------------------------------------
--- * Miscellaneous
+-- * Programs
 
 git :: String -> [String] -> TestM ()
 git cmd args = void $ git' cmd args
@@ -1007,6 +1010,12 @@ ghc' args = do
     recordHeader ["ghc"]
     runProgramM ghcProgram args Nothing
 
+ghcPkg_raw' :: [String] -> TestM Result
+ghcPkg_raw' args = do
+  recordHeader ["ghc-pkg"]
+  runProgramM ghcPkgProgram args Nothing
+
+
 python3 :: [String] -> TestM ()
 python3 args = void $ python3' args
 
@@ -1015,41 +1024,6 @@ python3' args = do
     recordHeader ["python3"]
     runProgramM python3Program args Nothing
 
--- | If a test needs to modify or write out source files, it's
--- necessary to make a hermetic copy of the source files to operate
--- on.  This function arranges for this to be done.
---
--- This requires the test repository to be a Git checkout, because
--- we use the Git metadata to figure out what files to copy into the
--- hermetic copy.
---
--- Also see 'withSourceCopyDir'.
-withSourceCopy :: TestM a -> TestM a
-withSourceCopy m = do
-    env <- getTestEnv
-    let cwd  = testCurrentDir env
-        dest = testSourceCopyDir env
-    r <- git' "ls-files" ["--cached", "--modified"]
-    forM_ (lines (resultOutput r)) $ \f -> do
-        unless (isTestFile f) $ do
-            liftIO $ createDirectoryIfMissing True (takeDirectory (dest </> f))
-            liftIO $ copyFile (cwd </> f) (dest </> f)
-    withReaderT (\nenv -> nenv { testHaveSourceCopy = True }) m
-
--- | If a test needs to modify or write out source files, it's
--- necessary to make a hermetic copy of the source files to operate
--- on.  This function arranges for this to be done in a subdirectory
--- with a given name, so that tests that are sensitive to the path
--- that they're running in (e.g., autoconf tests) can run.
---
--- This requires the test repository to be a Git checkout, because
--- we use the Git metadata to figure out what files to copy into the
--- hermetic copy.
---
--- Also see 'withSourceCopy'.
-withSourceCopyDir :: FilePath -> TestM a -> TestM a
-withSourceCopyDir dir =
-  withReaderT (\nenv -> nenv { testSourceCopyRelativeDir = dir }) . withSourceCopy
 
 -- | Look up the 'InstalledPackageId' of a package name.
 getIPID :: String -> TestM String
@@ -1111,55 +1085,39 @@ withSymlink oldpath newpath0 act = do
 
 writeSourceFile :: FilePath -> String -> TestM ()
 writeSourceFile fp s = do
-    requireHasSourceCopy
     cwd <- fmap testCurrentDir getTestEnv
     liftIO $ writeFile (cwd </> fp) s
 
 copySourceFileTo :: FilePath -> FilePath -> TestM ()
 copySourceFileTo src dest = do
-    requireHasSourceCopy
     cwd <- fmap testCurrentDir getTestEnv
     liftIO $ copyFile (cwd </> src) (cwd </> dest)
 
-requireHasSourceCopy :: TestM ()
-requireHasSourceCopy = do
-    env <- getTestEnv
-    unless (testHaveSourceCopy env) $ do
-        error "This operation requires a source copy; use withSourceCopy and 'git add' all test files"
-
--- NB: Keep this synchronized with partitionTests
-isTestFile :: FilePath -> Bool
-isTestFile f =
-    case takeExtensions f of
-        ".test.hs"      -> True
-        ".multitest.hs" -> True
-        _               -> False
-
 -- | Work around issue #4515 (store paths exceeding the Windows path length
 -- limit) by creating a temporary directory for the new-build store. This
 -- function creates a directory immediately under the current drive on Windows.
 -- The directory must be passed to new- commands with --store-dir.
-withShorterPathForNewBuildStore :: (FilePath -> IO a) -> IO a
-withShorterPathForNewBuildStore test = do
-  tempDir <- if buildOS == Windows
-             then takeDrive `fmap` getCurrentDirectory
-             else getTemporaryDirectory
-  withTempDirectory normal tempDir "cabal-test-store" test
+withShorterPathForNewBuildStore :: TestM a -> TestM a
+withShorterPathForNewBuildStore test =
+  withTestDir normal "cabal-test-store" (\f -> withStoreDir f test)
 
 -- | Find where a package locates in the store dir. This works only if there is exactly one 1 ghc version
 -- and exactly 1 directory for the given package in the store dir.
-findDependencyInStore :: FilePath -- ^store dir
-                      -> String -- ^package name prefix
-                      -> IO FilePath -- ^package dir
-findDependencyInStore storeDir pkgName = do
-    storeDirForGhcVersion <- head <$> listDirectory storeDir
-    packageDirs <- listDirectory (storeDir </> storeDirForGhcVersion)
-    -- Ideally, we should call 'hashedInstalledPackageId' from 'Distribution.Client.PackageHash'.
-    -- But 'PackageHashInputs', especially 'PackageHashConfigInputs', is too hard to construct.
-    let pkgName' =
-            if buildOS == OSX
-            then filter (not . flip elem "aeiou") pkgName
-                -- simulates the way 'hashedInstalledPackageId' uses to compress package name
-            else pkgName
-    let libDir = head $ filter (pkgName' `isPrefixOf`) packageDirs
-    pure (storeDir </> storeDirForGhcVersion </> libDir)
+findDependencyInStore :: String -- ^package name prefix
+                      -> TestM FilePath -- ^package dir
+findDependencyInStore pkgName = do
+    storeDir <- testStoreDir <$> getTestEnv
+    liftIO $ do
+      storeDirForGhcVersion:_ <- listDirectory storeDir
+      packageDirs <- listDirectory (storeDir </> storeDirForGhcVersion)
+      -- Ideally, we should call 'hashedInstalledPackageId' from 'Distribution.Client.PackageHash'.
+      -- But 'PackageHashInputs', especially 'PackageHashConfigInputs', is too hard to construct.
+      let pkgName' =
+              if buildOS == OSX
+              then filter (not . flip elem "aeiou") pkgName
+                  -- simulates the way 'hashedInstalledPackageId' uses to compress package name
+              else pkgName
+      let libDir = case filter (pkgName' `isPrefixOf`) packageDirs of
+                      [] -> error $ "Could not find " <> pkgName' <> " when searching for " <> pkgName' <> " in\n" <> show packageDirs
+                      (dir:_) -> dir
+      pure (storeDir </> storeDirForGhcVersion </> libDir)
