@@ -40,6 +40,9 @@ import qualified GHC.IO.Exception as GHC
 import Distribution.Verbosity
 
 import System.Process.Internals
+  ( ProcessHandle__( OpenHandle )
+  , withProcessHandle
+  )
 #if mingw32_HOST_OS
 import qualified System.Win32.Process as Win32
 #endif
@@ -215,7 +218,7 @@ runMain ref m = do
 startServer :: Chan ServerLogMsg -> ScriptEnv -> IO Server
 startServer chan senv = do
     (prog, _) <- requireProgram verbosity ghcProgram (runnerProgramDb senv)
-    let ghc_args = runnerGhcArgs senv ++ ["--interactive", "-v0", "-ignore-dot-ghci"]
+    let ghc_args = runnerGhcArgs senv Nothing ++ ["--interactive", "-v0", "-ignore-dot-ghci"]
         proc_spec = (proc (programPath prog) ghc_args) {
                         create_group = True,
                         -- Closing fds is VERY important to avoid
