@@ -9,7 +9,7 @@ import GHC.IO
 main = do
    let
       maxWord = fromIntegral (maxBound :: Word)
-      invalid = NB (bigNatOne# void#) -- 1 would fit into the NS constructor.
+      invalid = NB (bigNatOne# (# #)) -- 1 would fit into the NS constructor.
 
    -- byteArray whose size is not a multiple of Word size
    invalid2 <- IO $ \s -> case newByteArray# 27# s of
