@@ -6,5 +6,7 @@ main = do
   putStrLn $ traceShow (3 :: Int) "four"
   putStrLn $ traceId "five"
   print $ traceShowId (6 :: Int)
+  putStrLn $ traceWith (take 3) "seven"
+  putStrLn $ traceShowWith length "eight"
   traceIO "nine"
   traceM "ten"
