@@ -1,13 +1,13 @@
 {-# LANGUAGE CPP #-}
 
+#include "MachDeps.h"
+
 -- | Why does this module exist? There is "GHC.ByteOrder" in base.
 -- But that module is /broken/ until base-4.14/ghc-8.10, so we
 -- can't rely on it until we drop support for older ghcs.
 -- See https://gitlab.haskell.org/ghc/ghc/-/issues/20338
 -- and https://gitlab.haskell.org/ghc/ghc/-/issues/18445
 
-#include "MachDeps.h"
-
 module Data.ByteString.Utils.ByteOrder
   ( ByteOrder(..)
   , hostByteOrder
@@ -15,9 +15,7 @@ module Data.ByteString.Utils.ByteOrder
   , whenBigEndian
   ) where
 
-data ByteOrder
-  = LittleEndian
-  | BigEndian
+import GHC.ByteOrder (ByteOrder(..))
 
 hostByteOrder :: ByteOrder
 hostByteOrder =
