@@ -1,4 +1,5 @@
 {-# LANGUAGE CPP #-}
+{-# OPTIONS_HADDOCK not-home #-}
 -- |
 -- Stability: unstable
 -- Portability: unportable
@@ -13,26 +14,18 @@
 module System.Directory.Internal
   ( module System.Directory.Internal.Common
 
-#ifdef mingw32_HOST_OS
+#if defined(mingw32_HOST_OS)
   , module System.Directory.Internal.Windows
 #else
   , module System.Directory.Internal.Posix
 #endif
 
-#ifdef HAVE_UTIMENSAT
-  , module System.Directory.Internal.C_utimensat
-#endif
-
   ) where
 
 import System.Directory.Internal.Common
 
-#ifdef mingw32_HOST_OS
+#if defined(mingw32_HOST_OS)
 import System.Directory.Internal.Windows
 #else
 import System.Directory.Internal.Posix
 #endif
-
-#ifdef HAVE_UTIMENSAT
-import System.Directory.Internal.C_utimensat
-#endif
