@@ -1,15 +1,15 @@
 {-# LANGUAGE CPP #-}
 module RenameFile001 where
 #include "util.inl"
-import System.Directory
+import System.Directory.Internal
 
 main :: TestEnv -> IO ()
 main _t = do
   writeFile tmp1 contents1
-  renameFile tmp1 tmp2
+  renameFile (os tmp1) (os tmp2)
   T(expectEq) () contents1 =<< readFile tmp2
   writeFile tmp1 contents2
-  renameFile tmp2 tmp1
+  renameFile (os tmp2) (os tmp1)
   T(expectEq) () contents1 =<< readFile tmp1
   where
     tmp1 = "tmp1"
