@@ -23,6 +23,11 @@ import qualified GHC.Num.Backend.Native as Native
 import {-# SOURCE #-} GHC.Num.Natural
 import {-# SOURCE #-} GHC.Num.Integer
 
+-- See W1 of Note [Tracking dependencies on primitives] in GHC.Internal.Base
+-- (This module uses the empty tuple () and string literals.)
+import GHC.Tuple ()
+import GHC.CString ()
+
 default ()
 
 -- | ghc-bignum backend name
