@@ -28,6 +28,11 @@ import GHC.Num.Primitives
 import GHC.Prim
 import GHC.Types
 
+-- See W1 of Note [Tracking dependencies on primitives] in GHC.Internal.Base
+-- (This module uses the empty tuple () and string literals.)
+import GHC.Tuple ()
+import GHC.CString ()
+
 default ()
 
 -- | ghc-bignum backend name
