@@ -1646,3 +1646,22 @@ instance Ord BigNat where
    BN# a >  BN# b = bigNatGt a b
    BN# a >= BN# b = bigNatGe a b
 
+{- Note [ghc-bignum and error symbols]
+~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+ghc-bignum only depends on ghc-prim, not base.
+Hence it may not introduce any symbol references to the error symbols in
+Control.Exception.Base such as `patError` or `impossibleError`, otherwise
+we get linker errors.
+
+This implies that
+
+  * Every pattern match must be "obviously complete", otherwise GHC's desugaring
+    inserts `patError`s for the missing cases. "Obviously complete" in the sense
+    that the desugarer is able to infer completeness, so considering type info
+    but not long distance information (as the pattern-match checker would do).
+  * We compile with -fno-catch-nonexhaustive-cases so that we don't risk
+    introducing `impossibleError`
+  * ... probably other similar observations that will be added to this list in
+    the future ...
+
+-}
