@@ -1,3 +1,4 @@
+{-# LANGUAGE Safe #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Monad.Writer.Strict
@@ -19,9 +20,9 @@
 
 module Control.Monad.Writer.Strict (
     -- * MonadWriter class
-    MonadWriter(..),
-    listens,
-    censor,
+    MonadWriter.MonadWriter(..),
+    MonadWriter.listens,
+    MonadWriter.censor,
     -- * The Writer monad
     Writer,
     runWriter,
@@ -31,19 +32,11 @@ module Control.Monad.Writer.Strict (
     WriterT(..),
     execWriterT,
     mapWriterT,
-    module Control.Monad,
-    module Control.Monad.Fix,
     module Control.Monad.Trans,
-    module Data.Monoid,
   ) where
 
-import Control.Monad.Writer.Class
-
+import qualified Control.Monad.Writer.Class as MonadWriter
 import Control.Monad.Trans
 import Control.Monad.Trans.Writer.Strict (
         Writer, runWriter, execWriter, mapWriter,
         WriterT(..), execWriterT, mapWriterT)
-
-import Control.Monad
-import Control.Monad.Fix
-import Data.Monoid
