@@ -149,11 +149,7 @@ newtype ParsecT s u m a
               -> (ParseError -> m b)                   -- empty err
               -> m b
              }
-#if MIN_VERSION_base(4,7,0)
      deriving ( Typeable )
-     -- GHC 7.6 doesn't like deriving instances of Typeable for types with
-     -- non-* type-arguments.
-#endif
 
 -- | Low-level unpacking of the ParsecT type. To run your parser, please look to
 -- runPT, runP, runParserT, runParser and other such functions.
@@ -222,11 +218,7 @@ instance Semigroup.Semigroup a => Semigroup.Semigroup (ParsecT s u m a) where
     --  discarding the first.
     (<>)     = Applicative.liftA2 (Semigroup.<>)
 
-#if MIN_VERSION_base(4,8,0)
     sconcat  = fmap Semigroup.sconcat . sequence
-#else
-    sconcat  = fmap (Semigroup.sconcat . NE.fromList) . sequence . NE.toList
-#endif
     stimes b = Semigroup.sconcat . NE.fromList . genericReplicate b
 
 -- | The 'Monoid' instance for 'ParsecT' is used for the same purposes as
