@@ -1,5 +1,3 @@
-{-# LANGUAGE CPP #-}
-{-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE PolymorphicComponents #-}
 {-# LANGUAGE Safe #-}
@@ -30,9 +28,7 @@ module Text.Parsec.Token
     ) where
 
 import Data.Char ( isAlpha, toLower, toUpper, isSpace, digitToInt )
-#if MIN_VERSION_base(4,7,0)
 import Data.Typeable ( Typeable )
-#endif
 import Data.List ( nub, sort )
 import Control.Monad.Identity (Identity)
 
@@ -107,9 +103,7 @@ data GenLanguageDef s u m
     caseSensitive  :: Bool
 
     }
-#if MIN_VERSION_base(4,7,0)
     deriving ( Typeable )
-#endif
 
 -----------------------------------------------------------
 -- A first class module: TokenParser
@@ -317,9 +311,7 @@ data GenTokenParser s u m
 
         commaSep1        :: forall a . ParsecT s u m a -> ParsecT s u m [a]
     }
-#if MIN_VERSION_base(4,7,0)
     deriving ( Typeable )
-#endif
 
 -----------------------------------------------------------
 -- Given a LanguageDef, create a token parser.
