@@ -39,6 +39,7 @@ module Data.Text.Internal.Builder
    ( -- * Public API
      -- ** The Builder type
      Builder
+   , LazyTextBuilder
    , toLazyText
    , toLazyTextWith
 
@@ -96,6 +97,8 @@ newtype Builder = Builder {
                 -> ST s [S.Text]
    }
 
+type LazyTextBuilder = Builder
+
 instance Semigroup Builder where
    (<>) = append
    {-# INLINE (<>) #-}
@@ -209,7 +212,7 @@ fromString str = Builder $ \k (Buffer p0 o0 u0 l0) ->
     in loop p0 o0 u0 l0 str
   where
     chunkSize = smallChunkSize
-{-# INLINE fromString #-}
+{-# INLINEABLE fromString #-}
 
 -- | /O(1)./ A @Builder@ taking a lazy @Text@, satisfying
 --
