@@ -3,31 +3,34 @@ module Main where
 import Data.Time
 
 monthBeginning :: TimeZone -> Integer -> Int -> UTCTime
-monthBeginning zone year month = localTimeToUTC zone
-    (LocalTime (fromGregorian year month 1) midnight)
+monthBeginning zone year month = localTimeToUTC zone (LocalTime (fromGregorian year month 1) midnight)
 
-findTransition :: UTCTime -> UTCTime -> IO [(UTCTime,TimeZone,TimeZone)]
+findTransition :: UTCTime -> UTCTime -> IO [(UTCTime, TimeZone, TimeZone)]
 findTransition a b = do
     za <- getTimeZone a
     zb <- getTimeZone b
-    if za == zb then return [] else do
-        let c = addUTCTime ((diffUTCTime b a) / 2) a
-        if a == c then return [(b,za,zb)] else do
-            tp <- findTransition a c
-            tq <- findTransition c b
-            return (tp ++ tq)
+    if za == zb
+        then return []
+        else do
+            let c = addUTCTime ((diffUTCTime b a) / 2) a
+            if a == c
+                then return [(b, za, zb)]
+                else do
+                    tp <- findTransition a c
+                    tq <- findTransition c b
+                    return (tp ++ tq)
 
 showZoneTime :: TimeZone -> UTCTime -> String
 showZoneTime zone time = show (utcToZonedTime zone time)
 
-showTransition :: (UTCTime,TimeZone,TimeZone) -> String
-showTransition (time,zone1,zone2) = (showZoneTime zone1 time) ++ " => " ++ (showZoneTime zone2 time)
+showTransition :: (UTCTime, TimeZone, TimeZone) -> String
+showTransition (time, zone1, zone2) = (showZoneTime zone1 time) ++ " => " ++ (showZoneTime zone2 time)
 
 main :: IO ()
 main = do
     now <- getCurrentTime
     zone <- getTimeZone now
-    let (year,_,_) = toGregorian (localDay (utcToLocalTime zone now))
+    let (year, _, _) = toGregorian (localDay (utcToLocalTime zone now))
     putStrLn ("DST adjustments for " ++ show year ++ ":")
     let t0 = monthBeginning zone year 1
     let t1 = monthBeginning zone year 4
