@@ -1,10 +1,5 @@
 {-# LANGUAGE CApiFFI #-}
-{-# LANGUAGE CPP #-}
-#if __GLASGOW_HASKELL__ >= 709
 {-# LANGUAGE Safe #-}
-#else
-{-# LANGUAGE Trustworthy #-}
-#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  System.Posix.Time
