@@ -8,14 +8,14 @@
 
 #pragma once
 
-extern StgClosure DLL_IMPORT_DATA_VARNAME(base_GHCziStackziCloneStack_StackSnapshot_closure);
-#define StackSnapshot_constructor_closure DLL_IMPORT_DATA_REF(base_GHCziStackziCloneStack_StackSnapshot_closure)
+extern StgClosure DLL_IMPORT_DATA_VARNAME(ghczminternal_GHCziInternalziStackziCloneStack_StackSnapshot_closure);
+#define StackSnapshot_constructor_closure DLL_IMPORT_DATA_REF(ghczminternal_GHCziInternalziStackziCloneStack_StackSnapshot_closure)
 
 StgStack* cloneStack(Capability* capability, const StgStack* stack);
 
 void sendCloneStackMessage(StgTSO *tso, HsStablePtr mvar);
 
-StgMutArrPtrs* decodeClonedStack(Capability *cap, StgStack* stack);
+StgArrBytes* decodeClonedStack(Capability *cap, StgStack* stack);
 
 #include "BeginPrivate.h"
 
@@ -23,10 +23,4 @@ StgMutArrPtrs* decodeClonedStack(Capability *cap, StgStack* stack);
 void handleCloneStackMessage(MessageCloneStack *msg);
 #endif
 
-StgWord getStackFrameCount(StgStack* stack);
-StgWord getStackChunkClosureCount(StgStack* stack);
-void copyPtrsToArray(Capability *cap, StgMutArrPtrs* arr, StgStack* stack);
-StgClosure* createPtrClosure(Capability* cap, InfoProvEnt* ipe);
-StgMutArrPtrs* allocateMutableArray(StgWord size);
-
 #include "EndPrivate.h"
