@@ -69,6 +69,10 @@ void *      removeStrHashTable ( StrHashTable *table, const char * key,
  */
 typedef int HashFunction(const HashTable *table, StgWord key);
 typedef int CompareFunction(StgWord key1, StgWord key2);
+
+// Helper for implementing hash functions
+int hashBuffer(const HashTable *table, const void *buf, size_t len);
+
 int hashWord(const HashTable *table, StgWord key);
 int hashStr(const HashTable *table, StgWord w);
 void        insertHashTable_ ( HashTable *table, StgWord key,
@@ -79,6 +83,7 @@ void *      removeHashTable_ ( HashTable *table, StgWord key,
                                const void *data, HashFunction f,
                                CompareFunction cmp );
 
+
 /* Freeing hash tables
  */
 void freeHashTable ( HashTable *table, void (*freeDataFun)(void *) );
@@ -91,7 +96,7 @@ INLINE_HEADER void freeStrHashTable ( StrHashTable *table, void (*freeDataFun)(v
 /*
  * Hash set API
  *
- * A hash set is bascially a hash table where values are NULL.
+ * A hash set is basically a hash table where values are NULL.
  */
 
 typedef struct hashtable HashSet;
