@@ -14,9 +14,7 @@
 #include "BeginPrivate.h"
 
 void dumpIPEToEventLog(void);
-void updateIpeMap(void);
 void initIpe(void);
 void exitIpe(void);
-void decompressIPEBufferListNodeIfCompressed(IpeBufferListNode*, IpeBufferEntry**, char**);
 
 #include "EndPrivate.h"
