@@ -32,6 +32,7 @@ tag functions as tag inference currently doesn't rely on those being properly ta
 #include "rts/PosixSource.h"
 #include "Rts.h"
 #include "RtsAPI.h"
+#include "RtsFlags.h"
 #include "rts/Bytecodes.h"
 
 // internal headers
@@ -158,11 +159,11 @@ tag functions as tag inference currently doesn't rely on those being properly ta
    cap->r.rRet = (retcode);                             \
    return cap;
 
-#define Sp_plusB(n)  ((void *)(((StgWord8*)Sp) + (n)))
-#define Sp_minusB(n) ((void *)(((StgWord8*)Sp) - (n)))
+#define Sp_plusB(n)  ((void *)((StgWord8*)Sp + (ptrdiff_t)(n)))
+#define Sp_minusB(n) ((void *)((StgWord8*)Sp - (ptrdiff_t)(n)))
 
-#define Sp_plusW(n)  (Sp_plusB((n) * sizeof(W_)))
-#define Sp_minusW(n) (Sp_minusB((n) * sizeof(W_)))
+#define Sp_plusW(n)  (Sp_plusB((ptrdiff_t)(n) * (ptrdiff_t)sizeof(W_)))
+#define Sp_minusW(n) (Sp_minusB((ptrdiff_t)(n) * (ptrdiff_t)sizeof(W_)))
 
 #define Sp_addB(n)   (Sp = Sp_plusB(n))
 #define Sp_subB(n)   (Sp = Sp_minusB(n))
@@ -325,7 +326,7 @@ static StgWord app_ptrs_itbl[] = {
 };
 
 HsStablePtr rts_breakpoint_io_action; // points to the IO action which is executed on a breakpoint
-                                      // it is set in compiler/GHC.hs:runStmt
+                                      // it is set in ghci/GHCi/Run.hs:withBreakAction
 
 Capability *
 interpretBCO (Capability* cap)
@@ -339,8 +340,9 @@ interpretBCO (Capability* cap)
 
     LOAD_THREAD_STATE();
 
-    cap->r.rHpLim = (P_)1; // HpLim is the context-switch flag; when it
-                           // goes to zero we must return to the scheduler.
+    // N.B. HpLim is the context-switch flag; when it
+    // goes to zero we must return to the scheduler.
+    RELAXED_STORE_ALWAYS(&cap->r.rHpLim, (P_)1);
 
     IF_DEBUG(interpreter,
              debugBelch(
@@ -1111,9 +1113,9 @@ run_BCO:
         /* check for a breakpoint on the beginning of a let binding */
         case bci_BRK_FUN:
         {
-            int arg1_brk_array, arg2_array_index, arg3_module_name;
+            int arg1_brk_array, arg2_tick_mod, arg3_info_mod, arg4_tick_index, arg5_info_index;
 #if defined(PROFILING)
-            int arg4_cc;
+            int arg6_cc;
 #endif
             StgArrBytes *breakPoints;
             int returning_from_break;
@@ -1128,10 +1130,12 @@ run_BCO:
             int size_words;
 
             arg1_brk_array      = BCO_GET_LARGE_ARG;
-            arg2_array_index    = BCO_NEXT;
-            arg3_module_name    = BCO_GET_LARGE_ARG;
+            arg2_tick_mod       = BCO_GET_LARGE_ARG;
+            arg3_info_mod       = BCO_GET_LARGE_ARG;
+            arg4_tick_index     = BCO_NEXT;
+            arg5_info_index     = BCO_NEXT;
 #if defined(PROFILING)
-            arg4_cc             = BCO_GET_LARGE_ARG;
+            arg6_cc             = BCO_GET_LARGE_ARG;
 #else
             BCO_GET_LARGE_ARG;
 #endif
@@ -1144,7 +1148,7 @@ run_BCO:
 
 #if defined(PROFILING)
             cap->r.rCCCS = pushCostCentre(cap->r.rCCCS,
-                                          (CostCentre*)BCO_LIT(arg4_cc));
+                                          (CostCentre*)BCO_LIT(arg6_cc));
 #endif
 
             // if we are returning from a break then skip this section
@@ -1156,11 +1160,11 @@ run_BCO:
                // stop the current thread if either the
                // "rts_stop_next_breakpoint" flag is true OR if the
                // ignore count for this particular breakpoint is zero
-               StgInt ignore_count = ((StgInt*)breakPoints->payload)[arg2_array_index];
+               StgInt ignore_count = ((StgInt*)breakPoints->payload)[arg4_tick_index];
                if (rts_stop_next_breakpoint == false && ignore_count > 0)
                {
                   // decrement and write back ignore count
-                  ((StgInt*)breakPoints->payload)[arg2_array_index] = --ignore_count;
+                  ((StgInt*)breakPoints->payload)[arg4_tick_index] = --ignore_count;
                }
                else if (rts_stop_next_breakpoint == true || ignore_count == 0)
                {
@@ -1193,8 +1197,10 @@ run_BCO:
                   // Arrange the stack to call the breakpoint IO action, and
                   // continue execution of this BCO when the IO action returns.
                   //
-                  // ioAction :: Int#        -- the breakpoint index
-                  //          -> Addr#       -- the module name
+                  // ioAction :: Addr#       -- the breakpoint tick module
+                  //          -> Int#        -- the breakpoint tick index
+                  //          -> Addr#       -- the breakpoint info module
+                  //          -> Int#        -- the breakpoint info index
                   //          -> Bool        -- exception?
                   //          -> HValue      -- the AP_STACK, or exception
                   //          -> IO ()
@@ -1202,15 +1208,19 @@ run_BCO:
                   ioAction = (StgClosure *) deRefStablePtr (
                       rts_breakpoint_io_action);
 
-                  Sp_subW(11);
-                  SpW(10) = (W_)obj;
-                  SpW(9)  = (W_)&stg_apply_interp_info;
-                  SpW(8)  = (W_)new_aps;
-                  SpW(7)  = (W_)False_closure;         // True <=> an exception
-                  SpW(6)  = (W_)&stg_ap_ppv_info;
-                  SpW(5)  = (W_)BCO_LIT(arg3_module_name);
+                  Sp_subW(15);
+                  SpW(14) = (W_)obj;
+                  SpW(13) = (W_)&stg_apply_interp_info;
+                  SpW(12) = (W_)new_aps;
+                  SpW(11) = (W_)False_closure;         // True <=> an exception
+                  SpW(10) = (W_)&stg_ap_ppv_info;
+                  SpW(9)  = (W_)arg5_info_index;
+                  SpW(8)  = (W_)&stg_ap_n_info;
+                  SpW(7)  = (W_)BCO_LIT(arg3_info_mod);
+                  SpW(6)  = (W_)&stg_ap_n_info;
+                  SpW(5)  = (W_)arg4_tick_index;
                   SpW(4)  = (W_)&stg_ap_n_info;
-                  SpW(3)  = (W_)arg2_array_index;
+                  SpW(3)  = (W_)BCO_LIT(arg2_tick_mod);
                   SpW(2)  = (W_)&stg_ap_n_info;
                   SpW(1)  = (W_)ioAction;
                   SpW(0)  = (W_)&stg_enter_info;
@@ -1299,21 +1309,21 @@ run_BCO:
 
         case bci_PUSH8_W: {
             W_ off = BCO_GET_LARGE_ARG;
-            *(StgWord*)(Sp_minusW(1)) = *(StgWord8*)(Sp_plusB(off));
+            *(StgWord8*)(Sp_minusW(1)) = *(StgWord8*)(Sp_plusB(off));
             Sp_subW(1);
             goto nextInsn;
         }
 
         case bci_PUSH16_W: {
             W_ off = BCO_GET_LARGE_ARG;
-            *(StgWord*)(Sp_minusW(1)) = *(StgWord16*)(Sp_plusB(off));
+            *(StgWord16*)(Sp_minusW(1)) = *(StgWord16*)(Sp_plusB(off));
             Sp_subW(1);
             goto nextInsn;
         }
 
         case bci_PUSH32_W: {
             W_ off = BCO_GET_LARGE_ARG;
-            *(StgWord*)(Sp_minusW(1)) = *(StgWord32*)(Sp_plusB(off));
+            *(StgWord32*)(Sp_minusW(1)) = *(StgWord32*)(Sp_plusB(off));
             Sp_subW(1);
             goto nextInsn;
         }
@@ -2026,7 +2036,7 @@ run_BCO:
             // context switching: sometimes the scheduler can invoke
             // the interpreter with context_switch == 1, particularly
             // if the -C0 flag has been given on the cmd line.
-            if (cap->r.rHpLim == NULL) {
+            if (RELAXED_LOAD(&cap->r.rHpLim) == NULL) {
                 Sp_subW(1); SpW(0) = (W_)&stg_enter_info;
                 RETURN_TO_SCHEDULER(ThreadInterpret, ThreadYielding);
             }
