@@ -232,11 +232,11 @@ static void ghciRemoveSymbolTable(StrHashTable *table, const SymbolName* key,
 static const char *
 symbolTypeString (SymType type)
 {
-    switch (type & ~SYM_TYPE_DUP_DISCARD) {
+    switch (type & ~(SYM_TYPE_DUP_DISCARD | SYM_TYPE_HIDDEN)) {
         case SYM_TYPE_CODE: return "code";
         case SYM_TYPE_DATA: return "data";
         case SYM_TYPE_INDIRECT_DATA: return "indirect-data";
-        default: barf("symbolTypeString: unknown symbol type");
+        default: barf("symbolTypeString: unknown symbol type (%d)", type);
     }
 }
 
@@ -283,10 +283,19 @@ int ghciInsertSymbolTable(
    }
    else if (pinfo->type ^ type)
    {
-       /* We were asked to discard the symbol on duplicates, do so quietly.  */
-       if (!(type & SYM_TYPE_DUP_DISCARD))
+       if(pinfo->type & SYM_TYPE_HIDDEN)
        {
-         debugBelch("Symbol type mismatch.\n");
+            /* The existing symbol is hidden, let's replace it */
+            pinfo->value = data;
+            pinfo->owner = owner;
+            pinfo->strength = strength;
+            pinfo->type = type;
+            return 1;
+       }
+       /* We were asked to discard the symbol on duplicates, do so quietly.  */
+       if (!(type & (SYM_TYPE_DUP_DISCARD | SYM_TYPE_HIDDEN)))
+       {
+         debugBelch("Symbol type mismatch (existing %d, new %d).\n", pinfo->type, type);
          debugBelch("Symbol %s was defined by %" PATH_FMT " to be a %s symbol.\n",
                     key, obj_name, symbolTypeString(type));
          debugBelch("      yet was defined by %" PATH_FMT " to be a %s symbol.\n",
@@ -909,12 +918,6 @@ SymbolAddr* loadSymbol(SymbolName *lbl, RtsSymbolInfo *pinfo) {
         if (!r) {
             return NULL;
         }
-
-#if defined(PROFILING)
-        // collect any new cost centres & CCSs
-        // that were defined during runInit
-        refreshProfilingCCSs();
-#endif
     }
 
     return pinfo->value;
@@ -1082,7 +1085,7 @@ freePreloadObjectFile (ObjectCode *oc)
  */
 void freeObjectCode (ObjectCode *oc)
 {
-    IF_DEBUG(linker, ocDebugBelch(oc, "start\n"));
+    IF_DEBUG(linker, ocDebugBelch(oc, "freeObjectCode: start\n"));
 
     // Run finalizers
     if (oc->type == STATIC_OBJECT &&
@@ -1158,7 +1161,7 @@ void freeObjectCode (ObjectCode *oc)
         stgFree(oc->sections);
     }
 
-    freeProddableBlocks(oc);
+    freeProddableBlocks(&oc->proddables);
     freeSegments(oc);
 
     /* Free symbol_extras.  On x86_64 Windows, symbol_extras are allocated
@@ -1243,7 +1246,7 @@ mkOc( ObjectType type, pathchar *path, char *image, int imageSize,
    oc->sections          = NULL;
    oc->n_segments        = 0;
    oc->segments          = NULL;
-   oc->proddables        = NULL;
+   initProddableBlockSet(&oc->proddables);
    oc->foreign_exports   = NULL;
 #if defined(NEED_SYMBOL_EXTRAS)
    oc->symbol_extras     = NULL;
@@ -1665,6 +1668,12 @@ int runPendingInitializers (void)
             return r;
         }
     }
+
+#if defined(PROFILING)
+    // collect any new cost centres & CCSs that were defined during runInit
+    refreshProfilingCCSs();
+#endif
+
     return 1;
 }
 
@@ -1691,11 +1700,6 @@ static HsInt resolveObjs_ (void)
         return 0;
     }
 
-#if defined(PROFILING)
-    // collect any new cost centres & CCSs that were defined during runInit
-    refreshProfilingCCSs();
-#endif
-
     IF_DEBUG(linker, debugBelch("resolveObjs: done\n"));
     return 1;
 }
@@ -1797,50 +1801,6 @@ OStatus getObjectLoadStatus (pathchar *path)
     return r;
 }
 
-/* -----------------------------------------------------------------------------
- * Sanity checking.  For each ObjectCode, maintain a list of address ranges
- * which may be prodded during relocation, and abort if we try and write
- * outside any of these.
- */
-void
-addProddableBlock ( ObjectCode* oc, void* start, int size )
-{
-   ProddableBlock* pb
-      = stgMallocBytes(sizeof(ProddableBlock), "addProddableBlock");
-
-   IF_DEBUG(linker, debugBelch("addProddableBlock: %p %p %d\n", oc, start, size));
-   ASSERT(size > 0);
-   pb->start      = start;
-   pb->size       = size;
-   pb->next       = oc->proddables;
-   oc->proddables = pb;
-}
-
-void
-checkProddableBlock (ObjectCode *oc, void *addr, size_t size )
-{
-   ProddableBlock* pb;
-
-   for (pb = oc->proddables; pb != NULL; pb = pb->next) {
-      char* s = (char*)(pb->start);
-      char* e = s + pb->size;
-      char* a = (char*)addr;
-      if (a >= s && (a+size) <= e) return;
-   }
-   barf("checkProddableBlock: invalid fixup in runtime linker: %p", addr);
-}
-
-void freeProddableBlocks (ObjectCode *oc)
-{
-    ProddableBlock *pb, *next;
-
-    for (pb = oc->proddables; pb != NULL; pb = next) {
-        next = pb->next;
-        stgFree(pb);
-    }
-    oc->proddables = NULL;
-}
-
 /* -----------------------------------------------------------------------------
  * Section management.
  */
