@@ -10,6 +10,7 @@
 
 #include "Capability.h"
 #include "Updates.h" // for DEBUG_FILL_SLOP
+#include "Schedule.h" // for SCHED_INTERRUPTING
 #include "SMPClosureOps.h"
 
 #include "BeginPrivate.h"
@@ -26,8 +27,9 @@ INLINE_HEADER void
 doneWithMsgThrowTo (Capability *cap, MessageThrowTo *m)
 {
     // The message better be locked (unless we are running single-threaded,
-    // where we are a bit more lenient (#19075).
-    ASSERT(getNumCapabilities() == 1 || m->header.info == &stg_WHITEHOLE_info);
+    // where we are a bit more lenient (#19075) or we got here from
+    // deleteAllThreads() due to RTS shutdown).
+    ASSERT(getNumCapabilities() == 1 || m->header.info == &stg_WHITEHOLE_info || getSchedState() == SCHED_INTERRUPTING);
     IF_NONMOVING_WRITE_BARRIER_ENABLED {
       updateRemembSetPushMessageThrowTo(cap, m);
     }
