@@ -142,6 +142,23 @@ restore_locale( void )
 unsigned int era;
 static uint32_t max_era;
 
+StgWord user_era;
+
+void
+setUserEra (StgWord w){
+  user_era = w;
+}
+
+StgWord
+getUserEra (void){
+  return user_era;
+}
+
+StgWord
+incrementUserEra (StgWord w){
+  return atomic_inc(&user_era, w);
+}
+
 inline void
 initLDVCtr( counter *ctr )
 {
@@ -180,6 +197,11 @@ closureIdentity( const StgClosure *p )
         return p->header.prof.ccs->cc->module;
     case HEAP_BY_DESCR:
         return GET_PROF_DESC(get_itbl(p));
+    case HEAP_BY_ERA:
+        // Static objects should have user_era = 0
+        // MP: If user_era == 0 then closureIdentity returns the NULL pointer, and
+        // the closure is not counted to the census
+        return (void *)p->header.prof.hp.era;
     case HEAP_BY_TYPE:
         return GET_PROF_TYPE(get_itbl(p));
     case HEAP_BY_RETAINER:
@@ -220,21 +242,6 @@ closureIdentity( const StgClosure *p )
 /* --------------------------------------------------------------------------
  * Profiling type predicates
  * ----------------------------------------------------------------------- */
-#if defined(PROFILING)
-STATIC_INLINE bool
-doingLDVProfiling( void )
-{
-    return (RtsFlags.ProfFlags.doHeapProfile == HEAP_BY_LDV
-            || RtsFlags.ProfFlags.bioSelector != NULL);
-}
-
-bool
-doingRetainerProfiling( void )
-{
-    return (RtsFlags.ProfFlags.doHeapProfile == HEAP_BY_RETAINER
-            || RtsFlags.ProfFlags.retainerSelector != NULL);
-}
-#endif /* PROFILING */
 
 // Processes a closure 'c' being destroyed whose size is 'size'.
 // Make sure that LDV_recordDead() is not invoked on 'inherently used' closures
@@ -356,6 +363,10 @@ freeEra(Census *census)
 static void
 nextEra( void )
 {
+    if (user_era > 0 && RtsFlags.ProfFlags.incrementUserEra){
+      user_era++;
+    }
+
 #if defined(PROFILING)
     if (doingLDVProfiling()) {
         era++;
@@ -437,18 +448,14 @@ initHeapProfiling(void)
         stem = stgMallocBytes(strlen(RtsFlags.CcFlags.outputFileNameStem) + 1, "initHeapProfiling");
         strcpy(stem, RtsFlags.CcFlags.outputFileNameStem);
     } else {
-
         stem = stgMallocBytes(strlen(prog_name) + 1, "initHeapProfiling");
         strcpy(stem, prog_name);
+
+        // Drop the platform's executable suffix if there is one
 #if defined(mingw32_HOST_OS)
-            // on Windows, drop the .exe suffix if there is one
-            {
-                char *suff;
-                suff = strrchr(stem,'.');
-                if (suff != NULL && !strcmp(suff,".exe")) {
-                    *suff = '\0';
-                }
-            }
+        dropExtension(stem, ".exe");
+#elif defined(wasm32_HOST_ARCH)
+        dropExtension(stem, ".wasm");
 #endif
     }
 
@@ -483,6 +490,14 @@ initHeapProfiling(void)
 #endif
 #endif
 
+#if defined(PROFILING)
+    if (doingErasProfiling()){
+      user_era = 1;
+    }
+#else
+    user_era = 0;
+#endif
+
     // we only count eras if we're doing LDV profiling.  Otherwise era
     // is fixed at zero.
 #if defined(PROFILING)
@@ -703,6 +718,9 @@ closureSatisfiesConstraints( const StgClosure* p )
                                 RtsFlags.ProfFlags.typeSelector );
        if (!b) return false;
    }
+   if (RtsFlags.ProfFlags.eraSelector) {
+      return (p->header.prof.hp.era == RtsFlags.ProfFlags.eraSelector);
+   }
    if (RtsFlags.ProfFlags.retainerSelector) {
        RetainerSet *rs;
        uint32_t i;
@@ -939,6 +957,12 @@ dumpCensus( Census *census )
             traceHeapProfSampleCostCentre(0, (CostCentreStack *)ctr->identity,
                                           count * sizeof(W_));
             break;
+        case HEAP_BY_ERA:
+            fprintf(hp_file, "%" FMT_Word, (StgWord)ctr->identity);
+            char str_era[100];
+            sprintf(str_era, "%" FMT_Word, (StgWord)ctr->identity);
+            traceHeapProfSampleString(0, str_era, count * sizeof(W_));
+            break;
         case HEAP_BY_MOD:
         case HEAP_BY_DESCR:
         case HEAP_BY_TYPE:
