@@ -6,21 +6,14 @@
  *
  * ---------------------------------------------------------------------------*/
 
-#ifndef PROFHEAP_H
-#define PROFHEAP_H
+#pragma once
 
 #include "BeginPrivate.h"
 
 void        heapCensus         (Time t);
-uint32_t    initHeapProfiling  (void);
+void        initHeapProfiling  (void);
 void        endHeapProfiling   (void);
+void        freeHeapProfiling  (void);
 bool        strMatchesSelector (const char* str, const char* sel);
 
-#ifdef PROFILING
-// doingRetainerProfiling: `-hr` or `-hr<cc> -h<x>`
-bool doingRetainerProfiling(void);
-#endif
-
 #include "EndPrivate.h"
-
-#endif /* PROFHEAP_H */
