@@ -24,6 +24,11 @@ void initRtsFlagsDefaults (void);
 void setupRtsFlags        (int *argc, char *argv[], RtsConfig rtsConfig);
 void freeRtsArgs          (void);
 bool is_io_mng_native_p   (void);
+#if defined(PROFILING)
+bool doingLDVProfiling (void);
+bool doingRetainerProfiling(void);
+bool doingErasProfiling(void);
+#endif
 
 extern RtsConfig rtsConfig;
 
