@@ -17,17 +17,27 @@
 void initAllocator(void);
 void shutdownAllocator(void);
 
+void stgFree(void* p);
+
 void *stgMallocBytes(size_t n, char *msg)
-    GNUC3_ATTRIBUTE(__malloc__);
+    STG_MALLOC STG_MALLOC1(stgFree)
+    STG_ALLOC_SIZE1(1);
+/* Note: unlike `stgReallocBytes` and `stgCallocBytes`, `stgMallocBytes` is
+ * *not* `STG_RETURNS_NONNULL`, since it will return `NULL` when the requested
+ * allocation size is zero.
+ *
+ * See: https://gitlab.haskell.org/ghc/ghc/-/issues/22380
+ */
 
 void *stgReallocBytes(void *p, size_t n, char *msg);
 
 void *stgCallocBytes(size_t count, size_t size, char *msg)
-     GNUC3_ATTRIBUTE(__malloc__);
+    STG_MALLOC STG_MALLOC1(stgFree)
+    STG_ALLOC_SIZE2(1, 2)
+    STG_RETURNS_NONNULL;
 
-char *stgStrndup(const char *s, size_t n);
-
-void stgFree(void* p);
+char *stgStrndup(const char *s, size_t n)
+    STG_MALLOC STG_MALLOC1(stgFree);
 
 void *stgMallocAlignedBytes(size_t n, size_t align, char *msg);
 
@@ -52,4 +62,7 @@ void checkFPUStack(void);
 #define xstr(s) str(s)
 #define str(s) #s
 
+// Drop the given extension from a filepath.
+void dropExtension(char *path, const char *extension);
+
 #include "EndPrivate.h"
