@@ -28,8 +28,11 @@ StgTSO *getTopHandlerThread(void) {
     ACQUIRE_LOCK(&m);
     StgWeak *weak = (StgWeak*)deRefStablePtr(topHandlerPtr);
     RELEASE_LOCK(&m);
-    const StgInfoTable *info = weak->header.info;
-    load_load_barrier();
+    if (weak == NULL) {
+        // topHandlerPtr was never initialised
+        return NULL;
+    }
+    const StgInfoTable *info = ACQUIRE_LOAD(&weak->header.info);
     if (info == &stg_WEAK_info) {
         StgClosure *key = ((StgWeak*)weak)->key;
 
