@@ -4,7 +4,7 @@
  *
  * ---------------------------------------------------------------------------*/
 
-#include "PosixSource.h"
+#include "rts/PosixSource.h"
 #include "Rts.h"
 #include "Hooks.h"
 #include "RtsFlags.h"
