@@ -105,10 +105,10 @@
 #  include <elf_abi.h>
 #endif
 
+#include "elf_got.h"
+
 #if defined(arm_HOST_ARCH) || defined(aarch64_HOST_ARCH)
-#  define NEED_GOT
 #  define NEED_PLT
-#  include "elf_got.h"
 #  include "elf_plt.h"
 #  include "elf_reloc.h"
 #endif
@@ -802,7 +802,7 @@ ocGetNames_ELF ( ObjectCode* oc )
          /* This is a non-empty .bss section.  Allocate zeroed space for
             it, and set its .sh_offset field such that
             ehdrC + .sh_offset == addr_of_zeroed_space.  */
-#if defined(NEED_GOT) || RTS_LINKER_USE_MMAP
+#if RTS_LINKER_USE_MMAP
           if (USE_CONTIGUOUS_MMAP || RtsFlags.MiscFlags.linkerAlwaysPic) {
               /* The space for bss sections is already preallocated */
               CHECK(oc->bssBegin != NULL);
@@ -926,7 +926,7 @@ ocGetNames_ELF ( ObjectCode* oc )
           oc->sections[i].info->stubs = NULL;
 #endif
 
-          addProddableBlock(oc, start, size);
+          addProddableBlock(&oc->proddables, start, size);
       } else {
           addSection(&oc->sections[i], kind, alloc, oc->image+offset, size,
                      0, 0, 0);
@@ -1073,6 +1073,9 @@ ocGetNames_ELF ( ObjectCode* oc )
                } else {
                    sym_type = SYM_TYPE_DATA;
                }
+               if(ELF_ST_VISIBILITY(symbol->elf_sym->st_other) == STV_HIDDEN) {
+                   sym_type |= SYM_TYPE_HIDDEN;
+               }
 
                /* And the decision is ... */
 
@@ -1115,13 +1118,11 @@ ocGetNames_ELF ( ObjectCode* oc )
       }
    }
 
-#if defined(NEED_GOT)
    if(makeGot( oc ))
        errorBelch("Failed to create GOT for %s",
                   oc->archiveMemberName
                   ? oc->archiveMemberName
                   : oc->fileName);
-#endif
    result = 1;
    goto end;
 
@@ -1272,7 +1273,7 @@ do_Elf_Rel_relocations ( ObjectCode* oc, char* ehdrC,
                 debugBelch("Reloc: P = %p   S = %p   A = %p   type=%d\n",
                            (void*)P, (void*)S, (void*)A, reloc_type ));
 #if defined(DEBUG)
-       checkProddableBlock ( oc, pP, sizeof(Elf_Word) );
+       checkProddableBlock ( &oc->proddables, pP, sizeof(Elf_Word) );
 #else
        (void) pP; /* suppress unused varialbe warning in non-debug build */
 #endif
@@ -1684,7 +1685,7 @@ do_Elf_Rela_relocations ( ObjectCode* oc, char* ehdrC,
 #if defined(DEBUG)
       IF_DEBUG(linker_verbose,debugBelch("Reloc: P = %p   S = %p   A = %p\n",
                                          (void*)P, (void*)S, (void*)A ));
-      checkProddableBlock(oc, (void*)P, sizeof(Elf_Word));
+      checkProddableBlock(&oc->proddables, (void*)P, sizeof(Elf_Word));
 #endif
 
 #if defined(powerpc_HOST_ARCH) || defined(x86_64_HOST_ARCH)
@@ -1989,13 +1990,11 @@ ocResolve_ELF ( ObjectCode* oc )
         }
     }
 
-#if defined(NEED_GOT)
     if(fillGot( oc ))
         return 0;
     /* silence warnings */
     (void) shnum;
     (void) shdr;
-#endif /* NEED_GOT */
 
 #if defined(aarch64_HOST_ARCH)
     /* use new relocation design */
