@@ -252,7 +252,7 @@ resolveImports(
             return 0;
         }
 
-        checkProddableBlock(oc,
+        checkProddableBlock(&oc->proddables,
                             ((void**)(oc->image + sect->offset)) + i,
                             sizeof(void *));
         ((void**)(oc->image + sect->offset))[i] = addr;
@@ -286,7 +286,7 @@ decodeAddend(ObjectCode * oc, Section * section, MachORelocationInfo * ri) {
     /* the instruction. It is 32bit wide */
     uint32_t * p = (uint32_t*)((uint8_t*)section->start + ri->r_address);
 
-    checkProddableBlock(oc, (void*)p, 1 << ri->r_length);
+    checkProddableBlock(&oc->proddables, (void*)p, 1 << ri->r_length);
 
     switch(ri->r_type) {
         case ARM64_RELOC_UNSIGNED: {
@@ -363,7 +363,7 @@ encodeAddend(ObjectCode * oc, Section * section,
              MachORelocationInfo * ri, int64_t addend) {
     uint32_t * p = (uint32_t*)((uint8_t*)section->start + ri->r_address);
 
-    checkProddableBlock(oc, (void*)p, 1 << ri->r_length);
+    checkProddableBlock(&oc->proddables, (void*)p, 1 << ri->r_length);
 
     switch (ri->r_type) {
         case ARM64_RELOC_UNSIGNED: {
@@ -738,7 +738,7 @@ relocateSection(ObjectCode* oc, int curSection)
             default:
                 barf("Unknown size.");
         }
-        checkProddableBlock(oc,thingPtr,relocLenBytes);
+        checkProddableBlock(&oc->proddables,thingPtr,relocLenBytes);
 
         /*
          * With SIGNED_N the relocation is not at the end of the
@@ -984,9 +984,9 @@ relocateSection(ObjectCode* oc, int curSection)
          */
         if (0 == reloc->r_extern) {
             if (reloc->r_pcrel) {
-                checkProddableBlock(oc, (void *)((char *)thing + baseValue), 1);
+                checkProddableBlock(&oc->proddables, (void *)((char *)thing + baseValue), 1);
             } else {
-                checkProddableBlock(oc, (void *)thing, 1);
+                checkProddableBlock(&oc->proddables, (void *)thing, 1);
             }
         }
 
@@ -1293,7 +1293,7 @@ ocGetNames_MachO(ObjectCode* oc)
                 secArray[sec_idx].info->stub_size = 0;
                 secArray[sec_idx].info->stubs = NULL;
 #endif
-                addProddableBlock(oc, start, section->size);
+                addProddableBlock(&oc->proddables, start, section->size);
             }
 
             curMem = (char*) secMem + section->size;
