@@ -4,6 +4,7 @@
 
 #include "ghcplatform.h"
 #include "PEi386.h"
+#include "linker/InitFini.h"
 #include <stdint.h>
 #include <stdio.h>
 
@@ -17,17 +18,9 @@ struct SectionFormatInfo {
     uint64_t virtualAddr;
  };
 
-// A linked-list of Sections; used to represent the set of initializer/finalizer
-// list sections.
-struct SectionList {
-    Section *section;
-    int priority;
-    struct SectionList *next;
-};
-
 struct ObjectCodeFormatInfo {
-    struct SectionList* init;  // Freed by ocRunInit_PEi386
-    struct SectionList* fini;  // Freed by ocRunFini_PEi386
+    struct InitFiniList* init; // Freed by ocRunInit_PEi386
+    struct InitFiniList* fini; // Freed by ocRunFini_PEi386
     Section* pdata;
     Section* xdata;
     COFF_HEADER_INFO* ch_info; // Freed by ocResolve_PEi386
