@@ -13,7 +13,7 @@
 
 #include "RtsUtils.h"
 #include "sm/OSMem.h"
-#include "sm/HeapAlloc.h"
+#include "rts/storage/HeapAlloc.h"
 
 #if defined(HAVE_UNISTD_H)
 #include <unistd.h>
@@ -455,21 +455,6 @@ StgWord64 getPhysicalMemorySize (void)
     return physMemSize;
 }
 
-void setExecutable (void *p, W_ len, bool exec)
-{
-    StgWord pageSize = getPageSize();
-
-    /* malloced memory isn't executable by default on OpenBSD */
-    StgWord mask             = ~(pageSize - 1);
-    StgWord startOfFirstPage = ((StgWord)p          ) & mask;
-    StgWord startOfLastPage  = ((StgWord)p + len - 1) & mask;
-    StgWord size             = startOfLastPage - startOfFirstPage + pageSize;
-    if (mprotect((void*)startOfFirstPage, (size_t)size,
-                 (exec ? PROT_EXEC : 0) | PROT_READ | PROT_WRITE) != 0) {
-        barf("setExecutable: failed to protect 0x%p\n", p);
-    }
-}
-
 #if defined(USE_LARGE_ADDRESS_SPACE)
 
 static void *
@@ -545,7 +530,7 @@ void *osReserveHeapMemory(void *startAddressPtr, W_ *len)
             (void*)startAddress, (void*)minimumAddress);
     }
 
-#if defined(HAVE_SYS_RESOURCE_H) && defined(HAVE_SYS_TIME_H)
+#if defined(HAVE_SYS_RESOURCE_H) && defined(HAVE_SYS_TIME_H) && defined(RLIMIT_AS)
     struct rlimit asLimit;
     /* rlim_t is signed on some platforms, including FreeBSD;
      * explicitly cast to avoid sign compare error */
