@@ -7,25 +7,20 @@
  * Documentation on the architecture of the Garbage Collector can be
  * found in the online commentary:
  * 
- *   http://ghc.haskell.org/trac/ghc/wiki/Commentary/Rts/Storage/GC
+ *   https://gitlab.haskell.org/ghc/ghc/wikis/commentary/rts/storage/gc
  *
  * ---------------------------------------------------------------------------*/
 
-#ifndef SM_MARKWEAK_H
-#define SM_MARKWEAK_H
+#pragma once
 
 #include "BeginPrivate.h"
 
-extern StgWeak *old_weak_ptr_list;
 extern StgTSO *resurrected_threads;
-extern StgTSO *exception_threads;
 
 void    collectFreshWeakPtrs   ( void );
 void    initWeakForGC          ( void );
-bool    traverseWeakPtrList    ( void );
+bool    traverseWeakPtrList    ( StgWeak **dead_weak_ptr_list, StgTSO **resurrected_threads );
 void    markWeakPtrList        ( void );
 void    scavengeLiveWeak       ( StgWeak * );
 
 #include "EndPrivate.h"
-
-#endif /* SM_MARKWEAK_H */
