@@ -213,7 +213,8 @@ extern StgIndStatic * debug_caf_list;
 extern StgIndStatic * revertible_caf_list;
 
 INLINE_HEADER void clear_blocks(bdescr *bd) {
-   memset(bd->start, 0, BLOCK_SIZE * bd->blocks);
+  memset(bd->start, RtsFlags.DebugFlags.zero_on_gc ? 0xaa : 0,
+         BLOCK_SIZE * bd->blocks);
 }
 
 #include "EndPrivate.h"
