@@ -61,6 +61,7 @@ import GHC.Types.Basic
 import GHC.Types.Name
 import GHC.Types.Name.Set
 import qualified GHC.Types.SrcLoc as SrcLoc
+import GHC.Unit.Module.Deps (dep_orphs)
 import GHC.Unit.State (PackageName (..), UnitState)
 import qualified GHC.Utils.Outputable as O
 import GHC.Utils.Panic (pprPanic)
@@ -70,7 +71,7 @@ import GHC.Builtin.Names
 import GHC.Builtin.Types
 import GHC.Builtin.Types.Prim
 import GHC.Types.SafeHaskell
-import Control.Arrow ((&&&))
+import Control.Arrow ((&&&), first)
 import GHC.Iface.Syntax
 
 createInterface1
@@ -157,6 +158,8 @@ createInterface1 flags unit_state mod_sum mod_iface ifaces inst_ifaces (instance
     let docsDecls = Map.fromList $ UniqMap.nonDetUniqMapToList mod_iface_docs.docs_decls
     traverse (processDocStringsParas dflags pkg_name) docsDecls
 
+  exportsSinceMap <- mkExportSinceMap dflags pkg_name mod_iface_docs
+
   (argMap :: Map Name (Map Int (MDoc Name))) <- do
       let docsArgs = Map.fromList $ UniqMap.nonDetUniqMapToList mod_iface_docs.docs_args
       (result :: Map Name (IntMap (MDoc Name))) <-
@@ -188,6 +191,7 @@ createInterface1 flags unit_state mod_sum mod_iface ifaces inst_ifaces (instance
     pkg_name
     mdl
     transitiveWarnings
+    exportsSinceMap
     docMap
     argMap
     fixities
@@ -229,6 +233,7 @@ createInterface1 flags unit_state mod_sum mod_iface ifaces inst_ifaces (instance
     , ifaceVisibleExports    = visible_names
     , ifaceFixMap            = fixities
     , ifaceInstances         = instances
+    , ifaceOrphanDeps        = dep_orphs $ mi_deps mod_iface
     , ifaceOrphanInstances   = [] -- Filled in attachInstances
     , ifaceRnOrphanInstances = [] -- Filled in renameInterfaceRn
     , ifaceHaddockCoverage   = coverage
@@ -246,6 +251,29 @@ createInterface1 flags unit_state mod_sum mod_iface ifaces inst_ifaces (instance
     --
     funAvail  = [ AvailTC fUNTyConName [fUNTyConName] ]
 
+-------------------------------------------------------------------------------
+-- Export @since annotations
+-------------------------------------------------------------------------------
+mkExportSinceMap
+  :: forall m. (MonadIO m)
+  => DynFlags
+  -> Maybe Package
+  -> Docs
+  -> IfM m (Map Name MetaSince)
+mkExportSinceMap dflags pkg_name docs = do
+    Map.unions <$> traverse processExportDoc (UniqMap.nonDetUniqMapToList (docs_exports docs))
+  where
+    processExportDoc :: (Name, HsDoc GhcRn) -> IfM m (Map Name MetaSince)
+    processExportDoc (nm, doc) = do
+      mdoc <- processDocStringsParas dflags pkg_name [doc]
+      case _doc mdoc of
+        DocEmpty -> return ()
+        _ -> warn "Export docstrings may only contain @since annotations"
+      case _metaSince (_meta mdoc) of
+        Nothing -> return mempty
+        Just since -> return $ Map.singleton nm since
+
+
 -------------------------------------------------------------------------------
 -- Warnings
 -------------------------------------------------------------------------------
@@ -294,6 +322,7 @@ parseWarning dflags w = case w of
 
     format x bs = DocWarning . DocParagraph . DocAppend (DocString x)
                   <$> foldrM (\doc rest -> docAppend <$> processDocString dflags doc <*> pure rest) DocEmpty bs
+
 -------------------------------------------------------------------------------
 -- Doc options
 --
@@ -352,6 +381,7 @@ mkExportItems
   -> Maybe Package      -- this package
   -> Module             -- this module
   -> WarningMap
+  -> Map Name MetaSince
   -> DocMap Name
   -> ArgMap Name
   -> FixMap
@@ -362,7 +392,8 @@ mkExportItems
   -> OccEnv Name
   -> IfM m [ExportItem GhcRn]
 mkExportItems
-  prr modMap pkgName thisMod warnings docMap argMap fixMap namedChunks dsItems
+  prr modMap pkgName thisMod warnings exportSinceMap
+  docMap argMap fixMap namedChunks dsItems
   instIfaceMap dflags defMeths =
     concat <$> traverse lookupExport dsItems
   where
@@ -395,7 +426,7 @@ mkExportItems
 
     availExport :: MonadIO m => AvailInfo -> IfM m [ExportItem GhcRn]
     availExport avail =
-      availExportItem prr modMap thisMod warnings
+      availExportItem prr modMap thisMod warnings exportSinceMap
         docMap argMap fixMap instIfaceMap dflags avail defMeths
 
 unrestrictedModExports
@@ -451,6 +482,7 @@ availExportItem
   -> IfaceMap
   -> Module             -- this module
   -> WarningMap
+  -> Map Name MetaSince  -- ^ export \@since declarations
   -> Map Name (MDoc Name)        -- docs (keyed by 'Name's)
   -> ArgMap Name        -- docs for arguments (keyed by 'Name's)
   -> FixMap
@@ -460,7 +492,7 @@ availExportItem
   -> OccEnv Name       -- Default methods
   -> IfM m [ExportItem GhcRn]
 availExportItem
-    prr modMap thisMod warnings docMap argMap fixMap instIfaceMap dflags
+    prr modMap thisMod warnings exportSinceMap docMap argMap fixMap instIfaceMap dflags
     availInfo defMeths
   =
     declWith availInfo
@@ -479,7 +511,9 @@ availExportItem
             then pure (lookupDocs avail warnings docMap argMap defMeths)
             else case Map.lookup tmod modMap of
               Just iface ->
-                pure (lookupDocs avail warnings (ifaceDocMap iface) (ifaceArgMap iface) (mkOccEnv (ifaceDefMeths iface)))
+                pure
+                  $ first (applyExportSince exportSinceMap t)
+                  $ lookupDocs avail warnings (ifaceDocMap iface) (ifaceArgMap iface) (mkOccEnv (ifaceDefMeths iface))
               Nothing ->
                 -- We try to get the subs and docs
                 -- from the installed .haddock file for that package.
@@ -493,7 +527,9 @@ availExportItem
                     let subs_ = availNoDocs avail
                     pure (noDocForDecl, subs_)
                   Just instIface ->
-                    pure (lookupDocs avail warnings (instDocMap instIface) (instArgMap instIface) (mkOccEnv (instDefMeths instIface)))
+                    pure
+                      $ first (applyExportSince exportSinceMap t)
+                      $ lookupDocs avail warnings (instDocMap instIface) (instArgMap instIface) (mkOccEnv (instDefMeths instIface))
 
     -- Tries 'extractDecl' first then falls back to 'hiDecl' if that fails
     availDecl :: Name -> LHsDecl GhcRn -> IfM m (LHsDecl GhcRn)
@@ -587,6 +623,22 @@ availNoDocs :: AvailInfo -> [(Name, DocForDecl Name)]
 availNoDocs avail =
   zip (availSubordinates avail) (repeat noDocForDecl)
 
+-- | Override 'MetaSince' of a declaration with that of its export if appropriate.
+applyExportSince
+  :: Map Name MetaSince
+  -> Name
+  -> DocForDecl Name
+  -> DocForDecl Name
+applyExportSince exportSinceMap nm (dd, argDoc)
+  | Just since <- Map.lookup nm exportSinceMap =
+    let dd' = dd { documentationDoc = setMDocSince (documentationDoc dd) }
+        setMDocSince :: Maybe (MDoc name) -> Maybe (MDoc name)
+        setMDocSince (Just (MetaDoc meta doc)) = Just $ MetaDoc (meta {_metaSince = Just since}) doc
+        setMDocSince Nothing                   = Just $ MetaDoc (Meta {_metaSince = Just since}) DocEmpty
+     in (dd', argDoc)
+applyExportSince _ _ dd = dd
+
+
 hiDecl
   :: MonadIO m
   => DynFlags
@@ -616,6 +668,7 @@ lookupDocs
   -> ArgMap Name
   -> OccEnv Name
   -> (DocForDecl Name, [(Name, DocForDecl Name)])
+     -- ^ documentation for declaration and its subordinates
 lookupDocs avail warningMap docMap argMap def_meths_env =
   let
     n = availName avail
@@ -769,8 +822,8 @@ extractPatternSyn nm t tvs cons =
               RecCon (L _ fields) -> cd_fld_type . unLoc <$> fields
               InfixCon arg1 arg2 -> map hsScaledThing [arg1, arg2]
             ConDeclGADT { con_g_args = con_args' } -> case con_args' of
-              PrefixConGADT args' -> map hsScaledThing args'
-              RecConGADT (L _ fields) _ -> cd_fld_type . unLoc <$> fields
+              PrefixConGADT _ args' -> map hsScaledThing args'
+              RecConGADT _ (L _ fields) -> cd_fld_type . unLoc <$> fields
         typ = longArrow args (data_ty con)
         typ' =
           case con of
@@ -780,14 +833,14 @@ extractPatternSyn nm t tvs cons =
     in PatSynSig noAnn [noLocA nm] (mkEmptySigType typ'')
 
   longArrow :: [LHsType GhcRn] -> LHsType GhcRn -> LHsType GhcRn
-  longArrow inputs output = foldr (\x y -> noLocA (HsFunTy noAnn (HsUnrestrictedArrow noHsUniTok) x y)) output inputs
+  longArrow inputs output = foldr (\x y -> noLocA (HsFunTy noExtField (HsUnrestrictedArrow noExtField) x y)) output inputs
 
   data_ty con
     | ConDeclGADT{} <- con = con_res_ty con
     | otherwise = foldl' (\x y -> noLocA (mkAppTyArg x y)) (noLocA (HsTyVar noAnn NotPromoted (noLocA t))) tvs
                     where mkAppTyArg :: LHsType GhcRn -> LHsTypeArg GhcRn -> HsType GhcRn
-                          mkAppTyArg f (HsValArg ty) = HsAppTy noExtField f ty
-                          mkAppTyArg f (HsTypeArg at ki) = HsAppKindTy noExtField f at ki
+                          mkAppTyArg f (HsValArg _ ty) = HsAppTy noExtField f ty
+                          mkAppTyArg f (HsTypeArg _ ki) = HsAppKindTy noExtField f ki
                           mkAppTyArg f (HsArgPar _) = HsParTy noAnn f
 
 extractRecSel :: Name -> Name -> [LHsTypeArg GhcRn] -> [LConDecl GhcRn]
@@ -797,7 +850,7 @@ extractRecSel _ _ _ [] = Left "extractRecSel: selector not found"
 extractRecSel nm t tvs (L _ con : rest) =
   case getRecConArgs_maybe con of
     Just (L _ fields) | ((l,L _ (ConDeclField _ _nn ty _)) : _) <- matching_fields fields ->
-      pure (L (noAnnSrcSpan l) (TypeSig noAnn [noLocA nm] (mkEmptyWildCardBndrs $ mkEmptySigType (noLocA (HsFunTy noAnn (HsUnrestrictedArrow noHsUniTok) data_ty (getBangType ty))))))
+      pure (L (noAnnSrcSpan l) (TypeSig noAnn [noLocA nm] (mkEmptyWildCardBndrs $ mkEmptySigType (noLocA (HsFunTy noExtField (HsUnrestrictedArrow noExtField) data_ty (getBangType ty))))))
     _ -> extractRecSel nm t tvs rest
  where
   matching_fields :: [LConDeclField GhcRn] -> [(SrcSpan, LConDeclField GhcRn)]
@@ -808,8 +861,8 @@ extractRecSel nm t tvs (L _ con : rest) =
     | ConDeclGADT{} <- con = con_res_ty con
     | otherwise = foldl' (\x y -> noLocA (mkAppTyArg x y)) (noLocA (HsTyVar noAnn NotPromoted (noLocA t))) tvs
                    where mkAppTyArg :: LHsType GhcRn -> LHsTypeArg GhcRn -> HsType GhcRn
-                         mkAppTyArg f (HsValArg ty) = HsAppTy noExtField f ty
-                         mkAppTyArg f (HsTypeArg at ki) = HsAppKindTy noExtField f at ki
+                         mkAppTyArg f (HsValArg _ ty) = HsAppTy noExtField f ty
+                         mkAppTyArg f (HsTypeArg _ ki) = HsAppKindTy noExtField f ki
                          mkAppTyArg f (HsArgPar _) = HsParTy noAnn f
 
 -- | Keep export items with docs.
