@@ -136,10 +136,13 @@ rename dflags renamer = rn
       DocIdentifier i -> do
         let NsRdrName ns x = unwrap i
             occ = rdrNameOcc x
-        let valueNsChoices | isDataOcc occ = isDataConNameSpace
-                           | otherwise     = isTermVarOrFieldNameSpace
-            typeNsChoices  | isDataOcc occ = isTcClsNameSpace
-                           | otherwise     = isTvNameSpace
+        let valueNsChoices
+              | isDataOcc occ = isDataConNameSpace
+              | otherwise = isTermVarOrFieldNameSpace
+            typeNsChoices
+              | isDataOcc occ = isTcClsNameSpace
+              | isSymOcc occ = isTcClsNameSpace
+              | otherwise = isTvNameSpace
         -- Generate the choices for the possible kind of thing this
         -- is. We narrow down the possibilities with the namespace (if
         -- there is one).
