@@ -75,7 +75,9 @@ type SubMap        = Map Name [Name]
 type DeclMap       = Map Name DeclMapEntry
 type InstMap       = Map RealSrcSpan Name
 type FixMap        = Map Name Fixity
-type DocPaths      = (FilePath, Maybe FilePath) -- paths to HTML and sources
+data DocPaths      = DocPaths { docPathsHtml :: FilePath  -- ^ path to HTML Haddocks
+                              , docPathsSources :: Maybe FilePath -- ^ path to hyperlinked sources
+                              }
 type WarningMap    = Map Name (Doc Name)
 
 
@@ -134,6 +136,10 @@ data Interface = Interface
     -- | Instances exported by the module.
   , ifaceInstances       :: [ClsInst]
 
+  -- | The list of modules to check for orphan instances if this module is
+  -- imported.
+  , ifaceOrphanDeps :: [Module]
+
     -- | Orphan instances
   , ifaceOrphanInstances   :: [DocInstance GhcRn]
   , ifaceRnOrphanInstances :: [DocInstance DocNameI]
@@ -594,8 +600,11 @@ type MDoc id = MetaDoc (Wrap (ModuleName, OccName)) (Wrap id)
 
 type DocMarkup id a = DocMarkupH (Wrap (ModuleName, OccName)) id a
 
+instance NFData MetaSince where
+  rnf (MetaSince v p) = v `deepseq` p `deepseq` ()
+
 instance NFData Meta where
-  rnf (Meta v p) = v `deepseq` p `deepseq` ()
+  rnf (Meta since) = since `deepseq` ()
 
 instance NFData id => NFData (MDoc id) where
   rnf (MetaDoc m d) = m `deepseq` d `deepseq` ()
@@ -816,8 +825,8 @@ type instance Anno (HsTyVarBndr flag DocNameI)       = SrcSpanAnnA
 type instance Anno [LocatedA (HsType DocNameI)]      = SrcSpanAnnC
 type instance Anno (HsType DocNameI)                 = SrcSpanAnnA
 type instance Anno (DataFamInstDecl DocNameI)        = SrcSpanAnnA
-type instance Anno (DerivStrategy DocNameI)          = SrcAnn NoEpAnns
-type instance Anno (FieldOcc DocNameI)               = SrcAnn NoEpAnns
+type instance Anno (DerivStrategy DocNameI)          = EpAnn NoEpAnns
+type instance Anno (FieldOcc DocNameI)               = SrcSpanAnnA
 type instance Anno (ConDeclField DocNameI)           = SrcSpan
 type instance Anno (Located (ConDeclField DocNameI)) = SrcSpan
 type instance Anno [Located (ConDeclField DocNameI)] = SrcSpan
@@ -827,20 +836,34 @@ type instance Anno (TyFamInstDecl DocNameI)          = SrcSpanAnnA
 type instance Anno [LocatedA (TyFamInstDecl DocNameI)] = SrcSpanAnnL
 type instance Anno (FamilyDecl DocNameI)               = SrcSpan
 type instance Anno (Sig DocNameI)                      = SrcSpan
-type instance Anno (InjectivityAnn DocNameI)           = SrcAnn NoEpAnns
+type instance Anno (InjectivityAnn DocNameI)           = EpAnn NoEpAnns
 type instance Anno (HsDecl DocNameI)                   = SrcSpanAnnA
-type instance Anno (FamilyResultSig DocNameI)          = SrcAnn NoEpAnns
+type instance Anno (FamilyResultSig DocNameI)          = EpAnn NoEpAnns
 type instance Anno (HsOuterTyVarBndrs Specificity DocNameI) = SrcSpanAnnA
 type instance Anno (HsSigType DocNameI)                     = SrcSpanAnnA
 
 type XRecCond a
-  = ( XParTy a           ~ EpAnn AnnParen
+  = ( XParTy a ~ AnnParen
     , NoGhcTc a ~ a
     , MapXRec a
     , UnXRec a
     , WrapXRec a (HsType a)
     )
 
+type instance XValArg  DocNameI = NoExtField
+type instance XTypeArg DocNameI = NoExtField
+type instance XArgPar  DocNameI = NoExtField
+type instance XXArg    DocNameI = DataConCantHappen
+
+type instance XBndrRequired    DocNameI = NoExtField
+type instance XBndrInvisible   DocNameI = NoExtField
+type instance XXBndrVis        DocNameI = DataConCantHappen
+
+type instance XUnrestrictedArrow DocNameI = NoExtField
+type instance XLinearArrow       DocNameI = NoExtField
+type instance XExplicitMult      DocNameI = NoExtField
+type instance XXArrow            DocNameI = DataConCantHappen
+
 type instance XForAllTy        DocNameI = EpAnn [AddEpAnn]
 type instance XQualTy          DocNameI = EpAnn [AddEpAnn]
 type instance XTyVar           DocNameI = EpAnn [AddEpAnn]
@@ -852,7 +875,7 @@ type instance XListTy          DocNameI = EpAnn AnnParen
 type instance XTupleTy         DocNameI = EpAnn AnnParen
 type instance XSumTy           DocNameI = EpAnn AnnParen
 type instance XOpTy            DocNameI = EpAnn [AddEpAnn]
-type instance XParTy           DocNameI = EpAnn AnnParen
+type instance XParTy           DocNameI = AnnParen
 type instance XIParamTy        DocNameI = EpAnn [AddEpAnn]
 type instance XKindSig         DocNameI = EpAnn [AddEpAnn]
 type instance XSpliceTy        DocNameI = DataConCantHappen
@@ -901,6 +924,10 @@ type instance XConDeclGADT    DocNameI = NoExtField
 type instance XConDeclH98     DocNameI = NoExtField
 type instance XXConDecl       DocNameI = DataConCantHappen
 
+type instance XPrefixConGADT       DocNameI = NoExtField
+type instance XRecConGADT          DocNameI = NoExtField
+type instance XXConDeclGADTDetails DocNameI = DataConCantHappen
+
 type instance XDerivD     DocNameI = NoExtField
 type instance XInstD      DocNameI = NoExtField
 type instance XForD       DocNameI = NoExtField
@@ -976,11 +1003,7 @@ instance NFData Fixity where
   rnf (Fixity sourceText n dir) =
     sourceText `deepseq` n `deepseq` dir `deepseq` ()
 
-instance NFData ann => NFData (SrcSpanAnn' ann) where
-  rnf (SrcSpanAnn a ss) = a `deepseq` ss `deepseq` ()
-
 instance NFData (EpAnn NameAnn) where
-  rnf EpAnnNotUsed = ()
   rnf (EpAnn en ann cs) = en `deepseq` ann `deepseq` cs `deepseq` ()
 
 instance NFData NameAnn where
@@ -1026,9 +1049,11 @@ instance NFData NameAnn where
   rnf (NameAnnTrailing a) = rnf a
 
 instance NFData TrailingAnn where
-  rnf (AddSemiAnn epaL)  = rnf epaL
-  rnf (AddCommaAnn epaL) = rnf epaL
-  rnf (AddVbarAnn epaL)  = rnf epaL
+  rnf (AddSemiAnn epaL)    = rnf epaL
+  rnf (AddCommaAnn epaL)   = rnf epaL
+  rnf (AddVbarAnn epaL)    = rnf epaL
+  rnf (AddDarrowAnn epaL)  = rnf epaL
+  rnf (AddDarrowUAnn epaL) = rnf epaL
 
 instance NFData NameAdornment where
   rnf NameParens     = ()
@@ -1036,8 +1061,11 @@ instance NFData NameAdornment where
   rnf NameBackquotes = ()
   rnf NameSquare     = ()
 
-instance NFData EpaLocation where
-  rnf (EpaSpan ss bs)  = ss `seq` bs `deepseq` ()
+instance NFData NoComments where
+  rnf NoComments = ()
+
+instance (NFData a) => NFData (EpaLocation' a) where
+  rnf (EpaSpan ss)  = rnf ss
   rnf (EpaDelta dp lc) = dp `seq` lc `deepseq` ()
 
 instance NFData EpAnnComments where
@@ -1052,7 +1080,6 @@ instance NFData EpaCommentTok where
   rnf (EpaDocOptions s)   = rnf s
   rnf (EpaLineComment s)  = rnf s
   rnf (EpaBlockComment s) = rnf s
-  rnf EpaEofComment       = ()
 
 
 instance NFData a => NFData (Strict.Maybe a) where
@@ -1065,13 +1092,6 @@ instance NFData BufSpan where
 instance NFData BufPos where
   rnf (BufPos n) = rnf n
 
-instance NFData Anchor where
-  rnf (Anchor ss op) = ss `seq` op `deepseq` ()
-
-instance NFData AnchorOperation where
-  rnf UnchangedAnchor  = ()
-  rnf (MovedAnchor dp) = rnf dp
-
 instance NFData DeltaPos where
   rnf (SameLine n)        = rnf n
   rnf (DifferentLine n m) = n `deepseq` m `deepseq` ()
