@@ -28,8 +28,7 @@ instance IsString a => IsString (Maybe a) where
 emptyMeta :: Meta
 emptyMeta =
   Meta {
-    _version = Nothing
-  , _package = Nothing
+    _metaSince = Nothing
   }
 
 parseParas :: String -> MetaDoc () String
@@ -283,6 +282,13 @@ spec = do
       it "supports title for deprecated picture syntax" $ do
         "<<b a z>>" `shouldParseTo` image "b" "a z"
 
+    context "when parsing inline math" $ do
+      it "accepts inline math immediately after punctuation" $ do
+        "(\\(1 + 2 = 3\\) is an example of addition)"
+          `shouldParseTo` "("
+          <> DocMathInline "1 + 2 = 3"
+          <> " is an example of addition)"
+
     context "when parsing display math" $ do
 
       it "accepts markdown syntax for display math containing newlines" $ do
@@ -540,6 +546,12 @@ spec = do
           MetaDoc { _meta = emptyMeta { _version = Nothing }
                   , _doc = DocParagraph "@since 0.5.0 foo" }
 
+      it "parses package name" $ do
+        parseParas "@since foo-bar-0.5.0" `shouldBe`
+          MetaDoc { _meta = emptyMeta { _metaSince = Just $ MetaSince { sincePackage = Just "foo-bar"
+                                                                      , sinceVersion = [0,5,0] } }
+                  , _doc = DocEmpty }
+
 
       context "when given multiple times" $ do
         it "gives last occurrence precedence" $ do
@@ -547,8 +559,11 @@ spec = do
               "@since 0.5.0"
             , "@since 0.6.0"
             , "@since 0.7.0"
-            ] `shouldBe` MetaDoc { _meta = emptyMeta { _version = Just [0,7,0] }
-                                 , _doc = DocEmpty }
+                                 ]
+          `shouldBe`
+          MetaDoc { _meta = emptyMeta { _metaSince = Just $ MetaSince { sincePackage = Nothing
+                                                                      , sinceVersion = [0,7,0] } }
+                  , _doc = DocEmpty }
 
 
     context "when parsing text paragraphs" $ do
@@ -793,6 +808,29 @@ spec = do
       it "accepts unicode in examples" $ do
         ">>> 灼眼\nシャナ" `shouldParseTo` DocExamples [Example "灼眼" ["シャナ"]]
 
+      it "preserves indentation in consecutive example lines" $ do
+        unlines
+          [ ">>> line 1"
+          , ">>>   line 2"
+          , ">>> line 3"
+          ]
+          `shouldParseTo` DocExamples
+            [ Example "line 1" []
+            , Example "  line 2" []
+            , Example "line 3" []
+            ]
+
+      it "resets indentation after results" $ do
+        unlines
+          [ ">>> line 1"
+          , "result"
+          , ">>>   line 2"
+          ]
+          `shouldParseTo` DocExamples
+            [ Example "line 1" ["result"]
+            , Example "line 2" []
+            ]
+
       context "when prompt is prefixed by whitespace" $ do
         it "strips the exact same amount of whitespace from result lines" $ do
           unlines [
