@@ -3,16 +3,17 @@
 {-# LANGUAGE UndecidableInstances #-}
 module Bug806 where
 
+import Data.Kind (Type)
 import Data.Proxy
 
 -- | 'F1' docs
-type family F1 a b :: * -> *
+type family F1 a b :: Type -> Type
 -- | 'F2' docs
-type family F2 a b :: * -> * where
+type family F2 a b :: Type -> Type where
   F2 Int b = Maybe
   F2 a   b = []
 -- | 'D' docs
-data family D a :: * -> *
+data family D a :: Type -> Type
 
 v :: Int
 v = 42
