@@ -2,8 +2,10 @@
 {-# LANGUAGE KindSignatures, FlexibleInstances, GADTs, DataKinds #-}
 module Bug923 where
 
+import Data.Kind (Type)
+
 -- | A promoted tuple type
-data T :: (* -> (*,*)) -> * where
+data T :: (Type -> (Type,Type)) -> Type where
   T :: a -> T ('(,) a)
 
 -- | A promoted tuple type in an instance
