@@ -2,9 +2,11 @@
 {-# LANGUAGE TypeFamilies #-}
 module DeprecatedTypeFamily where
 
+import Data.Kind (Type)
+
 -- | some documentation
-data family SomeTypeFamily k :: * -> *
+data family SomeTypeFamily k :: Type -> Type
 {-# DEPRECATED SomeTypeFamily "SomeTypeFamily" #-}
 
-data family SomeOtherTypeFamily k :: * -> *
+data family SomeOtherTypeFamily k :: Type -> Type
 {-# DEPRECATED SomeOtherTypeFamily "SomeOtherTypeFamily" #-}
